/*=============================================================================
 * COPYRIGHT :  2003 Rohde & Schwarz
 *
 * MODULE    : RSCMUNB.C
 *
 * PURPOSE   : this program is a CMU driver for the WCDMA-NB Test (3GPP/
 *             FDD). It contains C VISA functions to send remote controls to
 *             the CMU by IEEE or RS232, and to return results.
 *
 * VERSION   : 4.40.0
 *
 * COMPILER  : LabWindows/CVI 8.5
 *
 * AUTHOR(S) : Anton Strohmeier (Rohde & Schwarz )
 *             e-Mail    : anton.strohmeier@rohde-schwarz.com
 *
 *
 * HISTORY   :
 *  see rscmunb_vxi.chm
 *===========================================================================*/

/*****************************************************************************/
/*  INCLUDE FILES                                                            */
/*****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <ctype.h>
#include <stdarg.h>
#include <locale.h>
#include <visa.h>
#include <rscmu200.h>
#include "rscmunb.h"

/*****************************************************************************/
/*  IMPORT                                                                   */
/*****************************************************************************/

// None

/*****************************************************************************/
/*  EXPORT                                                                   */
/*****************************************************************************/

// None

/*****************************************************************************/
/*  LOCAL VARIABLES DECLARATION                                              */
/*****************************************************************************/

static ViString RSCMUNB_function_group_name[]           = {"\"WCDMA1900NBFDD_NSig\"","\"WCDMA1900NBFDD_Sig\"",VI_NULL};
static ViString RSCMUNB_mode_On_Off[]                   = {"OFF","ON",VI_NULL};
static ViString RSCMUNB_Unit_Frequency[]                = {"HZ","KHZ","MHZ","GHZ",VI_NULL};
static ViString RSCMUNB_ScramblingCode_Length[]         = {"LONG","SHOR","OFF",VI_NULL};
static ViString RSCMUNB_DedicatedChannel_Mode[]         = {"GPPR","PHYS","HSDP",VI_NULL};
static ViString RSCMUNB_PowerControl_Mode[]             = {"ALG1","ALG2",VI_NULL};
static ViString RSCMUNB_ChannelDataSource_DTCH[]        = {"ALL0","ALL1","ALT","PR9","PR11","PR15","PR16",VI_NULL};
static ViString RSCMUNB_DownlinkPowerControl_Mode[]     = {"MOD0","MOD1",VI_NULL};
static ViString RSCMUNB_PCFieldSettings_ReadOutMode[]   = {"AL01","AL10","ALL0","ALL1","CPAT","SPA0","SPA1","SPAL",VI_NULL};
static ViString RSCMUNB_Connector_Input[]               = {"RF1","RF2","RF4",VI_NULL};
static ViString RSCMUNB_Connector_Output[]              = {"RF1","RF2","RF3",VI_NULL};
static ViString RSCMUNB_ExpectedPower_Mode[]            = {"MAN","AUTO",VI_NULL};
static ViString RSCMUNB_ExpectedPower_Attenuation[]     = {"NORM","LNO","LDIS",VI_NULL};
static ViString RSCMUNB_Trigger_Source[]                = {"FRUN","EXT","FRAM","SFR",VI_NULL};
static ViString RSCMUNB_WiredSync_Polarity[]            = {"RIS","FALL", VI_NULL};
static ViString RSCMUNB_MeasStatus[]                    = {"OFF","RUN","STOP","ERR","STEP","RDY","NONE",VI_NULL};
static ViString RSCMUNB_Measurement_Result_Mode[]       = {"ARR","SCAL",VI_NULL};
static ViString RSCMUNB_Repetition[]                    = {"CONT","SING",VI_NULL};
static ViString RSCMUNB_StopCondition[]                 = {"NONE","SON",VI_NULL};
static ViString RSCMUNB_StepMode[]                      = {"NONE","STEP",VI_NULL};
static ViString RSCMUNB_EventReport[]                   = {"OFF","SRQ","SOPC","SRSQ",VI_NULL};
static ViString RSCMUNB_CDP_Mode[]                      = {"WSCH","NSCH",VI_NULL};
static ViString RSCMUNB_CDP_Unit[]                      = {"ABS","RCP","RAVG",VI_NULL};
static ViString RSCMUNB_Cmd[]                           = {"READ","FETCh","SAMPle",VI_NULL};
static ViString RSCMUNB_StatisticsMode[]                = {"CURRent","AVERage","MMAX","CURRent","AVERage","MAXimum","MINimum",VI_NULL};
static ViString RSCMUNB_CDP_ScalarResults[]             = {"IQOOffset","IQIMbalance","CFERror","EVMagnitude","PCDerror", VI_NULL};
static ViString RSCMUNB_ACLR_Unit[]                     = {"RELative","ABSolute",VI_NULL};
static ViString RSCMUNB_ResulutionBandwidth[]           = {"WRRC","MHZ5","MHZ7",VI_NULL};
static ViString RSCMUNB_AntennaDiversity[]              = {"OFF","A1","A2",VI_NULL};
static ViString RSCMUNB_EmissionMask[]                  = {"AUTO","CL1","CL2","CL3","CL4","USER",VI_NULL};
static ViString RSCMUNB_BerBlerVerifMode[]              = {"OFF","BER","BLER",VI_NULL};
static ViString RSCMUNB_TestModel[]                     = {"M116","M132","M164","M2","M316","M332","M4","M5308","M5144","M562","USR1","USR2","ACD",VI_NULL};
static ViString RSCMUNB_UTM_Chan[]                      = {"DEF","PICH","CPIC","PCCP","SCCP","PDSC","HSSC","HS16","HSQP","DPCH",VI_NULL};
static ViString RSCMUNB_UTM_Srate[]                     = {"DEF","S75","S15","S30","S60","S120","S240","S480","S960",VI_NULL};
static ViString RSCMUNB_UTM_Pilot[]                     = {"DEF","PB0","PB2","PB4","PB8","PB16","INV",VI_NULL};
static ViString RSCMUNB_Sig_Act[]                       = {"SSTP","SRUN",VI_NULL};
static ViString RSCMUNB_Sig_Stat[]                      = {"UNS","SYN","SIP",VI_NULL};
static ViString RSCMUNB_Sig_Sto_CPICH[]                 = {"CPICH_UNS","CPICH_SYN","CPICH_LOST","NAN",VI_NULL};
static ViString RSCMUNB_Sig_Sto_BCH[]                   = {"BCH_UNS","BCH_SYN","BCH_LOST",VI_NULL};
static ViString RSCMUNB_Unit_NBS_Freq[]                 = {"HZ","KHZ","MHZ","GHZ","CH",VI_NULL};
static ViString RSCMUNB_Ref_Chan_Type[]                 = {"12.2","64","144","384","2048",VI_NULL};
static ViString RSSMUNB_SEQ_PR[]                        = {"PR9","PR11","PR15","PR16",VI_NULL};
static ViString RSCMUNB_ChannelDataSource[]             = {"ALL0","ALL1","ALT","PR9","PR11","PR15","PR16",VI_NULL};
static ViString RSCMUNB_Trig_Slope[]                    = {"NEG", "POS", VI_NULL};
static ViString RSCMUNB_SIB_Type[]                      = {"MIB","SIB1","SIB2","SIB3","SIB4","SIB5","SIB6","SIB7","SIB8","SIB9","SIB10","SIB11","SIB12","SIB13","SIB14","SIB15","SIB16","SIB17","SIB18","SIB13_1","SIB13_2","SIB13_3","SIB13_4","SIB15_1","SIB15_2","SIB15_3","SIB15_4","SIB15_5","SB1","SB2",VI_NULL};
static ViString RSCMUNB_DLBER_SEQ_PR[]                  = {"PR9","PR11","PR15","PR16","LOOP",VI_NULL};
static ViString RSCMUNB_STORAGE[]                       = {"SAVE","REC",VI_NULL};
static ViString RSCMUNB_STOR_DEV[]                      = {"INTERNAL","EXTERNAL",VI_NULL};
static ViString RSCMUNB_FastChanDet[]                   = {"USR1","USR2",VI_NULL};
static ViString RSCMUNB_LimitParameter[]                = {"EVMP","EVMR","MEP","MERM","PEP","PERM","IQO","IQIM","CFER","WFQ",VI_NULL};
static ViString RSCMUNB_SubArr[]                        = {"ALL","ARIT","MIN","MAX",VI_NULL};
static ViString RSCMUNB_statType[]                      = {"CMMAX", "AVER", VI_NULL};
static ViString RSCMUNB_trigOutpPinSig[]                = {"NONE","SLOT","FRAM","SFR",VI_NULL};
static ViString RSCMUNB_bchmEmpBuff[]                   = {"EMPT","FULL",VI_NULL};
static ViString RSCMUNB_rachPreSfnm[]                   = {"SFNN","IMM",VI_NULL};
static ViString RSCMUNB_HsdpaMeasMode[]                 = {"IMM","HSFN","UEID",VI_NULL};
static ViString RSCMUNB_HsscchDisplayMode[]             = {"HSFN","UEID",VI_NULL};
static ViString RSCMUNB_HsdpaType[]                     = {"ANC","CQI","ANAC",VI_NULL};       
static ViString RSCMUNB_ACKTransmitMode[]               = {"FRUN","UITR",VI_NULL};
static ViString RSCMUNB_HsdpaTransmitMode[]             = {"OFF","ACK","NACK","CQI",VI_NULL};
static ViString RSCMUNB_activationHSFNMode[]            = {"IMM","HNUM",VI_NULL};
static ViString RSCMUNB_bufferWriting[]                 = {"EMPT","FULL",VI_NULL};   
static ViString RSCMUNB_recDestination[]                = {"NONE","FILE","COM",VI_NULL};


static ViInt32    RSCMUNB_nbFwVersion      = 0;
static ViInt32    RSCMUNB_baseFwVersion    = 0;
static ViInt32    RSCMUNB_nbDrvVersion     = 0;
static ViInt32    RSCMUNB_baseDrvVersion   = 0;

static ViBoolean  K75Present               = VI_FALSE;
static ViBoolean  K76Present               = VI_FALSE;

// Structure "RSCMU_WCDMA_Sig_Globals" is initialized for each Session in an 'init' function
struct  RSCMUNB_Sig_Globals
{
  struct RSCMU_SESSION_Globals RSCMU_tGlobal;
};

// Structure "RSCMU_WCDMA_NSig_Globals" is initialized for each Session in an 'init' function
struct  RSCMUNB_NSig_Globals
{
  struct RSCMU_SESSION_Globals RSCMU_tGlobal;
};

/*****************************************************************************/
/*  LOCAL TYPES DECLARATION                                                  */
/*****************************************************************************/

// None

/*****************************************************************************/
/*  LOCAL CONSTANTS DEFINITION                                               */
/*****************************************************************************/
// #define _DEBUGOUTPUT_

/*****************************************************************************/
/*  LOCAL MACROS DEFINITION                                                  */
/*****************************************************************************/

// None

/*****************************************************************************/
/*  LOCAL FUNCTIONS DECLARATION                                               */
/*****************************************************************************/

// ViStatus dec2bin(ViUInt32 dec, ViChar* bin);
ViStatus _VI_FUNC RSCMU_WCDMA_DefaultSettings    (ViSession, ViChar*, ViInt32, ViStatus, ViChar[]);
ViStatus _VI_FUNC RSCMU_WCDMA_ConfContRmod       (ViSession, ViChar*, ViInt32, ViStatus, ViChar[]);
ViStatus _VI_FUNC RSCMU_WCDMA_ConfContStat       (ViSession, ViChar*, ViInt32, ViStatus, ViChar[]);
ViStatus _VI_FUNC RSCMU_WCDMA_ConfContRep        (ViSession, ViChar*, ViInt32, ViInt32, ViInt32, ViInt32, ViStatus, ViChar[]);
ViStatus _VI_FUNC RSCMU_WCDMA_ConfContRep2       (ViSession, ViChar*, ViInt32, ViInt32, ViInt32, ViStatus, ViChar[]);
ViStatus _VI_FUNC RSCMU_WCDMA_ConfContMlen       (ViSession, ViChar*, ViInt32, ViChar[]);
ViStatus _VI_FUNC RSCMU_WCDMA_ConfSub            (ViSession, ViChar*, ViInt32, ViInt32, ViInt32, ViChar[]);
ViStatus _VI_FUNC RSCMU_WCDMA_MeasCont           (ViSession, ViChar*, ViChar*, ViChar[]);
ViStatus _VI_FUNC RSCMU_WCDMA_FetcStat           (ViSession, ViChar*, ViInt32*, ViInt32*, ViInt32*, ViChar[]);
ViStatus _VI_FUNC RSCMU_WCDMA_ConfErep           (ViSession, ViChar*, ViInt32, ViStatus, ViChar[]);
ViStatus _VI_FUNC RSCMU_WCDMA_ConfOempQpskLimUpp (ViSession, ViInt32, ViReal64, ViReal64, ViReal64, ViReal64, ViReal64, ViReal64, ViReal64, ViReal64, ViInt32, ViReal64, ViChar[]);
ViStatus _VI_FUNC RSCMU_WCDMA_ConfOempWcdmLimUpp (ViSession, ViInt32, ViInt32, ViReal64, ViReal64, ViReal64, ViReal64, ViReal64, ViReal64, ViReal64, ViReal64, ViInt32, ViReal64, ViReal64, ViReal64, ViChar[]);
ViStatus _VI_FUNC RSCMU_WCDMA_MeasPow            (ViSession, ViChar*, ViInt32, ViReal64*, ViReal64*, ViReal64*, ViReal64*, ViReal64*, ViReal64*, ViReal64*, ViReal64*, ViChar[]);
ViStatus _VI_FUNC RSCMU_WCDMA_MeasPowIltp        (ViSession, ViChar*, ViInt32, ViReal64*, ViReal64*, ViReal64*, ViReal64*, ViReal64*, ViReal64*, ViReal64*, ViReal64*, ViChar[]);
ViStatus _VI_FUNC RSCMU_WCDMA_MeasOverQpsk       (ViSession, ViChar*, ViInt32, ViReal64*, ViReal64*, ViReal64*, ViReal64*, ViReal64*, ViReal64*, ViReal64*, ViReal64*, ViReal64*, ViReal64*, ViReal64*, ViReal64*, ViChar[]);
ViStatus _VI_FUNC RSCMU_WCDMA_MeasOverWcdm       (ViSession, ViChar*, ViInt32, ViInt32, ViReal64*, ViReal64*, ViReal64*, ViReal64*, ViReal64*, ViReal64*, ViReal64*, ViReal64*, ViReal64*, ViReal64*, ViReal64*, ViReal64*, ViReal64*, ViReal64*, ViInt32*, ViChar[]);
ViStatus _VI_FUNC RSCMU_WCDMA_MeasEvmQpsk        (ViSession, ViChar*, ViInt32, ViReal64*, ViReal64*, ViReal64*, ViReal64*, ViReal64*, ViReal64*, ViChar[]);
ViStatus _VI_FUNC RSCMU_WCDMA_MeasEvmWcdm        (ViSession, ViChar*, ViInt32, ViInt32, ViReal64*, ViReal64*, ViReal64*, ViReal64*, ViReal64*, ViReal64*, ViReal64*, ViInt32*, ViChar[]);
ViStatus _VI_FUNC RSCMU_WCDMA_MeasMerrQpsk       (ViSession, ViChar*, ViInt32, ViReal64*, ViReal64*, ViReal64*, ViReal64*, ViReal64*, ViReal64*, ViChar[]);
ViStatus _VI_FUNC RSCMU_WCDMA_MeasMerrWcdm       (ViSession, ViChar*, ViInt32, ViInt32, ViReal64*, ViReal64*, ViReal64*, ViReal64*, ViReal64*, ViReal64*, ViReal64*, ViInt32*, ViChar[]);
ViStatus _VI_FUNC RSCMU_WCDMA_MeasPerrQpsk       (ViSession, ViChar*, ViInt32, ViReal64*, ViReal64*, ViReal64*, ViReal64*, ViReal64*, ViReal64*, ViChar[]);
ViStatus _VI_FUNC RSCMU_WCDMA_MeasPerrWcdm       (ViSession, ViChar*, ViInt32, ViInt32, ViReal64*, ViReal64*, ViReal64*, ViReal64*, ViReal64*, ViReal64*, ViReal64*, ViInt32*, ViChar[]);
ViStatus _VI_FUNC RSCMU_WCDMA_MeasSpecMfil       (ViSession, ViChar*, ViInt32, ViInt32, ViReal64*, ViReal64*, ViReal64*, ViReal64*, ViReal64*, ViReal64*, ViReal64*, ViReal64*, ViReal64*, ViChar[]);
ViStatus _VI_FUNC RSCMU_WCDMA_MeasSpecMfft       (ViSession, ViChar*, ViInt32, ViInt32, ViReal64*, ViReal64*, ViReal64*, ViReal64*, ViReal64*, ViReal64*, ViReal64*, ViReal64*, ViReal64*, ViReal64*, ViReal64*, ViReal64*, ViChar[]);
ViStatus _VI_FUNC RSCMU_WCDMA_MeasSpecEmas       (ViSession, ViChar*, ViReal64*, ViReal64*, ViReal64*, ViChar[]);
ViStatus _VI_FUNC RSCMU_WCDMA_MeasCdpAuto        (ViSession, ViChar*, ViInt32, ViReal64*, ViReal64*, ViReal64*, ViReal64*, ViChar[]);
ViStatus _VI_FUNC RSCMU_WCDMA_MeasCdpMan         (ViSession, ViChar*, ViInt32, ViReal64*, ViReal64*, ViReal64*, ViReal64*, ViReal64*, ViChar[]);
ViStatus _VI_FUNC RSCMU_WCDMA_MeasArr            (ViSession, ViChar*, ViChar*, ViReal64[], ViChar[]);
ViInt32  RSCMU_WCDMA_ChanToNr           (ViInt32);
ViStatus _VI_FUNC RSCMU_WCDMA_CheckMinFwVersion  (ViSession, ViInt32, ViInt32, ViChar[]);
ViStatus _VI_FUNC RSCMU_WCDMA_MeasRXQuBER       (ViSession ,ViChar*,ViReal64*, ViReal64*,ViReal64*,ViInt32*,ViInt32*,ViChar[]);

/*****************************************************************************/
/* GLOBAL FUNCTIONS CODE                                                     */
/*****************************************************************************/
/* ----------------------------------------------------------------------------
 * WCDMA MEASUREMENTS FOR BOTH MODES, SIGNALLING AND NON SIGNALLING:
 *
 * This functions are available in both modes, Signalling and in Non Signalling.
 * ----------------------------------------------------------------------------
 */

ViStatus _VI_FUNC RSCMUNB_Init_NB_Sig (ViSession instrumentHandle_Base,
                                       ViSession *instrumentHandle_NBSig,
                                       ViChar errorMessage[])
{
    return RSCMUNB_Init_NB_NSig (instrumentHandle_Base, instrumentHandle_NBSig,RSCMUNB_WCDMA1900FDD_SIG, errorMessage);
}

/*===========================================================================*/
/* Function: CDP Fast Channel Detection                                      */
/* Purpose : This function activates automatic channel detection, stores the */
/*           detected DL channel configuration to the selected user test     */
/*           model and activates the user test model.                        */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_SigNsig_ConfCdpFchd (ViSession instrumentHandle,
                                      ViInt32 testModel, ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setEnum(instrumentHandle,
                              "CONFigure:CDPower:FCHDetection",
                              testModel - 10,
                              RSCMUNB_FastChanDet,
                              RSCMU_ERROR_PARAMETER2));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: CDP Default                                                     */
/* Purpose : This function sets all parameters of the subsystem to default   */
/*           values (the setting OFF results in an error message).           */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_SigNsig_DefCdpCont (ViSession instrumentHandle,
                                     ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_sendCmd(instrumentHandle,
                                  "DEFault:CDPower:CDPower:CONTrol ON"));

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Check Limits                                                    */
/* Purpose : This function indicates whether and in which way the permissible*/
/*           tolerances for the scalar results in the code domain power      */
/*           measurement have been exceeded.                                 */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_SigNsig_CalcCdpCdpMatcLim (ViSession instrumentHandle,
                                            ViInt32 errorVectorMagnitude[],
                                            ViInt32 PCDError[],
                                            ViInt32 IQOriginOffsetCurrent[],
                                            ViInt32 IQImbalance[],
                                            ViInt32 carrierFrequencyError[],
                                            ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViChar      tableOfDataValues[100][100];
    ViChar      rdBuf[RD_BUFFER_SIZE]   = "";
    int         i                       = 0;

    CHECKRETMSG(RSCMU_getViString(instrumentHandle, "CALCulate:SCALar:CDPower:CDPower:MATChing:LIMit?", rdBuf, RD_BUFFER_SIZE, 0));

    // Split the multiple data values separated by a comma:
    RSCMU_SeparateDataValues (rdBuf, ",", tableOfDataValues);

    for(i=0; i<3; i++)
    {
        if ((RSCMU_Status = RSCMU_GetLimitCheckValue (tableOfDataValues, i, &errorVectorMagnitude[i])) < 0)
            return RSCMU_Status;
    }
    for(i=3; i<6; i++)
    {
        if ((RSCMU_Status = RSCMU_GetLimitCheckValue (tableOfDataValues, i, &PCDError[i])) < 0)
            return RSCMU_Status;
    }
    for(i=6; i<9; i++)
    {
        if ((RSCMU_Status = RSCMU_GetLimitCheckValue (tableOfDataValues, i, &IQOriginOffsetCurrent[i])) < 0)
            return RSCMU_Status;
    }
    for(i=9; i<12; i++)
    {
        if ((RSCMU_Status = RSCMU_GetLimitCheckValue (tableOfDataValues, i, &IQImbalance[i])) < 0)
            return RSCMU_Status;
    }
    for(i=12; i<15; i++)
    {
        if ((RSCMU_Status = RSCMU_GetLimitCheckValue (tableOfDataValues, i, &carrierFrequencyError[i])) < 0)
            return RSCMU_Status;
    }

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: PCDE Default                                                    */
/* Purpose : This function sets all parameters of the subsystem to default   */
/*           values (the setting OFF results in an error message).           */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_SigNsig_DefCdpPcdeCont (ViSession instrumentHandle,
                                         ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_sendCmd(instrumentHandle,
                                  "DEFault:CDPower:PCDerror:CONTrol ON"));

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Check Limits                                                    */
/* Purpose : This function indicates whether and in which way the permissible*/
/*           tolerances for the scalar results in the peak code domain error */
/*           measurement have been exceeded.                                 */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_SigNsig_CalcCdpPcdMatcLim (ViSession instrumentHandle,
                                            ViInt32 errorVectorMagnitude[],
                                            ViInt32 PCDError[],
                                            ViInt32 IQOriginOffsetCurrent[],
                                            ViInt32 IQImbalance[],
                                            ViInt32 carrierFrequencyError[],
                                            ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViChar      tableOfDataValues[100][100];
    ViChar      rdBuf[RD_BUFFER_SIZE]   = "";
    int         i                       = 0;

    CHECKRETMSG(RSCMU_getViString(instrumentHandle, "CALCulate:SCALar:CDPower:PCDerror:MATChing:LIMit?", rdBuf, RD_BUFFER_SIZE, 0));

    // Split the multiple data values separated by a comma:
    RSCMU_SeparateDataValues (rdBuf, ",", tableOfDataValues);

    for(i=0; i<3; i++)
    {
        if ((RSCMU_Status = RSCMU_GetLimitCheckValue (tableOfDataValues, i, &errorVectorMagnitude[i])) < 0)
            return RSCMU_Status;
    }
    for(i=3; i<6; i++)
    {
        if ((RSCMU_Status = RSCMU_GetLimitCheckValue (tableOfDataValues, i, &PCDError[i])) < 0)
            return RSCMU_Status;
    }
    for(i=6; i<9; i++)
    {
        if ((RSCMU_Status = RSCMU_GetLimitCheckValue (tableOfDataValues, i, &IQOriginOffsetCurrent[i])) < 0)
            return RSCMU_Status;
    }
    for(i=9; i<12; i++)
    {
        if ((RSCMU_Status = RSCMU_GetLimitCheckValue (tableOfDataValues, i, &IQImbalance[i])) < 0)
            return RSCMU_Status;
    }
    for(i=12; i<15; i++)
    {
        if ((RSCMU_Status = RSCMU_GetLimitCheckValue (tableOfDataValues, i, &carrierFrequencyError[i])) < 0)
            return RSCMU_Status;
    }

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Fast Channel Detection                                          */
/* Purpose : This function activates automatic channel detection, stores the */
/*           detected DL channel configuration to the selected user test     */
/*           model and activates the user test model.                        */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_SigNsig_ConfModFchd (ViSession instrumentHandle,
                                      ViInt32 testModel, ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setEnum(instrumentHandle,
                              "CONFigure:MODulation:FCHDetection",
                              testModel - 10,
                              RSCMUNB_FastChanDet,
                              RSCMU_ERROR_PARAMETER2));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Mod Overview Default                                            */
/* Purpose : This function sets all parameters of the subsystem to default   */
/*           values (the setting OFF results in an error message).           */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_SigNsig_DefModOverWcdmaCont (ViSession instrumentHandle,
                                              ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_sendCmd(instrumentHandle,
                            "DEFault:MODulation:OVERview:WCDMa:CONTrol ON"));

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Check Limits                                                    */
/* Purpose : This function indicates whether and in which way the permissible*/
/*           tolerances for the scalar results in the measurement have been  */
/*           exceeded.                                                       */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_SigNsig_CalcModOvwMatcLim (ViSession instrumentHandle,
                                            ViInt32 EVM_Peak[],
                                            ViInt32 EVM_RMS[],
                                            ViInt32 magnError_Peak[],
                                            ViInt32 magnError_RMS[],
                                            ViInt32 phaseError_Peak[],
                                            ViInt32 phaseError_RMS[],
                                            ViInt32 IQOriginOffsetCurrent[],
                                            ViInt32 IQImbalance[],
                                            ViInt32 frequencyError[],
                                            ViInt32 waveformQuality[],
                                            ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViChar      tableOfDataValues[100][100];
    ViChar      rdBuf[RD_BUFFER_SIZE]   = "";
    int         i                       = 0;

    CHECKRETMSG(RSCMU_getViString(instrumentHandle, "CALC:MOD:OVER:WCDM:MATC:LIM?", rdBuf, RD_BUFFER_SIZE, 0));

    // Split the multiple data values separated by a comma:
    RSCMU_SeparateDataValues (rdBuf, ",", tableOfDataValues);

    for(i=0; i<3; i++)
    {
        if ((RSCMU_Status = RSCMU_GetLimitCheckValue (tableOfDataValues, i, &EVM_Peak[i])) < 0)
            return RSCMU_Status;
    }
    for(i=3; i<6; i++)
    {
        if ((RSCMU_Status = RSCMU_GetLimitCheckValue (tableOfDataValues, i, &EVM_RMS[i])) < 0)
            return RSCMU_Status;
    }
    for(i=6; i<9; i++)
    {
        if ((RSCMU_Status = RSCMU_GetLimitCheckValue (tableOfDataValues, i, &magnError_Peak[i])) < 0)
            return RSCMU_Status;
    }
    for(i=9; i<12; i++)
    {
        if ((RSCMU_Status = RSCMU_GetLimitCheckValue (tableOfDataValues, i, &magnError_RMS[i])) < 0)
            return RSCMU_Status;
    }
    for(i=12; i<15; i++)
    {
        if ((RSCMU_Status = RSCMU_GetLimitCheckValue (tableOfDataValues, i, &phaseError_Peak[i])) < 0)
            return RSCMU_Status;
    }
    for(i=15; i<18; i++)
    {
        if ((RSCMU_Status = RSCMU_GetLimitCheckValue (tableOfDataValues, i, &phaseError_RMS[i])) < 0)
            return RSCMU_Status;
    }
    for(i=18; i<21; i++)
    {
        if ((RSCMU_Status = RSCMU_GetLimitCheckValue (tableOfDataValues, i, &IQOriginOffsetCurrent[i])) < 0)
            return RSCMU_Status;
    }
    for(i=21; i<24; i++)
    {
        if ((RSCMU_Status = RSCMU_GetLimitCheckValue (tableOfDataValues, i, &IQImbalance[i])) < 0)
            return RSCMU_Status;
    }
    for(i=24; i<27; i++)
    {
        if ((RSCMU_Status = RSCMU_GetLimitCheckValue (tableOfDataValues, i, &frequencyError[i])) < 0)
            return RSCMU_Status;
    }
    for(i=27; i<30; i++)
    {
        if ((RSCMU_Status = RSCMU_GetLimitCheckValue (tableOfDataValues, i, &waveformQuality[i])) < 0)
            return RSCMU_Status;
    }

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Mod EVM Default                                                 */
/* Purpose : This function sets all parameters of the subsystem to default   */
/*           values (the setting OFF results in an error message).           */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_SigNsig_DefModEvmWcdmCont (ViSession instrumentHandle,
                                            ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_sendCmd(instrumentHandle,
                          "DEFault:MODulation:EVMagnitude:WCDMa:CONTrol ON"));

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Check Limits                                                    */
/* Purpose : This function indicates whether and in which way the permissible*/
/*           tolerances for the scalar results in the measurement have been  */
/*           exceeded.                                                       */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_SigNsig_CalcModEvmMatcLim (ViSession instrumentHandle,
                                            ViInt32 EVM_Peak[],
                                            ViInt32 EVM_RMS[],
                                            ViInt32 IQOriginOffsetCurrent[],
                                            ViInt32 IQImbalance[],
                                            ViInt32 frequencyError[],
                                            ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViChar      tableOfDataValues[100][100];
    ViChar      rdBuf[RD_BUFFER_SIZE]   = "";
    int         i                       = 0;

    CHECKRETMSG(RSCMU_getViString(instrumentHandle, "CALCulate:SCALar:MODulation:EVMagnitude:WCDMa:MATChing:LIMit?", rdBuf, RD_BUFFER_SIZE, 0));

    // Split the multiple data values separated by a comma:
    RSCMU_SeparateDataValues (rdBuf, ",", tableOfDataValues);

    for(i=0; i<3; i++)
    {
        if ((RSCMU_Status = RSCMU_GetLimitCheckValue (tableOfDataValues, i, &EVM_Peak[i])) < 0)
            return RSCMU_Status;
    }
    for(i=3; i<6; i++)
    {
        if ((RSCMU_Status = RSCMU_GetLimitCheckValue (tableOfDataValues, i, &EVM_RMS[i])) < 0)
            return RSCMU_Status;
    }
    for(i=6; i<9; i++)
    {
        if ((RSCMU_Status = RSCMU_GetLimitCheckValue (tableOfDataValues, i, &IQOriginOffsetCurrent[i])) < 0)
            return RSCMU_Status;
    }
    for(i=9; i<12; i++)
    {
        if ((RSCMU_Status = RSCMU_GetLimitCheckValue (tableOfDataValues, i, &IQImbalance[i])) < 0)
            return RSCMU_Status;
    }
    for(i=12; i<15; i++)
    {
        if ((RSCMU_Status = RSCMU_GetLimitCheckValue (tableOfDataValues, i, &frequencyError[i])) < 0)
            return RSCMU_Status;
    }

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Mod Magn Error Default                                          */
/* Purpose : This function sets all parameters of the subsystem to default   */
/*           values (the setting OFF results in an error message).           */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_SigNsig_DefModMerrWcdmaCont (ViSession instrumentHandle,
                                              ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_sendCmd(instrumentHandle,
                              "DEFault:MODulation:MERRor:WCDMa:CONTrol ON"));

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Check Limits                                                    */
/* Purpose : This function indicates whether and in which way the permissible*/
/*           tolerances for the scalar results in the measurement have been  */
/*           exceeded.                                                       */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_SigNsig_CalcModMerrMatcLim (ViSession instrumentHandle,
                                             ViInt32 magnError_Peak[],
                                             ViInt32 magnError_RMS[],
                                             ViInt32 IQOriginOffsetCurrent[],
                                             ViInt32 IQImbalance[],
                                             ViInt32 frequencyError[],
                                             ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViChar      tableOfDataValues[100][100];
    ViChar      rdBuf[RD_BUFFER_SIZE]   = "";
    int         i                       = 0;

    CHECKRETMSG(RSCMU_getViString(instrumentHandle, "CALCulate:SCALar:MODulation:MERRor:WCDMa:MATChing:LIMit?", rdBuf, RD_BUFFER_SIZE, 0));

    // Split the multiple data values separated by a comma:
    RSCMU_SeparateDataValues (rdBuf, ",", tableOfDataValues);

    for(i=0; i<3; i++)
    {
        if ((RSCMU_Status = RSCMU_GetLimitCheckValue (tableOfDataValues, i, &magnError_Peak[i])) < 0)
            return RSCMU_Status;
    }
    for(i=3; i<6; i++)
    {
        if ((RSCMU_Status = RSCMU_GetLimitCheckValue (tableOfDataValues, i, &magnError_RMS[i])) < 0)
            return RSCMU_Status;
    }
    for(i=6; i<9; i++)
    {
        if ((RSCMU_Status = RSCMU_GetLimitCheckValue (tableOfDataValues, i, &IQOriginOffsetCurrent[i])) < 0)
            return RSCMU_Status;
    }
    for(i=9; i<12; i++)
    {
        if ((RSCMU_Status = RSCMU_GetLimitCheckValue (tableOfDataValues, i, &IQImbalance[i])) < 0)
            return RSCMU_Status;
    }
    for(i=12; i<15; i++)
    {
        if ((RSCMU_Status = RSCMU_GetLimitCheckValue (tableOfDataValues, i, &frequencyError[i])) < 0)
            return RSCMU_Status;
    }

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Mod Phase Error Default                                         */
/* Purpose : This function sets all parameters of the subsystem to default   */
/*           values (the setting OFF results in an error message).           */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_SigNsig_DefModPerrWcdmaCont (ViSession instrumentHandle,
                                              ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_sendCmd(instrumentHandle,
                              "DEFault:MODulation:PERRor:WCDMa:CONTrol ON"));

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Check Limits                                                    */
/* Purpose : This function indicates whether and in which way the permissible*/
/*           tolerances for the scalar results in the measurement have been  */
/*           exceeded.                                                       */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_SigNsig_CalcModPerrMatcLim (ViSession instrumentHandle,
                                             ViInt32 phaseError_Peak[],
                                             ViInt32 phaseError_RMS[],
                                             ViInt32 IQOriginOffsetCurrent[],
                                             ViInt32 IQImbalance[],
                                             ViInt32 frequencyError[],
                                             ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViChar      tableOfDataValues[100][100];
    ViChar      rdBuf[RD_BUFFER_SIZE]   = "";
    int         i                       = 0;

    CHECKRETMSG(RSCMU_getViString(instrumentHandle, "CALCulate:SCALar:MODulation:PERRor:WCDMa:MATChing:LIMit?", rdBuf, RD_BUFFER_SIZE, 0));

    // Split the multiple data values separated by a comma:
    RSCMU_SeparateDataValues (rdBuf, ",", tableOfDataValues);

    for(i=0; i<3; i++)
    {
        if ((RSCMU_Status = RSCMU_GetLimitCheckValue (tableOfDataValues, i, &phaseError_Peak[i])) < 0)
            return RSCMU_Status;
    }
    for(i=3; i<6; i++)
    {
        if ((RSCMU_Status = RSCMU_GetLimitCheckValue (tableOfDataValues, i, &phaseError_RMS[i])) < 0)
            return RSCMU_Status;
    }
    for(i=6; i<9; i++)
    {
        if ((RSCMU_Status = RSCMU_GetLimitCheckValue (tableOfDataValues, i, &IQOriginOffsetCurrent[i])) < 0)
            return RSCMU_Status;
    }
    for(i=9; i<12; i++)
    {
        if ((RSCMU_Status = RSCMU_GetLimitCheckValue (tableOfDataValues, i, &IQImbalance[i])) < 0)
            return RSCMU_Status;
    }
    for(i=12; i<15; i++)
    {
        if ((RSCMU_Status = RSCMU_GetLimitCheckValue (tableOfDataValues, i, &frequencyError[i])) < 0)
            return RSCMU_Status;
    }

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Set All Limits                                                  */
/* Purpose : This function defines upper limits for the Current and Max./Min.*/
/*           traces evaluated over the whole evaluation period and for the   */
/*           scalar modulation parameters derived from them. For parameters  */
/*           with symmetric ranges of limits (e.g. the Phase Error (Peak)),  */
/*           the entry of positive or negative values (say +x or -x) is      */
/*           equivalent: The limit will be interpreted in such a way that    */
/*           the parameter must be confined in the range [-x,+x].            */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_SigNsig_ConfCdpLimScalAll (ViSession instrumentHandle,
                                         ViInt32 statisticalType,
                                         ViReal64 errorVectorMagn_Peak_,
                                         ViReal64 errorVectorMagn_RMS_,
                                         ViReal64 magnitudeError_Peak_,
                                         ViReal64 magnitudeError_RMS_,
                                         ViReal64 phaseError_Peak_Deg,
                                         ViReal64 phaseError_RMS_Deg,
                                         ViReal64 iQOriginOffsetdB,
                                         ViReal64 iQImbalancedB,
                                         ViReal64 carrierFrequencyErrorHz,
                                         ViReal64 waveformQuality,
                                         ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViChar      cmd[WR_BUFFER_SIZE]     = "";

    if (RSCMU_invalidViInt32Range (statisticalType, 0, 1))
        RETMSG(RSCMU_ERROR_PARAMETER2);
    if (RSCMU_invalidViReal64Range (errorVectorMagn_Peak_, 0.0, 100.0))
        RETMSG(RSCMU_ERROR_PARAMETER3);
    if (RSCMU_invalidViReal64Range (errorVectorMagn_RMS_, 0.0, 100.0))
        RETMSG(RSCMU_ERROR_PARAMETER4);
    if (RSCMU_invalidViReal64Range (magnitudeError_Peak_, -100.0, 100.0))
        RETMSG(RSCMU_ERROR_PARAMETER5);
    if (RSCMU_invalidViReal64Range (magnitudeError_RMS_, 0.0, 100.0))
        RETMSG(RSCMU_ERROR_PARAMETER6);
    if (RSCMU_invalidViReal64Range (phaseError_Peak_Deg, -180.0, 180.0))
        RETMSG(RSCMU_ERROR_PARAMETER7);
    if (RSCMU_invalidViReal64Range (phaseError_RMS_Deg, 0.0, 180.0))
        RETMSG(RSCMU_ERROR_PARAMETER8);
    if (RSCMU_invalidViReal64Range (iQOriginOffsetdB, -80.0, 0))
        RETMSG(RSCMU_ERROR_PARAMETER9);
    if (RSCMU_invalidViReal64Range (iQImbalancedB, -80.0, 0))
        RETMSG(RSCMU_ERROR_PARAMETER10);
    if (RSCMU_invalidViReal64Range (carrierFrequencyErrorHz,-4000.0, 4000.0))
        RETMSG(RSCMU_ERROR_PARAMETER11);
    if (RSCMU_invalidViReal64Range (waveformQuality, 0.7, 1))
        RETMSG(RSCMU_ERROR_PARAMETER12);

    sprintf(cmd, "CONFigure:MODulation:OEMP:WCDMa:%s:LIMit:SCALar:SYMMetric:COMBined:VALue %.12f,%.12f,%.12f,%.12f,%.12f,%.12f,%.12f,%.12f,%.12f,%.12f",
                                RSCMUNB_statType[statisticalType],
                                errorVectorMagn_Peak_, errorVectorMagn_RMS_,
                                magnitudeError_Peak_, magnitudeError_RMS_,
                                phaseError_Peak_Deg, phaseError_RMS_Deg,
                                iQOriginOffsetdB, iQImbalancedB,
                                carrierFrequencyErrorHz, waveformQuality);

    CHECKRETMSG(RSCMU_sendCmd(instrumentHandle, cmd));

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Set Limits                                                      */
/* Purpose : This function define upper and lower limits for the Current and */
/*           Max./Min. or Average traces evaluated over the whole evaluation */
/*           period and for the scalar modulation parameters derived from    */
/*           them. For parameters with symmetric ranges of limits (e.g. the  */
/*           Phase Error (Peak)), the entry of positive or negative values   */
/*           (say +x or -x) is equivalent: The limit will be interpreted in  */
/*           such a way that the parameter must be confined in the range     */
/*           [-x,+x].                                                        */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_SigNsig_ConfCdpLimScal (ViSession instrumentHandle,
                                         ViInt32 selectParameter,
                                         ViInt32 statisticalType,
                                         ViInt32 enableLimit,
                                         ViReal64 limitValue,
                                         ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViChar      cmd1[WR_BUFFER_SIZE]    = "";
    ViChar      cmd2[WR_BUFFER_SIZE]    = "";

    if (RSCMU_invalidViInt32Range (selectParameter, 0, 9))
        RETMSG(RSCMU_ERROR_PARAMETER2);
    if (RSCMU_invalidViBooleanRange ((ViBoolean)statisticalType))
        RETMSG(RSCMU_ERROR_PARAMETER3);
    if (RSCMU_invalidViBooleanRange ((ViBoolean)enableLimit))
        RETMSG(RSCMU_ERROR_PARAMETER4);
    /* Parameter 5 not range checked */

    switch (selectParameter)
    {
        case RSCMUNB_LIMIT_PARAMETER_MEP:
        case RSCMUNB_LIMIT_PARAMETER_PEP:
        case RSCMUNB_LIMIT_PARAMETER_CFER:
            sprintf(cmd1, "CONFigure:MODulation:OEMP:WCDMa:%s:%s:LIMit:SCALar:SYMMetric:VALue %.12f",
                            RSCMUNB_LimitParameter[selectParameter], RSCMUNB_statType[statisticalType], limitValue);
            sprintf(cmd2, "CONFigure:MODulation:OEMP:WCDMa:%s:%s:LIMit:SCALar:SYMMetric:ENABle %s",
                        RSCMUNB_LimitParameter[selectParameter], RSCMUNB_statType[statisticalType], RSCMUNB_mode_On_Off[enableLimit]);
            break;
        case RSCMUNB_LIMIT_PARAMETER_WFQ:
            sprintf(cmd1, "CONFigure:MODulation:OEMP:WCDMa:%s:%s:LIMit:SCALar:ASYMmetric:LOWer:VALue %.12f",
                            RSCMUNB_LimitParameter[selectParameter], RSCMUNB_statType[statisticalType], limitValue);
            sprintf(cmd2, "CONFigure:MODulation:OEMP:WCDMa:%s:%s:LIMit:SCALar:ASYMmetric:LOWer:ENABle %s",
                        RSCMUNB_LimitParameter[selectParameter], RSCMUNB_statType[statisticalType], RSCMUNB_mode_On_Off[enableLimit]);
            break;
        default:
            sprintf(cmd1, "CONFigure:MODulation:OEMP:WCDMa:%s:%s:LIMit:SCALar:ASYMmetric:UPPer:VALue %.12f",
                            RSCMUNB_LimitParameter[selectParameter], RSCMUNB_statType[statisticalType], limitValue);
            sprintf(cmd2, "CONFigure:MODulation:OEMP:WCDMa:%s:%s:LIMit:SCALar:ASYMmetric:UPPer:ENABle %s",
                        RSCMUNB_LimitParameter[selectParameter], RSCMUNB_statType[statisticalType], RSCMUNB_mode_On_Off[enableLimit]);
            break;
    }
    

    CHECKRETMSG(RSCMU_sendCmd(instrumentHandle, cmd1));


    CHECKRETMSG(RSCMU_sendCmd(instrumentHandle, cmd2));

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Default Limit                                                   */
/* Purpose : This function sets all parameters of the subsystem to default   */
/*           values (the setting OFF results in an error message).           */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_SigNsig_DefModOempWcdmaLim (ViSession instrumentHandle,
                                             ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_sendCmd(instrumentHandle,
                                "DEFault:MODulation:OEMP:WCDMa:LIMit ON"));

    RETMSG(RSCMU_Status);
}
/* ----------------------------------------------------------------------------
 * WCDMA MEASUREMENTS WITHOUT SIGNALLING (NON SIGNALLING):
 *
 * In the Non Signalling mode, a WCDMA-specific RF signal can be generated and
 * an RF signal with WCDMA characteristics analyzed. No signalling parameters
 * are transferred and measured.
 * ----------------------------------------------------------------------------
 * INITIALIZATION
 * ----------------------------------------------------------------------------
 */
/*===========================================================================*/
/* Function: Non Signalling initialization                                   */
/* Purpose : This function can be called only after the initialization of the*/
/*           "Basic Unit" (RSCMU200). It opens the instrument, queries the   */
/*           instrument for its ID, and initializes the instrument to a known*/
/*           state.                                                          */
/*===========================================================================*/

ViStatus _VI_FUNC RSCMUNB_Init_NB_NSig (ViSession instrumentHandle_Base,
                                        ViPSession instrumentHandle_NBNSig,
                                        ViUInt32 functionGroupName,
                                        ViChar _VI_FAR errorMessage[])
{
  ViStatus   RSCMU_Status      = VI_SUCCESS;
  ViInt32    secondAddr        = 0;
  ViChar     FctGrpBuffer[128] = "";
  ViInt32    comp              = 1;
  ViChar     resource[25]      = "";
  ViBoolean  Flag              = RSCMU_OFF;
  ViSession  DefaultRM;
  ViChar     baseRevString[RD_BUFFER_SIZE]     ="";
  ViChar     nbRevString[RD_BUFFER_SIZE]       ="";
  ViChar     rdBuf[MID_BUFFER_SIZE]            ="";
#ifdef _DEBUGOUTPUT_
  ViChar     search[RD_BUFFER_SIZE] = "";
#endif
  struct RSCMU_Base_Globals   *p_globalvarBase;
  struct RSCMUNB_NSig_Globals  *p_globalvarNB;



    /* --- Set standard "C" locale --- */
    {
        char *locale;

        locale = setlocale(LC_ALL, "C");
    }

  // Get the data used privately by this driver for this instrument session:
  if ((RSCMU_Status = viGetAttribute (instrumentHandle_Base, VI_ATTR_USER_DATA, (ViAddr)&p_globalvarBase)) < 0)
    return RSCMUNB_errorMessage (instrumentHandle_Base, RSCMU_Status, errorMessage);

  DefaultRM = p_globalvarBase->RSCMU_tGlobal.RSCMU_RmSessionDefault;
  // Verify that the VISA Resource Manager has been opened.
  if (DefaultRM != 0)
  {
    // Record "Function Group Name" for the actual Session.
    //strcpy (FctGrpBuffer, "\"WCDMA1900NBFDD_NSig\"");
    strcpy (FctGrpBuffer, RSCMUNB_function_group_name[functionGroupName]);

    // Init the Error Message
    strcpy (errorMessage, "No Error");

    // Find the Secondary address
    while ((comp != 0) && (secondAddr < 30))
    {
      comp = strncmp (p_globalvarBase->RSCMU_tSecAddr[secondAddr].FctGrp,
                      FctGrpBuffer,
                      strlen (FctGrpBuffer));
      secondAddr++;
      if (comp == 0)
      {
        Flag = RSCMU_ON;
      }
    }

    if (Flag == RSCMU_OFF)
    {
      return RSCMUNB_errorMessage (*instrumentHandle_NBNSig, RSCMU_ERROR_INVALID_FCT_GRP, errorMessage);
    }

    // Hardware Connection
    comp = strncmp ("GPIB", p_globalvarBase->RSCMU_tGlobal.RSCMU_SCPIConnection, 4);

    if (comp == 0)
    {
      sprintf (resource,
               "%s::%ld::%ld::INSTR",
               p_globalvarBase->RSCMU_tGlobal.RSCMU_SCPIConnection,
               p_globalvarBase->RSCMU_tGlobal.RSCMU_primary_address,
               secondAddr-1);
    }
    else
    {
      sprintf (resource,
               "%s::INSTR",
               p_globalvarBase->RSCMU_tGlobal.RSCMU_SCPIConnection);
    }

    // Open instrument session
    if ((RSCMU_Status = viOpen (DefaultRM, resource, VI_NULL, 3000, instrumentHandle_NBNSig)) < 0)
    {
      if ((RSCMU_Status = viClose (DefaultRM)) < 0)
      {
        return (RSCMU_Status);
      }
    }

    // Allocate space for "struct RSCMU_SESSION_Globals"
    if ((p_globalvarNB = (struct RSCMUNB_NSig_Globals *) malloc(sizeof(struct RSCMUNB_NSig_Globals))) == 0)
    {
      return RSCMU_initCleanUp (DefaultRM, *instrumentHandle_NBNSig, RSCMU_Status);
    }

    memset (p_globalvarNB, 0, sizeof(struct RSCMUNB_NSig_Globals));
    
    if ((RSCMU_Status = viSetAttribute (*instrumentHandle_NBNSig, VI_ATTR_USER_DATA, (ViAttrState)p_globalvarNB)) < 0)
    {
      return RSCMU_initCleanUp (DefaultRM, *instrumentHandle_NBNSig, RSCMU_Status);
    }

    // Set initial values of variables for function Group 'BT_SIG' .
    strcpy (p_globalvarNB->RSCMU_tGlobal.FGrp_Name, FctGrpBuffer);

    p_globalvarNB->RSCMU_tGlobal.RSCMU_primary_address = p_globalvarBase->RSCMU_tGlobal.RSCMU_primary_address;
    p_globalvarNB->RSCMU_tGlobal.RSCMU_secondary_address = secondAddr - 1;
    p_globalvarNB->RSCMU_tGlobal.RSCMU_TRACE = p_globalvarBase->RSCMU_tGlobal.RSCMU_TRACE;

    // Record the execution of this Init Function.
    p_globalvarBase->RSCMU_tSecAddr[secondAddr - 1].SessionNb = *instrumentHandle_NBNSig;

    // Base CMU FW available
    strcpy (p_globalvarNB->RSCMU_tGlobal.CmuVersion, p_globalvarBase->RSCMU_tGlobal.CmuVersion);
    p_globalvarNB->RSCMU_tGlobal.RSCMU_FW_Version = p_globalvarBase->RSCMU_tGlobal.RSCMU_FW_Version;

    
    CHECKRET(RSCMUNB_Get_NB_FW_Version(*instrumentHandle_NBNSig, &RSCMUNB_nbFwVersion, &RSCMUNB_baseFwVersion));

    CHECKRET(RSCMU_DrvVersionStringToIntegers(RSCMUNB_VERSION,&RSCMUNB_nbDrvVersion,&RSCMUNB_baseDrvVersion));
//  RSCMUBT_Get_BT_DRV_Version( &RSCMUBT_baseDrvVersion, &RSCMUBT_btDrvVersion);
    strcpy(nbRevString, RSCMUNB_REVISION);
    strcpy(baseRevString, RSCMU_BASE_REVISION);

    // check, if we use a correct firmware version
    if (RSCMUNB_nbFwVersion < 3201399)  // Version V3.20M
    {
      RSCMU_Status = RSCMUNB_ERROR_FW_VERSION;
        RSCMU_Base_errorMessage (*instrumentHandle_NBNSig, RSCMU_Status, errorMessage);
        RSCMU_initCleanUp (DefaultRM, *instrumentHandle_NBNSig, RSCMU_Status);
        return RSCMU_Status;
    }

    // check, if we use a base driver version >= 3.10.00
    if (RSCMUNB_nbFwVersion < 3100000)  // Version 3.10.00
    {
        RSCMU_Status = RSCMUNB_ERROR_BASEDRIVER_VERSION;
        RSCMU_Base_errorMessage (*instrumentHandle_NBNSig, RSCMU_Status, errorMessage);
        RSCMU_initCleanUp (DefaultRM, *instrumentHandle_NBNSig, RSCMU_Status);
        return RSCMU_Status;
    }

    CHECKRETMSG(RSCMU_getViString(instrumentHandle_Base,"*OPT?",rdBuf,MID_BUFFER_SIZE,0));
    

    #ifdef _DEBUGOUTPUT_
    strcpy(search, "K75");
    
    if(strstr(rdBuf,search));
        K75Present = VI_TRUE;
    else
        K75Present = VI_FALSE;

    strcpy(search, "K76");
    
    if( strstr(rdBuf,search));
        K76Present = VI_TRUE;
    else
        K76Present = VI_FALSE;

        printf ("RSCMUNB::INIT::Node-B Driver Revisionstring: %s\n",nbRevString);
        printf ("RSCMUNB::INIT::Base   Driver Revisionstring: %s\n",baseRevString);

        printf ("\nRSCMUNB::INIT::Node-B Firmwareversion:       %ld\n",RSCMUNB_nbFwVersion);
        printf ("RSCMUNB::INIT::Base   Firmwareversion:       %ld\n",RSCMUNB_baseFwVersion);

        printf ("\nRSCMUNB::INIT::Node-B Driverversion:         %ld\n",RSCMUNB_nbDrvVersion);
        printf ("RSCMUNB::INIT::Base   Driverversion:         %ld\n",RSCMUNB_baseDrvVersion);

        printf ("\nRSCMUNB::INIT::OptionK75:       %ld\n",K75Present);
        printf ("\nRSCMUNB::INIT::OptionK76:       %ld\n",K76Present);
    #endif //  _DEBUGOUTPUT_

    // Timeout changed to 15s
    if ((RSCMU_Status = viSetAttribute (*instrumentHandle_NBNSig, VI_ATTR_TMO_VALUE, 15000)) < 0)
    {
      return (RSCMU_Status);
    }
    
  }

  return (RSCMU_Status);
}

/*===========================================================================*/
/* Function: Revision Query                                                  */
/* Purpose:  This function returns the driver and instrument revisions.      */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_RevisionQuery (ViSession  instrumentHandle,
                                         ViChar     driverRev[],
                                         ViChar     instrRev[],
                                         ViChar    errorMessage[])
{
    int i=0;
    int len =0;

    ViStatus   RSCMU_Status = VI_SUCCESS;
    ViChar     readBuffer[RD_BUFFER_SIZE] = "";
    ViChar*    fgFwVersion = "";

    CHECKRET(RSCMU_Status = RSCMU_getViString(instrumentHandle,
                                     "SYSTem:VERSion:SW:MMI?",
                                     readBuffer,
                                     RD_BUFFER_SIZE,
                                     RSCMU_ERROR_PARAMETER2));

    strlen ((char*)readBuffer);

    for ( i=0; i<len; i++)
            readBuffer[i] = toupper(readBuffer[i]);

// printf("SYSTem:VERSion:SW:MMI? liefert: %s\nAddr instrRev: %p\n",readBuffer, instrRev);

    // Split the total string into functiongroup and base part
    GET_VI_STRING(fgFwVersion,readBuffer);
    strcpy (driverRev, RSCMUNB_REVISION);
    fgFwVersion[strlen(fgFwVersion)-1] = 0;   // remove trailing  "
    strcpy (instrRev,  fgFwVersion+1);        // remove leading "  (and coly the content)
// printf("SYSTem:VERSion:SW:MMI? liefert als instr Rev: %s\nAddr instrRev: %p\n",instrRev, instrRev);
    return RSCMU_Status;
}

/*===========================================================================*/
/* Function: OptionQueryK75                                                  */
/* Purpose:  This function returns if Option K75 (TX-Measurements) is        */
/*           available in the instrument.                                    */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_OptionQueryK75 (ViSession instrumentHandle,
                                 ViBoolean* optAvailable,
                                 ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    if (RSCMUNB_nbFwVersion < 32000)
        RSCMU_Status = RSCMUNB_ERROR_FW_VERSION;

    *optAvailable = K75Present;
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: OptionQueryK76                                                  */
/* Purpose:  This function returns if Option K76 (WCDMA Generator) is        */
/*           available in the instrument.                                    */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_OptionQueryK76 (ViSession instrumentHandle,
                                 ViBoolean* optAvailable,
                                 ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    if (RSCMUNB_nbFwVersion < 32000)
        RSCMU_Status = RSCMUNB_ERROR_FW_VERSION;

    *optAvailable = K76Present;
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: RF Analyzer Channel                                             */
/* Purpose:  This function defines the frequency of the RF signal analyzed.  */
/*           The default frequency unit is UARFCN (UTRA Absolute Radio       */
/*           Frequency Channel Numbers). The assignment of channel numbers   */
/*           and frequencies meets the 3GPP specifications.                  */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_RfanChan (ViSession instrumentHandle,
                                 ViInt32 channelNumber,
                                 ViChar _VI_FAR errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_viWrite (instrumentHandle,
                                "SENSe:RFANalyzer:FREQuency:UNIT CH",
                                strlen("SENSe:RFANalyzer:FREQuency:UNIT CH")));

    CHECKRETMSG(RSCMU_setViInt32(instrumentHandle,
                            "SENSe:RFANalyzer:FREQuency",
                            channelNumber,
                            0, 13500,
                            RSCMU_ERROR_PARAMETER2));

  RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: RF Analyzer Frequency                                           */
/* Purpose:  This function defines the frequency of the RF signal analyzed in*/
/*           specified frequency units.                                      */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_RfanFreq (ViSession instrumentHandle,
                                         ViUInt32 unit,
                                         ViReal64 frequency,
                                         ViChar _VI_FAR errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

  // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(instrumentHandle, unit, 0, 4, RSCMU_ERROR_PARAMETER2, errorMessage);

    switch (unit)
    {
        case RSCMU_HZ:
            CHECK_VI_REAL64_RANGE(instrumentHandle, frequency, 200000.0, 2700000000.0, RSCMU_ERROR_PARAMETER3, errorMessage);
        break;
        case RSCMU_KHZ:
            CHECK_VI_REAL64_RANGE(instrumentHandle, frequency, 200.0, 2700000.0, RSCMU_ERROR_PARAMETER3, errorMessage);
        break;
        case RSCMU_MHZ:
            CHECK_VI_REAL64_RANGE(instrumentHandle, frequency, 0.2, 2700.0, RSCMU_ERROR_PARAMETER3, errorMessage);
        break;
        case RSCMU_GHZ:
            CHECK_VI_REAL64_RANGE(instrumentHandle, frequency, 0.0002, 2.7, RSCMU_ERROR_PARAMETER3, errorMessage);
        break;
    }

    CHECKRETMSG(RSCMU_setEnum(instrumentHandle,
                              "SENSe:RFANalyzer:FREQuency:UNIT",
                              unit,
                              RSCMUNB_Unit_Frequency,
                              RSCMU_ERROR_PARAMETER2
                             )
                );

    CHECKRETMSG(RSCMU_setViReal64(instrumentHandle,
                                "SENSe:RFANalyzer:FREQuency",
                                frequency,
                                0, 2700000000UL,
                                RSCMU_ERROR_PARAMETER3));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: RF Analyzer Frequency Offset                                    */
/* Purpose:  This function defines an offset for the RF analyzer frequency   */
/*           set with the command [SENSe:]RFANalyzer:FREQuency. See. RF      */
/*           Analyzer Frequency (RSCMUNB_Nsig_RfanFreq) function. The offset*/
/*           frequency must be in multiples of 1 Hz.                         */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_RfanFoff (ViSession instrumentHandle, ViReal64 frequency,
                                 ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    CHECKRETMSG(RSCMU_setViReal64(instrumentHandle,
                                "SENSe:RFANalyzer:FOFFset",
                                frequency,
                                -100000, 100000,
                                RSCMU_ERROR_PARAMETER2));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Trigger Synchronized                                            */
/* Purpose:  This function answers with ON or OFF and tells the user if the  */
/*           CMU (Answer == ON) is synchronized with the trigger of the      */
/*           basestation.                                                    */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_SourRfgTSynch (ViSession instrumentHandle,
                                          ViInt32* triggerSync,
                                          ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_getViInt32(instrumentHandle,
                                "SOURce:RFGenerator:LEVel:DPDCh?",
                                triggerSync,
                                RSCMU_ERROR_PARAMETER2));
    RETMSG(RSCMU_Status);

}

/*===========================================================================*/
/* Function: Total Transmit Power                                            */
/* Purpose:  This function sets the Total TX Power in absolute units. The    */
/*           DPCCH/DPDCH power ratio and the number of DPDCHs is considered  */
/*           to be fixed for all channel configurations, so the command      */
/*           overwrites the value of SOURce:RFGenerator:EPOWer:DPCCh (DPCCH  */
/*           Level (RSCMUNB_Nsig_SourRfgLevDpcc) function) and vice versa   */
/*           (see description of manual control).                            */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_SourRfgLevTtp (ViSession instrumentHandle,
                                      ViReal64 totalTransmitPower,
                                      ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    CHECKRETMSG(RSCMU_setViReal64(instrumentHandle,
                                "SOURce:RFGenerator:LEVel:TTPower",
                                totalTransmitPower,
                                -200, 60,
                                RSCMU_ERROR_PARAMETER2));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: DPCCH Level                                                     */
/* Purpose:  This function sets the level of the Dedicated Physical Control  */
/*           Channel (DPCCH). The DPCCH/DPDCH power ratio and the number of  */
/*           DPDCHs is considered to be fixed for all channel configurations,*/
/*           so the command overwrites the value of                          */
/*           SOURce:RFGenerator:EPOWer:TTPower (Total Transmit Power         */
/*           (RSCMUNB_Nsig_SourRfgLevTtp) function) and vice versa (see     */
/*           description of manual control).                                 */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_SourRfgLevDpcc (ViSession instrumentHandle,
                                       ViReal64 levelDPCCH,
                                       ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    CHECKRETMSG(RSCMU_setViReal64(instrumentHandle,
                                "SOURce:RFGenerator:LEVel:DPCCh",
                                levelDPCCH,
                                -200, 60,
                                RSCMU_ERROR_PARAMETER2));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Get DPDCH Level                                                 */
/* Purpose:  This function queries and returns the level of the Dedicated    */
/*           Physical Data Channel (DPDCH). The DPDCH power depends on the   */
/*           channel configuration and the Total TX Power; see description of*/
/*           manual control.                                                 */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_GetSourRfgLevDpdc (ViSession instrumentHandle,
                                          ViReal64* levelDPDCH,
                                          ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    CHECKRETMSG(RSCMU_getViReal64(instrumentHandle,
                                "SOURce:RFGenerator:LEVel:DPDCh?",
                                levelDPDCH,
                                RSCMU_ERROR_PARAMETER2));
    RETMSG(RSCMU_Status);

}

/*===========================================================================*/
/* Function: AWGN Contribution                                               */
/* Purpose:  This function switches the Additional White Gaussian Noise      */
/*           contribution to the RF generator signal on or off.              */
/*           The AWGN level is defined separately.                           */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_SourRfgLevAwgn (ViSession instrumentHandle,
                                       ViInt32 awgn,
                                       ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    CHECKRETMSG(RSCMU_setEnum(instrumentHandle,
                              "SOURCe:RFGenerator:LEVel:AWGN",
                              awgn,
                              RSCMUNB_mode_On_Off,
                              RSCMU_ERROR_PARAMETER2
                             ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Noise Level                                                     */
/* Purpose:                        */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_SourRfgLevNoise (ViSession instrumentHandle,
                                       ViReal64 noiseLevel,
                                       ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    CHECKRETMSG(RSCMU_setViReal64(instrumentHandle,
                                "SOURce:RFGenerator:LEVel:NLEVel",
                                noiseLevel,
                                -137.0, -23.0,
                                RSCMU_ERROR_PARAMETER2));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Generator Channel                                               */
/* Purpose:  This function defines the frequency of the RF signal generated. */
/*           The frequency is set as channel number (UARFCNs) The assignment */
/*           of channel numbers and frequencies meets the 3GPP specification.*/
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_RfgChan (ViSession instrumentHandle,
                                        ViInt32 channelNumber,
                                        ViChar _VI_FAR errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_viWrite (instrumentHandle,
                                "UNIT:RFGenerator:FREQuency CH",
                                strlen("UNIT:RFGenerator:FREQuency CH")));

    CHECKRETMSG(RSCMU_setViInt32(instrumentHandle,
                            "SOURce:RFGenerator:FREQuency",
                            channelNumber,
                            0, 13500,
                            RSCMU_ERROR_PARAMETER2));

  RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Generator Frequency                                             */
/* Purpose:  This function defines the frequency of the RF signal generated. */
/*           The frequency is set in specified units.                        */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_RfgFreq (ViSession instrumentHandle,
                                        ViUInt32 unit,
                                        ViReal64 frequency,
                                        ViChar _VI_FAR errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

  // Check if the function is correctly used:
    CHECK_VI_INT32_RANGE(instrumentHandle, unit, 0, 4, RSCMU_ERROR_PARAMETER2, errorMessage);

    switch (unit)
    {
        case RSCMU_HZ:
            CHECK_VI_REAL64_RANGE(instrumentHandle, frequency, 200000.0, 2700000000.0, RSCMU_ERROR_PARAMETER3, errorMessage);
        break;
        case RSCMU_KHZ:
            CHECK_VI_REAL64_RANGE(instrumentHandle, frequency, 200.0, 2700000.0, RSCMU_ERROR_PARAMETER3, errorMessage);
        break;
        case RSCMU_MHZ:
            CHECK_VI_REAL64_RANGE(instrumentHandle, frequency, 0.2, 2700.0, RSCMU_ERROR_PARAMETER3, errorMessage);
        break;
        case RSCMU_GHZ:
            CHECK_VI_REAL64_RANGE(instrumentHandle, frequency, 0.0002, 2.7, RSCMU_ERROR_PARAMETER3, errorMessage);
        break;
    }

    CHECKRETMSG(RSCMU_setEnum(instrumentHandle,
                                "UNIT:RFGenerator:FREQuency",
                              unit,
                              RSCMUNB_Unit_Frequency,
                              RSCMU_ERROR_PARAMETER2
                             )
                );

    CHECKRETMSG(RSCMU_setViReal64(instrumentHandle,
                            "SOURce:RFGenerator:FREQuency",
                                frequency,
                                0, 2700000000UL,
                                RSCMU_ERROR_PARAMETER3));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Generator Frequency Offset                                      */
/* Purpose:  This function defines an offset for the RF generator frequency. */
/*           The offset frequency must be in multiples of 1 Hz.              */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_RfgFoff (ViSession instrumentHandle, ViReal64 frequency,
                                ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    CHECKRETMSG(RSCMU_setViReal64(instrumentHandle,
                            "SOURce:RFGenerator:FOFFset",
                                frequency,
                                -100000, 100000,
                                RSCMU_ERROR_PARAMETER2));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Dedicated Channel Mode                                          */
/* Purpose:  This command defines whether the dedicated channel is configured*/
/*           as a uplink reference channel according to 3GPP TS 25.141 or    */
/*           as a physical channel.                                          */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_SourRfgDchMode (ViSession instrumentHandle,
                                       ViUInt32 dedicatedChannelMode,
                                       ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setEnum(instrumentHandle,
                              "SOURce:RFGenerator:DCHannel:MODE",
                              dedicatedChannelMode,
                              RSCMUNB_DedicatedChannel_Mode,
                              RSCMU_ERROR_PARAMETER2
                             )
                );
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Scrambling Code                                                 */
/* Purpose:  This function selects the (long or short) scrambling code for   */
/*           the uplink RF generator signal.                                 */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_SourRfgDchScodVal (ViSession instrumentHandle,
                                          ViUInt32 codeLength,
                                          ViUInt32 scramblingCode,
                                          ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    ViChar    scramblingCodeBuffer[10] = "";

    CHECKRETMSG(RSCMU_setEnum(instrumentHandle,
                              "SOURce:RFGenerator:DCHannel:SCODe:TYPE",
                              codeLength,
                              RSCMUNB_ScramblingCode_Length,
                              RSCMU_ERROR_PARAMETER2
                             )
                );
    sprintf(scramblingCodeBuffer, "\"%lX\"", scramblingCode);
    RETMSG(RSCMU_setViString(instrumentHandle,
                             "SOURce:RFGenerator:DCHannel:SCODe:VALue",
                             scramblingCodeBuffer,
                             RSCMU_ERROR_PARAMETER3
                            )
                );
}

/*===========================================================================*/
/* Function: BFN - CPICH Offset                                              */
/* Purpose:  This function defines the delay between the start of the CPICH  */
/*           in a cell relative to the BFN (T_cell).                         */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_SourRfgDchTcel (ViSession instrumentHandle,
                                       ViUInt32 BFNCPICHOffset,
                                       ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setViInt32(instrumentHandle,
                                "SOURce:RFGenerator:DCHannel:TCELl",
                                BFNCPICHOffset,
                                0, 149,
                                RSCMU_ERROR_PARAMETER2));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function:CPICH - DLDCH Offset                                             */
/* Purpose: This function defines the delay between the start of the DL DCH  */
/*          relative to the CPICH (chip offset).                             */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_SourRfgDchTdch (ViSession instrumentHandle,
                                       ViUInt32 CPICHDLDCHOffset,
                                       ViChar errorMessage[])
{

    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setViInt32(instrumentHandle,
                                "SOURce:RFGenerator:DCHannel:TDCHoffset",
                                CPICHDLDCHOffset,
                                0, 149,
                                RSCMU_ERROR_PARAMETER2));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Round Trip Delay                                                */
/* Purpose:  This function defines the round trip delay in the entire signal */
/*           path to fine-tune the generator timing.                         */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_SourRfgDchRtd (ViSession instrumentHandle,
                                      ViUInt32 roundTripDelay,
                                      ViChar errorMessage[])
{

    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setViInt32(instrumentHandle,
                                "SOURce:RFGenerator:DCHannel:RTDelay",
                                roundTripDelay,
                                0, 500,
                                RSCMU_ERROR_PARAMETER2));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Uplink Power Control                                            */
/* Purpose:  This function defines the power difference between two          */
/*           consecutive slots where the R&S CMU is commanded to change its  */
/*           output power and sets the algorithm for uplink power control.   */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_SourRfgDchUlpc (ViSession instrumentHandle, ViUInt32 mode,
                                       ViUInt32 TPCStepsize,
                                       ViChar errorMessage[])
{

    ViStatus  RSCMU_Status = VI_SUCCESS;
    CHECKRETMSG(RSCMU_setEnum(instrumentHandle,
                              "SOURce:RFGenerator:DCHannel:ULPControl:MODE",
                              mode,
                              RSCMUNB_PowerControl_Mode,
                              RSCMU_ERROR_PARAMETER2
                             ));

    CHECKRETMSG(RSCMU_setViInt32(instrumentHandle,
                                "SOURce:RFGenerator:DCHannel:ULPControl:TPCSsize",
                                TPCStepsize,
                                1, 2,
                                RSCMU_ERROR_PARAMETER3
                             ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: CPICH - UL DPCH Offset                                          */
/* Purpose:  This function defines the timing delay of the start of the UL   */
/*           DPCH relative to the CPICH.                                     */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_SourRfgDchUldp (ViSession instrumentHandle,
                                       ViReal64 CPICHULDPOffs,
                                       ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setViReal64(instrumentHandle,
                                "SOURce:RFGenerator:DCHannel:ULDPoffset",
                                CPICHULDPOffs,
                                0.0, 39424.0,
                                RSCMU_ERROR_PARAMETER2));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Reference Channel Type                                          */
/* Purpose:  This function sets the information bit rate of the 3GPP UL      */
/*           reference channel.                                              */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_SourRfgGpprCtyp (ViSession instrumentHandle,
                                                ViInt32 referenceChannelType,
                                                ViChar _VI_FAR errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    ViReal64  type = 0;
    switch (referenceChannelType)
    {
        case RSCMUNB_3GPPREFCHANNEL_TYPE_12_2:
             type = 12.2;
        break;
        case RSCMUNB_3GPPREFCHANNEL_TYPE_64:
             type = 64;
        break;
        case RSCMUNB_3GPPREFCHANNEL_TYPE_144:
             type = 144;
        break;
        case RSCMUNB_3GPPREFCHANNEL_TYPE_384:
             type = 384;
        break;
        case RSCMUNB_3GPPREFCHANNEL_TYPE_2048:
             type = 2048;
        break;
        default:
             RSCMU_Status = RSCMU_ERROR_PARAMETER2;

        break;
    }
    if (RSCMU_Status == VI_SUCCESS)
        CHECKRETMSG(RSCMU_setViReal64(instrumentHandle,
                                      "SOURce:RFGenerator:GPPReference:CTYPe",
                                       type,
                                       12.2, 2048,
                                       RSCMU_ERROR_PARAMETER2));

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Channel Data Source DTCH                                        */
/* Purpose:  This function defines the bit pattern transmitted on the        */
/*           Dedicated Traffic Channel part of the 3GPP UL reference channel.*/
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_SourRfgGpprCdsDtch (ViSession instrumentHandle,
                                           ViUInt32 channelDataSourceDTCH,
                                           ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    CHECKRETMSG(RSCMU_setEnum(instrumentHandle,
                              "SOURce:RFGenerator:GPPReference:CDSource:DTCH",
                              channelDataSourceDTCH,
                              RSCMUNB_ChannelDataSource_DTCH,
                              RSCMU_ERROR_PARAMETER2
                             ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: BER BLER Verification Mode                                      */
/* Purpose:  This function enables or disables the BER or BLER verification  */
/*           mode. The BER or BLER of the signal is defined separately.      */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_SourRfgGpprBbv (ViSession instrumentHandle,
                                              ViInt32 berBlerVerifMode,
                                              ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setEnum(instrumentHandle,
                              "SOURCe:RFGenerator:GPPReference:BBVermode",
                              berBlerVerifMode,
                              RSCMUNB_BerBlerVerifMode,
                              RSCMU_ERROR_PARAMETER2
                             ));
    RETMSG(RSCMU_Status);

}

/*===========================================================================*/
/* Function: BER BLER Value                                                  */
/* Purpose:  This function defines the Bit Error Rate (BER) or Block Error   */
/*           Rate (BLER) of the uplink signal that the CMU generates if the  */
/*           BER or BLER verification mode is enabled.                       */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_SourRfgGpprBerBlerV (ViSession instrumentHandle,
                                          ViInt32 verifMode,
                                          ViReal64 value,
                                          ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECK_VI_INT32_RANGE(instrumentHandle, verifMode, 1, 2, RSCMU_ERROR_PARAMETER2, errorMessage);

    if (verifMode == RSCMUNB_3GPP_VERIFICATION_BER)
    {
        CHECKRETMSG(RSCMU_setViReal64(instrumentHandle,
                            "SOURCe:RFGenerator:GPPReference:BERValue",
                            value,
                            0.0, 100.0,
                            RSCMU_ERROR_PARAMETER3));
    }
    else
    {
            CHECKRETMSG(RSCMU_setViReal64(instrumentHandle,
                            "SOURCe:RFGenerator:GPPReference:BLERvalue",
                            value,
                            0.0, 100.0,
                            RSCMU_ERROR_PARAMETER3));
    }
RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: TFCI Bits                                                       */
/* Purpose:  This function defines the Transport Format Combination Indicator*/
/*           (TFCI) bits to be transmitted on the 3GPP UL reference channel. */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_SourRfgGpprTfci (ViSession instrumentHandle,
                                        ViUInt32 TFCIBits,
                                        ViChar errorMessage[])
{

    ViStatus  RSCMU_Status = VI_SUCCESS;
    ViChar    TFCIBitsBuffer[35] = "";

    RSCMU_Status = RSCMU_Dec2Bin(TFCIBits, TFCIBitsBuffer, "\"");
    RETMSG(RSCMU_setViString(instrumentHandle,
                             "SOURce:RFGenerator:GPPReference:TFCIbits",
                             TFCIBitsBuffer,
                             RSCMU_ERROR_PARAMETER2
                            ));
}

/*===========================================================================*/
/* Function: Get Power Ratio                                                 */
/* Purpose:  This function returns the DPCCH/DPDCH power ratio for 3GPP      */
/*           reference measurement channels in the uplink WCDMA signal. This */
/*           ratio is as specified in the standard for each reference        */
/*           channel.                                                        */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_SourRfgLevRcdGppr (ViSession instrumentHandle,
                                         ViReal64 *powerRatio,
                                         ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_getViReal64(instrumentHandle,
                                "SOURce:RFGenerator:LEVel:RCD:GPPReference?",
                                powerRatio,
                                RSCMU_ERROR_PARAMETER2));

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: TFCI Code Word                                                  */
/* Purpose:  This function defines the Transport Format Combination Indicator*/
/*           (TFCI) code word to be transmitted on the uplink physical       */
/*           channel.                                                        */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_SourRfgPchDpccTfciCwor (ViSession instrumentHandle,
                                               ViUInt32 TFCICodeWord,
                                               ViChar errorMessage[])
{
    ViChar    TFCICodeWordBuffer[11] = "";

    sprintf(TFCICodeWordBuffer, "\"%lX\"", TFCICodeWord);
    RETMSG(RSCMU_setViString(instrumentHandle,
                             "SOURce:RFGenerator:PCHannel:DPCCh:TFCI:CWORd",
                             TFCICodeWordBuffer,
                             RSCMU_ERROR_PARAMETER2
                            )
                );

}

/*===========================================================================*/
/* Function: Downlink Power Control Mode                                     */
/* Purpose:  This function defines the downlink power control mode.          */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_SourRfgPchDpccTpcDpcm (ViSession instrumentHandle,
                                              ViUInt32 downlinkPowerControlMode,
                                              ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setEnum(instrumentHandle,
                              "SOURce:RFGenerator:PCHannel:DPCCh:TPC:DPCMode",
                              downlinkPowerControlMode,
                              RSCMUNB_DownlinkPowerControl_Mode,
                              RSCMU_ERROR_PARAMETER2
                             ));
    RETMSG(RSCMU_Status);

}

/*===========================================================================*/
/* Function: Read Out Mode                                                   */
/* Purpose:  This function defines the rule for constructing a TPC pattern of*/
/*           infinite length to be transferred to Node B. The actual pattern */
/*           transferred depends on the finite single pattern defined via    */
/*           SOURce:RFGenerator:PCHannel:DPCCh:TPC:PATTern command (see      */
/*           Pattern (RSCMUNB_Nsig_SourRfgPchDpccTpcPatt) function) and the */
/*           read out mode.                                                  */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_SourRfgPchDpccTpcRom (ViSession instrumentHandle,
                                             ViUInt32 readOutMode,
                                             ViChar errorMessage[])
{

    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setEnum(instrumentHandle,
                              "SOURce:RFGenerator:PCHannel:DPCCh:TPC:ROMode",
                              readOutMode,
                              RSCMUNB_PCFieldSettings_ReadOutMode,
                              RSCMU_ERROR_PARAMETER2
                             ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Pattern                                                         */
/* Purpose:  This function defines the single TPC bit pattern transmitted on */
/*           the uplink physical channel.                                    */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_SourRfgPchDpccTpcPatt (ViSession instrumentHandle,
                                              ViUInt32 patternLength,
                                              ViUInt32 pattern,
                                              ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    ViChar    patternBuffer[35] = "";

    CHECKRETMSG(RSCMU_setViInt32(instrumentHandle,
                                "SOURce:RFGenerator:PCHannel:DPCCh:TPC:PLENgth",
                                patternLength,
                                0, 32,
                                RSCMU_ERROR_PARAMETER2));

    RSCMU_Status = RSCMU_Dec2Bin(pattern, patternBuffer, "\"");
    RETMSG(RSCMU_setViString(instrumentHandle,
                             "SOURce:RFGenerator:PCHannel:DPCCh:TPC:PATTern",
                             patternBuffer,
                             RSCMU_ERROR_PARAMETER3
                            ));

}

/*===========================================================================*/
/* Function: Channel Data Source DPDCH                                       */
/* Purpose:  This function defines the bit pattern transmitted on the        */
/*           downlink physical channel.                                      */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_SourRfgPchDpdcCds (ViSession instrumentHandle,
                                          ViUInt32 channelDataSourceDPDCH,
                                          ViChar errorMessage[])
{

    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setEnum(instrumentHandle,
                              "SOURce:RFGenerator:PCHannel:DPDCh:CDSource",
                              channelDataSourceDPDCH,
                              RSCMUNB_ChannelDataSource_DTCH,   //same range for CCH and DCH
                              RSCMU_ERROR_PARAMETER2
                             ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Symbol Rate                                                     */
/* Purpose:  This function defines the symbol rate and number of DPDCHs      */
/*           determining the properties of the UL physical channel. The      */
/*           default symbol rate is equal to the symbol rate transmitted on  */
/*           the DPDCH of a 12.2 kbps 3GPP UL reference channel.             */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_SourRfgPchDpdcSrat (ViSession instrumentHandle,
                                           ViUInt32 symbolRate,
                                           ViChar errorMessage[])
{

    ViStatus  RSCMU_Status = VI_SUCCESS;
    ViReal64 rate = 0;

    switch (symbolRate)
    {
        case RSCMUNB_DPDCHTOTALSYMOBLRATE_15:
             rate = 15;
        break;
        case RSCMUNB_DPDCHTOTALSYMOBLRATE_30:
             rate = 30;
        break;
        case RSCMUNB_DPDCHTOTALSYMOBLRATE_60:
             rate = 60;
        break;
        case RSCMUNB_DPDCHTOTALSYMOBLRATE_120:
             rate = 120;
        break;
        case RSCMUNB_DPDCHTOTALSYMOBLRATE_240:
             rate = 240;
        break;
        case RSCMUNB_DPDCHTOTALSYMOBLRATE_480:
             rate = 480;
        break;
        case RSCMUNB_DPDCHTOTALSYMOBLRATE_1_960:
             rate = 960;
        break;
        case RSCMUNB_DPDCHTOTALSYMOBLRATE_2_960:
             rate = 1920;
        break;
        case RSCMUNB_DPDCHTOTALSYMOBLRATE_3_960:
             rate = 2880;
        break;
        case RSCMUNB_DPDCHTOTALSYMOBLRATE_4_960:
             rate = 3840;
        break;
        case RSCMUNB_DPDCHTOTALSYMOBLRATE_5_960:
             rate = 4800;
        break;
        case RSCMUNB_DPDCHTOTALSYMOBLRATE_6_960:
             rate = 5760;
        break;
        default:
             RSCMU_Status = RSCMU_ERROR_PARAMETER2;
             RETMSG(RSCMU_Status);
    }
    CHECKRETMSG(RSCMU_setViReal64(instrumentHandle,
                            "SOURce:RFGenerator:PCHannel:DPDCh:TOTal:SRATe",
                                rate,
                                15, 5760,
                                RSCMU_ERROR_PARAMETER2));

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Init                                                            */
/* Purpose:  This function starts the RF generator, setting it to the RUN    */
/*           status.                                                         */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_RfgInit (ViSession instrumentHandle,
                                       ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    ViChar      wrtBuf[WR_BUFFER_SIZE];

    strcpy(wrtBuf,"INITiate:RFGenerator");
    CHECKRETMSG(RSCMU_sendCmd(instrumentHandle, wrtBuf));

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Abort                                                           */
/* Purpose:  This function stops the RF generator, setting it to the OFF     */
/*           status.                                                         */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_RfgAbor (ViSession instrumentHandle,
                                ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    ViChar      wrtBuf[WR_BUFFER_SIZE];

    strcpy(wrtBuf,"ABORt:RFGenerator");
    CHECKRETMSG(RSCMU_sendCmd(instrumentHandle, wrtBuf));

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Get Generator Status                                            */
/* Purpose:  This function returns the current generator status.             */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_FetcRfgStat (ViSession instrumentHandle,
                                    ViInt32 *generatorStatus,
                                    ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_getEnum(instrumentHandle,
                              "FETCh:RFGenerator:STATus?",
                              generatorStatus,
                              RSCMUNB_MeasStatus,
                              RSCMU_ERROR_INSTRUMENT_ERROR
                             ));

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Antenna Diversity                                               */
/* Purpose:  This function defines the antenna diversity of the Node B.      */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_ConfNbAdiv (ViSession instrumentHandle,
                                   ViUInt32 antenna_diversity,
                                   ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    CHECKRETMSG(RSCMU_setEnum(instrumentHandle,
                              "CONFigure:NB:ADIVersity",
                              antenna_diversity,
                              RSCMUNB_AntennaDiversity,
                              RSCMU_ERROR_PARAMETER2
                             ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Scrambling Code                                                 */
/* Purpose:  This function defines the primary and secondary scrambling codes*/
/*           of the Node B signal. The range of scrambling code numbers      */
/*           corresponds to 0 to 511 decimal.                                */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_ConfNbDchScod (ViSession instrumentHandle,
                                      ViUInt32 primaryScramblingCode,
                                      ViUInt32 secondaryScramblingCode,
                                      ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    ViChar    scramblingCodeBuffer[10] = "";

    sprintf(scramblingCodeBuffer, "\"%lX\"", primaryScramblingCode);
    CHECKRETMSG(RSCMU_setViString(instrumentHandle,
                             "CONFigure:NB:DCHannel:SCODe:PRIMary",
                             scramblingCodeBuffer,
                             RSCMU_ERROR_PARAMETER2
                            )

                );
                
    sprintf(scramblingCodeBuffer, "\"%lX\"", secondaryScramblingCode);
    RETMSG(RSCMU_setViString(instrumentHandle,
                             "CONFigure:NB:DCHannel:SCODe:SECondary",
                             scramblingCodeBuffer,
                             RSCMU_ERROR_PARAMETER3
                            )
                );
}

/*===========================================================================*/
/* Function: Input  Connector                                                */
/* Purpose:  This function determines the connector to be used for RF input  */
/*           signals. The bidirectional connectors RF 1 and RF 2 can be used */
/*           both as input and output connectors in the same measurement.    */
/*           Only one input and one output may be active at the same time, a */
/*           new RF input setting supersedes the previous one.               */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_InpStat (ViSession instrumentHandle,
                                ViUInt32 inputConnector, ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    CHECKRETMSG(RSCMU_setEnum(instrumentHandle,
                              "INPut:STATe",
                              inputConnector,
                              RSCMUNB_Connector_Input,
                              RSCMU_ERROR_PARAMETER2
                             ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Input  Attenuation                                              */
/* Purpose:  This function assigns an external attenuation value to the      */
/*           inputs of the instrument (RF 1, RF 2, RF 4 IN).                 */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_SensCorrLossInpMagn (ViSession instrumentHandle,
                                            ViUInt32 inputConnector,
                                            ViReal64 inputAttenuation,
                                            ViChar errorMessage[])
{
    ViStatus  RSCMU_Status      = VI_SUCCESS;
    ViChar    commandBuffer[40] = "";
    ViInt32   connector         = 0;

    switch (inputConnector)
    {
        case RSCMU_INPUT_RF1:
             connector = 1;
        break;
        case RSCMU_INPUT_RF2:
             connector = 2;
        break;
        case RSCMU_INPUT_RF4:
             connector = 4;
        break;
        default:
             RSCMU_Status = RSCMU_ERROR_PARAMETER2;

        break;
    }
    if (RSCMU_Status ==  VI_SUCCESS)
        sprintf(commandBuffer, "SENSe:CORRection:LOSS:INPut%ld:MAGNitude", connector);
    else
        RETMSG(RSCMU_Status);

    CHECKRETMSG(RSCMU_setViReal64(instrumentHandle,
                                  commandBuffer,
                                  inputAttenuation,
                                  -50, 90,
                                RSCMU_ERROR_PARAMETER3));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Output  Connector                                               */
/* Purpose:  This function determines the connector to be used for RF output */
/*           signals. The bidirectional connectors RF 1 and RF 2 can be used */
/*           as input and output connectors in the same measurement.         */
/*           Only one input and one output may be active at the same time, a */
/*           new RF output setting supersedes the previous one.              */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_OutpStat (ViSession instrumentHandle,
                                 ViUInt32 outputConnector, ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    CHECKRETMSG(RSCMU_setEnum(instrumentHandle,
                              "OUTPut:STATe",
                              outputConnector,
                              RSCMUNB_Connector_Output,
                              RSCMU_ERROR_PARAMETER2
                             ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Output  Attenuation                                             */
/* Purpose:  This function assigns an external attenuation value to the      */
/*           outputs of the instrument (RF 1, RF 2, RF 3 OUT).               */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_SourCorrLossOutpMagn (ViSession instrumentHandle,
                                             ViUInt32 outputConnector,
                                             ViReal64 outputAttenuation,
                                             ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    ViChar    commandBuffer[42] = "";
    ViInt32   connector         = 0;

    switch (outputConnector)
    {
        case RSCMU_OUTPUT_RF1:
             connector = 1;
        break;
        case RSCMU_OUTPUT_RF2:
             connector = 2;
        break;
        case RSCMU_OUTPUT_RF3:
             connector = 3;
        break;
        default:
             RSCMU_Status = RSCMU_ERROR_PARAMETER2;

        break;
    }
    if (RSCMU_Status ==  VI_SUCCESS)
        sprintf(commandBuffer, "SOURce:CORRection:LOSS:OUTPut%ld:MAGNitude", connector);
    else
        RETMSG(RSCMU_Status);

    CHECKRETMSG(RSCMU_setViReal64(instrumentHandle,
                                  commandBuffer,
                                  outputAttenuation,
                                  -50, 50,
                                RSCMU_ERROR_PARAMETER3));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Ref Out 2                                                       */
/* Purpose:  This function determines the system clock frequency applied to  */
/*           REF OUT 2 and turns it on or off.                               */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_SourDmCloc (ViSession instrumentHandle,
                                   ViUInt32 refOut2Mode,
                                   ViReal64 referenceFrequency,
                                   ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    ViReal64  f0                = 30.72;
    ViReal64  n             = 0;

    n = (f0/referenceFrequency)-1;
    if ((n != (int)n) &&  (n <= 32))
        RETMSG(RSCMU_ERROR_PARAMETER3);

    CHECKRETMSG(RSCMU_setViReal64(instrumentHandle,
                                  "SOURCe:DM:CLOCk:FREQuency",
                                  referenceFrequency,
                                  0.960, 30.72,
                                RSCMU_ERROR_PARAMETER3));

    CHECKRETMSG(RSCMU_setEnum(instrumentHandle,
                              "SOURce:DM:CLOCk:STATe",
                              refOut2Mode,
                              RSCMUNB_mode_On_Off,
                              RSCMU_ERROR_PARAMETER2
                             ));

    RETMSG(RSCMU_Status);

}

/*===========================================================================*/
/* Function: Trigger Source                                                  */
/* Purpose:  This function determines the trigger source.                    */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_ConfMeasTrigSour (ViSession instrumentHandle,
                                ViInt32 triggerSource, ViChar errorMessage[])
{

    ViStatus  RSCMU_Status = VI_SUCCESS;
    CHECKRETMSG(RSCMU_setEnum(instrumentHandle,
                              "CONFigure:MEAS:TRIGger:SOURce",
                              triggerSource,
                              RSCMUNB_Trigger_Source,
                              RSCMU_ERROR_PARAMETER2
                             ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Polarity                                                        */
/* Purpose:  This function defines the polarity of the trigger signal.       */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_ConfMeasTrigPol (ViSession instrumentHandle,
                                           ViInt32 polarity,
                                           ViChar errorMessage[])
{

    ViStatus  RSCMU_Status = VI_SUCCESS;
    CHECKRETMSG(RSCMU_setEnum(instrumentHandle,
                              "CONFigure:MEAS:TRIGger:POLarity",
                              polarity,
                              RSCMUNB_WiredSync_Polarity,
                              RSCMU_ERROR_PARAMETER2
                             ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Slot Offset                                                     */
/* Purpose:  This function defines an integer number of slots between the    */
/*           beginning of the Node B Transmission Time Interval (TTI) and    */
/*           the trigger time. The slot offset is added to the chip delay.   */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_ConfMeasTrigSoff (ViSession instrumentHandle,
                                            ViInt32 slotOffset,
                                            ViChar errorMessage[])
{

    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setViInt32(instrumentHandle,
                                  "CONFigure:MEAS:TRIGger:SOFFset",
                                  slotOffset,
                                  0, 14,
                                RSCMU_ERROR_PARAMETER2));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Chip Delay                                                      */
/* Purpose:  This function defines an integer number of chips between the    */
/*           beginning of the Node B Transmission Time Interval (TTI) and    */
/*           the trigger time. The chip delay is added to the slot offset.   */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_ConfMeasTrigCdel (ViSession instrumentHandle,
                                            ViInt32 chipDelay,
                                            ViChar errorMessage[])
{

    ViStatus  RSCMU_Status = VI_SUCCESS;
    CHECKRETMSG(RSCMU_setViInt32(instrumentHandle,
                                  "CONFigure:MEAS:TRIGger:CDELay",
                                  chipDelay,
                                  0, 10240,
                                  RSCMU_ERROR_PARAMETER2
                                  ));
    RETMSG(RSCMU_Status);
}

/****/

// TRIGger[:SEQuence]:SOURce
// CONFigure:EXTernal:TRIGger:SYNC:POLarity
// CONFigure:EXTernal:TRIGger:SYNC:SOFFset
// CONFigure:EXTernal:TRIGger:SYNC:CDELay

/*===========================================================================*/
/* Function: Trigger Source                                                  */
/* Purpose:  This function determines the trigger source.                    */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_TrigSeqSour (ViSession instrumentHandle,
                                    ViInt32 triggerSource, ViChar errorMessage[])
{

    ViStatus  RSCMU_Status = VI_SUCCESS;
    CHECKRETMSG(RSCMU_setEnum(instrumentHandle,
                              "TRIGger:SEQuence:SOURce",
                              triggerSource,
                              RSCMUNB_Trigger_Source,
                              RSCMU_ERROR_PARAMETER2
                             ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Polarity                                                        */
/* Purpose:  This function defines the polarity of the trigger signal.       */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_ConfExtTrigSyncPol (ViSession instrumentHandle,
                                           ViInt32 polarity,
                                           ViChar errorMessage[])
{

    ViStatus  RSCMU_Status = VI_SUCCESS;
    CHECKRETMSG(RSCMU_setEnum(instrumentHandle,
                              "CONFigure:EXTernal:TRIGger:SYNC:POLarity",
                              polarity,
                              RSCMUNB_WiredSync_Polarity,
                              RSCMU_ERROR_PARAMETER2
                             ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Slot Offset                                                     */
/* Purpose:  This function defines an integer number of slots between the    */
/*           beginning of the Node B Transmission Time Interval (TTI) and    */
/*           the trigger time. The slot offset is added to the chip delay.   */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_ConfExtTrigSyncSoff (ViSession instrumentHandle,
                                            ViInt32 slotOffset,
                                            ViChar errorMessage[])
{

    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setViInt32(instrumentHandle,
                                  "CONFigure:EXTernal:TRIGger:SYNC:SOFFset",
                                  slotOffset,
                                  0, 61440,
                                RSCMU_ERROR_PARAMETER2));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Chip Delay                                                      */
/* Purpose:  This function defines an integer number of chips between the    */
/*           beginning of the Node B Transmission Time Interval (TTI) and    */
/*           the trigger time. The chip delay is added to the slot offset.   */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_ConfExtTrigSyncCdel (ViSession instrumentHandle,
                                            ViInt32 chipDelay,
                                            ViChar errorMessage[])
{

    ViStatus  RSCMU_Status = VI_SUCCESS;
    CHECKRETMSG(RSCMU_setViInt32(instrumentHandle,
                                  "CONFigure:EXTernal:TRIGger:SYNC:CDELay",
                                  chipDelay,
                                  0, 10240,
                                  RSCMU_ERROR_PARAMETER2
                                  ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Expected Power                                                  */
/* Purpose:  This function defines the maximum expected input level. The     */
/*           value range depends on the RF input used and the external       */
/*           attenuation set. Some measurements are restricted to a narrower */
/*           input level range. See data sheet.                              */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_EpowVal (ViSession instrumentHandle,
                                ViReal64 expectedPower,
                                ViChar errorMessage[])
{

    ViStatus  RSCMU_Status = VI_SUCCESS;
    CHECKRETMSG(RSCMU_setViReal64(instrumentHandle,
                                  "SENSe:EPOWer:VALue",
                                  expectedPower,
                                  -79.0, +53.0,
                                RSCMU_ERROR_PARAMETER2));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Expected Power / Mode                                           */
/* Purpose:  This function defines how the RF analyzer is adapted to the     */
/*           expected input power.                                           */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_EpowMode (ViSession instrumentHandle,
                                 ViInt32 expectedPowerMode,
                                 ViChar errorMessage[])
{

    ViStatus  RSCMU_Status = VI_SUCCESS;
    CHECKRETMSG(RSCMU_setEnum(instrumentHandle,
                              "SENSe:EPOWer:MODE",
                              expectedPowerMode,
                              RSCMUNB_ExpectedPower_Mode,
                              RSCMU_ERROR_PARAMETER2));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Expected Power / Attenuation                                    */
/* Purpose:  This function tunes the RF analyzer for normal setting, low     */
/*           noise level (full dynamic range), or low distortion (high       */
/*           intermodulation spacing).                                       */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_EpowAtt (ViSession instrumentHandle,
                                ViInt32 expectedPowerAttenuation,
                                ViChar errorMessage[])
{

    ViStatus  RSCMU_Status = VI_SUCCESS;
    CHECKRETMSG(RSCMU_setEnum(instrumentHandle,
                              "SENSe:EPOWer:ATTenuation",
                              expectedPowerAttenuation,
                              RSCMUNB_ExpectedPower_Attenuation,
                              RSCMU_ERROR_PARAMETER2
                             ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Default Settings                                                */
/* Purpose:  This function sets all parameters of the subsystem to default   */
/*           values (the setting OFF results in an error message).           */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_DefEpow (ViSession instrumentHandle, ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_sendCmd(instrumentHandle,
                                  "DEFault:EPOWer ON"));

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/*===========================================================================*/
/* Code Domain Power Measurements                                            */
/*===========================================================================*/
/*===========================================================================*/

/*===========================================================================*/
/* Function: CDP Test Model                                                  */
/* Purpose:  This function selects the test model for all CDPower            */
/*           applications.                                                   */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_ConfCdpTmod (ViSession instrumentHandle,
                                            ViInt32 testModel,
                                            ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setEnum(instrumentHandle,
                              "CONFigure:CDPower:TMODel",
                              testModel,
                              RSCMUNB_TestModel,
                              RSCMU_ERROR_PARAMETER2
                             ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Set and Enable Limits                                           */
/* Purpose:  This function enables and defines upper limits for the scalar   */
/*           results of the Code Domain Power measurement.                   */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_ConfCdpLimScal (ViSession instrumentHandle,
                                       ViInt32   statisticalType,
                                       ViInt32   iqOriginOffsetEnable,
                                       ViReal64  iqOriginOffset,
                                       ViInt32   iqImbalanceEnable,
                                       ViReal64  iqImbalance,
                                       ViInt32   cfeEnable,
                                       ViReal64  cfe,
                                       ViInt32   evmEnable,
                                       ViReal64  evm,
                                       ViInt32   pcdeEnable,
                                       ViReal64  pcde,
                                       ViChar    errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    int i = 0;
    ViChar cmdEnable[5][100];
    ViChar cmdValue[5][100];

    for (i=0;i<5;i++)
    {
        if (i==2)
        {
            sprintf(cmdEnable[i],"CONFigure:CDPower:%s:%s:LIMit:SCALar:SYMMetric:Enable",RSCMUNB_CDP_ScalarResults[i],RSCMUNB_statType[statisticalType]);
            sprintf(cmdValue[i],"CONFigure:CDPower:%s:%s:LIMit:SCALar:SYMMetric:VALue",RSCMUNB_CDP_ScalarResults[i],RSCMUNB_statType[statisticalType]);
        }
        else
        {
            sprintf(cmdEnable[i],"CONFigure:CDPower:%s:%s:LIMit:SCALar:ASYMmetric:UPPer:Enable",RSCMUNB_CDP_ScalarResults[i],RSCMUNB_statType[statisticalType]);
            sprintf(cmdValue[i],"CONFigure:CDPower:%s:%s:LIMit:SCALar:ASYMmetric:UPPer:VALue",RSCMUNB_CDP_ScalarResults[i],RSCMUNB_statType[statisticalType]);
        }
    }

    CHECKRETMSG(RSCMU_setEnum    (instrumentHandle, cmdEnable[0], iqOriginOffsetEnable, RSCMUNB_mode_On_Off,RSCMU_ERROR_PARAMETER3));
    CHECKRETMSG(RSCMU_setViReal64(instrumentHandle, cmdValue[0],  iqOriginOffset, -80, 0,    RSCMU_ERROR_PARAMETER4));

    CHECKRETMSG(RSCMU_setEnum    (instrumentHandle, cmdEnable[1], iqImbalanceEnable, RSCMUNB_mode_On_Off,RSCMU_ERROR_PARAMETER5));
    CHECKRETMSG(RSCMU_setViReal64(instrumentHandle, cmdValue[1],  iqImbalance, -80, 0,    RSCMU_ERROR_PARAMETER6));

    CHECKRETMSG(RSCMU_setEnum    (instrumentHandle, cmdEnable[2], cfeEnable, RSCMUNB_mode_On_Off,RSCMU_ERROR_PARAMETER7));
    CHECKRETMSG(RSCMU_setViReal64(instrumentHandle, cmdValue[2],  cfe, -80, 4000,    RSCMU_ERROR_PARAMETER8));

    CHECKRETMSG(RSCMU_setEnum    (instrumentHandle, cmdEnable[3], evmEnable, RSCMUNB_mode_On_Off,RSCMU_ERROR_PARAMETER9));
    CHECKRETMSG(RSCMU_setViReal64(instrumentHandle, cmdValue[3],  evm, 0, 100,    RSCMU_ERROR_PARAMETER10));

    CHECKRETMSG(RSCMU_setEnum    (instrumentHandle, cmdEnable[4], pcdeEnable, RSCMUNB_mode_On_Off,RSCMU_ERROR_PARAMETER11));
    CHECKRETMSG(RSCMU_setViReal64(instrumentHandle, cmdValue[4],  pcde, -80, 0,    RSCMU_ERROR_PARAMETER12));

    RETMSG(RSCMU_Status);

}

/*===========================================================================*/
/* Function: CDP Result Mode                                                 */
/* Purpose:  This function specifies the type of measured values.            */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_ConfCdpCdpwContRmod (ViSession instrumentHandle,
                                            ViInt32 resultMode,
                                            ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setEnum(instrumentHandle,
                              "CONFigure:CDPower:CDPower:CONTrol:RMODe",
                              resultMode,
                              RSCMUNB_Measurement_Result_Mode,
                              RSCMU_ERROR_PARAMETER2
                             ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: CDP Statistical Depth                                           */
/* Purpose:  This function defines the number of slots forming a statistics  */
/*           cycle.                                                          */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_ConfCdpCdpwContStat (ViSession instrumentHandle,
                                            ViInt32 statisticalDepth,
                                            ViChar errorMessage[])
{

    ViStatus  RSCMU_Status = VI_SUCCESS;
    CHECKRETMSG(RSCMU_setViInt32(instrumentHandle,
                                  "CONFigure:CDPower:CDPower:CONTrol:STATistics",
                                  statisticalDepth,
                                  1, 1000,
                                  RSCMU_ERROR_PARAMETER2
                                  ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: CDP Repetition                                                  */
/* Purpose:  This function determines the number of statistics cycles, the   */
/*           stop condition and the stepping mode for the measurement.       */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_ConfCdpCdpwContRep (ViSession instrumentHandle,
                                           ViInt32 repetitionMode,
                                           ViInt32 stopCondition,
                                           ViChar errorMessage[])
{

    ViStatus  RSCMU_Status = VI_SUCCESS;
    ViChar    value[42] = "";

    if (RSCMU_invalidViInt32Range (repetitionMode, 0, 1))
        RETMSG(RSCMU_ERROR_PARAMETER2);

    if (RSCMU_invalidViInt32Range (stopCondition, 0, 1))
        RETMSG(RSCMU_ERROR_PARAMETER3);

    sprintf(value, "%s, %s, NONE", RSCMUNB_Repetition[repetitionMode], RSCMUNB_StopCondition[stopCondition]);
    CHECKRETMSG(RSCMU_setViString(instrumentHandle,
                                  "CONFigure:CDPower:CDPower:CONTrol:REPetition",
                                  value,
                                  RSCMU_ERROR_PARAMETER2
                                  ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: CDP Event Reporting                                             */
/* Purpose:  This function defines the events generated when the measurement */
/*           is terminated or stopped.                                       */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_ConfCdpCdpwContErep (ViSession instrumentHandle,
                                            ViInt32 eventReporting,
                                            ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setEnum(instrumentHandle,
                              "CONFigure:CDPower:CDPower:EREPorting",
                              eventReporting,
                              RSCMUNB_EventReport,
                              RSCMU_ERROR_PARAMETER2
                             ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: CDP Mode                                                        */
/* Purpose:  This function qualifies whether the Code Domain Power is        */
/*           measured with or without the SCH contributions.                 */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_ConfCdpCdpwContCdpm (ViSession instrumentHandle,
                                            ViInt32 CDPMode,
                                            ViChar errorMessage[])
{

    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setEnum(instrumentHandle,
                              "CONFigure:CDPower:CDPower:CONTrol:CDPMode",
                              CDPMode,
                              RSCMUNB_CDP_Mode,
                              RSCMU_ERROR_PARAMETER2
                             ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Unit                                                            */
/* Purpose:  This command selects the display unit for the code domain power */
/*           results retrieved via Get Array CDP Measurement Results         */
/*           (RSCMUNB_Nsig_GetArrCdpCdpChanResults) etc. functions          */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_ConfCdpCdpwContUnit (ViSession instrumentHandle,
                                            ViInt32 unit, ViChar errorMessage[])
{

    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setEnum(instrumentHandle,
                              "CONFigure:CDPower:CDPower:CONTrol:UNIT",
                              unit,
                              RSCMUNB_CDP_Unit,
                              RSCMU_ERROR_PARAMETER2
                             ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Code Channel                                                    */
/* Purpose:  This command selects the code channel for the power vs. slot    */
/*           evaluation. The power vs. slot results obtained with the        */
/*           selected code channel can be retrieved by means of the Get Array*/
/*           of CDP Measurement Results (Power / Channels)                   */
/*           (RSCMUNB_Nsig_GetArrCdpCdpChanResults) function.               */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_ConfCdpCdpwContCch (ViSession instrumentHandle,
                                           ViInt32 codeChannel,
                                           ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    CHECKRETMSG(RSCMU_setViInt32(instrumentHandle,
                                  "CONFigure:CDPower:CDPower:CONTrol:CCHannel",
                                  codeChannel,
                                  0, 511,
                                  RSCMU_ERROR_PARAMETER2
                                  ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: CDP CPICH Slot                                                  */
/* Purpose:  This command selects the CPICH slot no. for the power vs. code  */
/*           evaluation. The power vs. code results obtained with the        */
/*           selected CPICH slot can be retrieved by means of the Get Array  */
/*           of CDP Measurement Results (Power / Slots)                      */
/*           (RSCMUNB_Nsig_GetArrCdpCdpSlotResults) function.               */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_ConfCdpCdpwContCpic (ViSession instrumentHandle,
                                            ViInt32 CPICHSlot,
                                            ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    CHECKRETMSG(RSCMU_setViInt32(instrumentHandle,
                                  "CONFigure:CDPower:CDPower:CONTrol:CPIChslot",
                                  CPICHSlot,
                                  0, 14,
                                  RSCMU_ERROR_PARAMETER2
                                  ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Initiate CDP Measurement                                        */
/* Purpose:  This function starts the code domain power measurement.         */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_InitCdp (ViSession instrumentHandle,
                                ViChar errorMessage[])
{

    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setViString(instrumentHandle,
                                  "INITiate:CDPower:CDPower",
                                  "",
                                  RSCMUNB_ERROR_INITIATE_FAILED
                                  ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Abort CDP Measurement                                           */
/* Purpose:  This function aborts running code domain power measurement and  */
/*           switches off.                                                   */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_AborCdp (ViSession instrumentHandle,
                                ViChar errorMessage[])
{

    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setViString(instrumentHandle,
                                  "ABORt:CDPower:CDPower",
                                  "",
                                  RSCMUNB_ERROR_ABORT_FAILED
                                  ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Stop CDP Measurement                                            */
/* Purpose:  This function stops the code domain power measurement after     */
/*           current stat. cycle.                                            */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_StopCdp (ViSession instrumentHandle,
                                ViChar errorMessage[])
{

    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setViString(instrumentHandle,
                                  "STOP:CDPower:CDPower",
                                  "",
                                  RSCMUNB_ERROR_STOP_FAILED
                                  ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Continue CDP Measurement                                        */
/* Purpose:  This function initiates the next measurement step (only         */
/*           stepping mode) of the code domain power measurement.            */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_ContCdp (ViSession instrumentHandle,
                                ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setViString(instrumentHandle,
                                  "CONTinue:CDPower:CDPower",
                                  "",
                                  RSCMUNB_ERROR_CONTINUE_FAILED
                                  ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Get CDP Measurement Status                                      */
/* Purpose:  This function returns the status of the measurement.            */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_FetcScalCdpCdpStat (ViSession instrumentHandle,
                                            ViInt32 *measurementStatus,
                                            ViInt32 *currentStatistic,
                                            ViChar errorMessage[])
{
    ViStatus  RSCMU_Status    = VI_SUCCESS;
    ViChar   tableOfDataValues[100][100];

    GET_RD_BUF( instrumentHandle );
    CHECKRETMSG(RSCMU_getViString(instrumentHandle,
                                  "FETCh:CDPower:CDPower:STATus?",
                                  rdBuf,
                                  RD_BUFFER_SIZE,
                                  0
                                  ));

    // Split the multiple data values separated by a comma:
    RSCMU_SeparateDataValues (rdBuf, ",", tableOfDataValues);

    // Get the status of the measurement:
    if ((RSCMU_Status = RSCMU_GetMeasStatVal (measurementStatus, tableOfDataValues, 0)) < 0)
        RETMSG(RSCMU_Status);

    // Counting Mode is not supported by this driver
    // // Get the current value of the counting mode counter (the number of the current statistics cycle):
    // if ((RSCMU_Status = RSCMU_GetCountingNo (CountingNo, tableOfDataValues, 1)) < 0)
    //      RETMSG(RSCMU_Status);

    // Get the current evaluation period within a statistics cycle:
    if ((RSCMU_Status = RSCMU_GetStatisticNo (currentStatistic, tableOfDataValues, 2)) < 0)
        RETMSG(RSCMU_Status);

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Get Scalar CDP Measurement Results                              */
/* Purpose:  This function starts the Code Domain Power measurement and/or   */
/*           returns all scalar measurement results and the timing offset    */
/*           relative to the P-CPICH of the code channel selected by Code    */
/*           Channel (RSCMUNB_Nsig_ConfCdpCdpwContCch) function.            */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_GetScalCdpCdpResults (ViSession instrumentHandle,
                                             ViUInt32 mode,
                                             ViReal64 *timingOffset,
                                             ViReal64 arrayOfResults[],
                                             ViChar errorMessage[])
{
    ViStatus  RSCMU_Status    = VI_SUCCESS;
    ViChar   tableOfDataValues[100][100];
    ViChar   cmd[50]          = "";
    ViChar   rdBuf[RD_BUFFER_SIZE]     = "";
    int      i                = 0;

    CHECK_VI_UINT32_RANGE(instrumentHandle, mode, 0, 2, RSCMU_ERROR_PARAMETER2, errorMessage);
    sprintf(cmd,"%s:SCALar:CDPower:CDPower?",RSCMUNB_Cmd[mode]);

    CHECKRETMSG(RSCMU_getViString(instrumentHandle, cmd, rdBuf, RD_BUFFER_SIZE, 0));

    // Split the multiple data values separated by a comma:
    RSCMU_SeparateDataValues (rdBuf, ",", tableOfDataValues);

    for(i=0; i<18; i++)
    {
        if (strcmp (tableOfDataValues[i], "NAN") != 0)
            sscanf (tableOfDataValues[i], "%le", &arrayOfResults[i]);
        else
            arrayOfResults[i] = RSCMU_NAN;
    }

    CHECKRETMSG(RSCMU_getViReal64(instrumentHandle,
                                "FETCh:SCALar:CDPower:CDPower:TOFFset?",
                                timingOffset,
                                RSCMU_ERROR_INTERPRETING_RESPONSE));

    RETMSG(RSCMU_Status);

}

/*===========================================================================*/
/* Function: Get Array of CDP Measurement Results (Power / Channels)         */
/* Purpose:  This function starts a Code Domain Power measurement and/or     */
/*           return the CDP in all 512 code channels that may be active in   */
/*           the downlink WCDMA signal. The results are obtained with the    */
/*           CPICH slot selected via Code Domain Power CPICH Slot            */
/*           (RSCMUNB_Nsig_ConfCdpCdpwContCpic) function.                   */
/*           The unit of the results depends on the Unit                     */
/*           (Nsig_ConfCdpCdpwContUnit) function setting.                    */
/*           In the default setting (absolute powers), the value ranges      */
/*           quoted above apply. With relative unit settings, the results    */
/*           are in the range between -100.0 dB to 0.0 dB.                   */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_GetArrCdpCdpChanResults (ViSession instrumentHandle,
                                                ViUInt32 mode,
                                                ViUInt32 statisticMode,
                                                ViReal64 arrayOfResults[],
                                                ViChar errorMessage[])
{
    ViStatus  RSCMU_Status    = VI_SUCCESS;
    ViChar   *strValue;
    ViChar   cmd[50]          = "";
    ViChar   rdBuf[RD_BUFFER_SIZE_ARRAY] = "";
    int      i                = 0;

    CHECK_VI_UINT32_RANGE(instrumentHandle, mode, 0, 2, RSCMU_ERROR_PARAMETER2, errorMessage);
    CHECK_VI_UINT32_RANGE(instrumentHandle, statisticMode, 0, 3, RSCMU_ERROR_PARAMETER3, errorMessage);

    sprintf(cmd,"%s:ARRay:CDPower:CDPower:Channels:%s?",RSCMUNB_Cmd[mode], RSCMUNB_StatisticsMode[statisticMode + 3]);

    CHECKRETMSG(RSCMU_getViString(instrumentHandle, cmd, rdBuf, RD_BUFFER_SIZE_ARRAY, 0));

    strValue = strtok(rdBuf, ",");

    while (strValue)
    {
        if (strcmp (strValue, "NAN") != 0)
            sscanf (strValue, "%le", &arrayOfResults[i]);
        else
            arrayOfResults[i] = RSCMU_NAN;

        strValue = strtok (NULL, ",");
        i++;
    }

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Get Array of CDP Measurement Results (Power / Slots)            */
/* Purpose:  This function starts a Code Domain Power measurement and/or     */
/*           return the CDP in all 15 CPICH slots. The results are obtained  */
/*           with the code channel number selected via Code Channel          */
/*           (Nsig_ConfCdpCdpwContCch).                                      */
/*           The unit of the results depends on the Unit                     */
/*           (Nsig_ConfCdpCdpwContUnit) function setting.                    */
/*           In the default setting (absolute powers), the value ranges      */
/*           quoted above apply. With relative unit settings, the results    */
/*           are in the range between -100.0 dB to 0.0 dB.                   */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_GetArrCdpCdpSlotResults (ViSession instrumentHandle,
                                                ViUInt32 mode,
                                                ViUInt32 statisticMode,
                                                ViReal64 arrayOfResults[],
                                                ViChar errorMessage[])
{
    ViStatus  RSCMU_Status    = VI_SUCCESS;
    ViChar   *strValue;
    ViChar   cmd[50]          = "";
    ViChar   rdBuf[RD_BUFFER_SIZE] = "";
    int      i                = 0;

    CHECK_VI_UINT32_RANGE(instrumentHandle, mode, 0, 2, RSCMU_ERROR_PARAMETER2, errorMessage);
    CHECK_VI_UINT32_RANGE(instrumentHandle, statisticMode, 0, 3, RSCMU_ERROR_PARAMETER3, errorMessage);

    sprintf(cmd,"%s:ARRay:CDPower:CDPower:Slots:%s?",RSCMUNB_Cmd[mode], RSCMUNB_StatisticsMode[statisticMode + 3]);

    CHECKRETMSG(RSCMU_getViString(instrumentHandle, cmd, rdBuf, RD_BUFFER_SIZE, 0));

    strValue = strtok(rdBuf, ",");

    while (strValue)
    {
        if (strcmp (strValue, "NAN") != 0)
            sscanf (strValue, "%le", &arrayOfResults[i]);
        else
            arrayOfResults[i] = RSCMU_NAN;

        strValue = strtok (NULL, ",");
        i++;
    }

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/*===========================================================================*/
/* Peak Code Domain Error
/*===========================================================================*/
/*===========================================================================*/

/*===========================================================================*/
/* Function: PCDE Statistical Depth                                          */
/* Purpose:  This function defines the number of slots forming a statistics  */
/*           cycle.                                                          */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_ConfCdpPcdeContStat (ViSession instrumentHandle,
                                            ViInt32 statisticalDepth,
                                            ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    CHECKRETMSG(RSCMU_setViInt32(instrumentHandle,
                                  "CONFigure:CDPower:PCDerror:CONTrol:STATistics",
                                  statisticalDepth,
                                  1, 1000,
                                  RSCMU_ERROR_PARAMETER2
                                  ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: PCDE Repetition                                                 */
/* Purpose:  This function determines the number of statistics cycles, the   */
/*           stop condition and the stepping mode for the measurement.       */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_ConfCdpPcdeContRep (ViSession instrumentHandle,
                                           ViInt32 repetitionMode,
                                           ViInt32 stopCondition,
                                           ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    ViChar    value[42] = "";

    if (RSCMU_invalidViInt32Range (repetitionMode, 0, 1))
        RETMSG(RSCMU_ERROR_PARAMETER2);

    if (RSCMU_invalidViInt32Range (stopCondition, 0, 1))
        RETMSG(RSCMU_ERROR_PARAMETER3);

    sprintf(value, "%s, %s, NONE", RSCMUNB_Repetition[repetitionMode], RSCMUNB_StopCondition[stopCondition]);
    CHECKRETMSG(RSCMU_setViString(instrumentHandle,
                                  "CONFigure:CDPower:PCDerror:CONTrol:REPetition",
                                  value,
                                  RSCMU_ERROR_PARAMETER2
                                  ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: PCDE Event Reporting                                            */
/* Purpose:  This function defines the event generated when the measurement  */
/*           is terminated or stopped.                                       */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_ConfCdpPcdeContErep (ViSession instrumentHandle,
                                            ViInt32 eventReporting,
                                            ViChar errorMessage[])
{

    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setEnum(instrumentHandle,
                              "CONFigure:CDPower:PCDerror:EREPorting",
                              eventReporting,
                              RSCMUNB_EventReport,
                              RSCMU_ERROR_PARAMETER2
                             ));
    RETMSG(RSCMU_Status);

}

/*===========================================================================*/
/* Function: PCDE Spreading Factor                                           */
/* Purpose:  This function determines the display spreading factor of the    */
/*           WCDMA downlink signal. The display spreading factor defines the */
/*           length of the output arrays of the data get commands E.g. Get   */
/*           Array of Peak Code Domain Error Measurement Results (Power /    */
/*           Channels) (RSCMUNB_Nsig_GetArrCdpPcdeChanResults) function.    */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_ConfCdpPcdeContSfac (ViSession instrumentHandle,
                                            ViInt32 spreadingFactor,
                                            ViChar errorMessage[])
{

    ViStatus  RSCMU_Status = RSCMU_ERROR_PARAMETER2;
    int i   = 4;

    while (i <= 512)
    {
        if  (i == spreadingFactor)
        {
            RSCMU_Status = VI_SUCCESS;
            break;
        }
        i = i<<1;
    }
    if (RSCMU_Status != VI_SUCCESS)
        RETMSG(RSCMU_Status);

    CHECKRETMSG(RSCMU_setViInt32(instrumentHandle,
                                  "CONFigure:CDPower:PCDerror:CONTrol:SFACtor",
                                  spreadingFactor,
                                  4, 512,
                                  RSCMU_ERROR_PARAMETER2
                                  ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: PCDE CPICH Slot                                                 */
/* Purpose:  This function selects the CPICH slot no. for the power vs. code */
/*           evaluation. The power vs. code results obtained with the        */
/*           selected CPICH slot can be retrieved by function Get Array of   */
/*           Peak Code Domain Error Measurement Results (Power / Channels)   */
/*           (RSCMUNB_Nsig_GetArrCdpPcdeChanResults)                        */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_ConfCdpPcdeContCpic (ViSession instrumentHandle,
                                            ViInt32 CPICHSlot,
                                            ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    CHECKRETMSG(RSCMU_setViInt32(instrumentHandle,
                                  "CONFigure:CDPower:PCDerror:CONTrol:CPIChslot",
                                  CPICHSlot,
                                  0, 14,
                                  RSCMU_ERROR_PARAMETER2
                                  ));
    RETMSG(RSCMU_Status);

}

/*===========================================================================*/
/* Function: Initiate Peak Code Domain Error Measurement                     */
/* Purpose:  This function starts the new peak code domain error measurement.*/
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_InitPcd (ViSession instrumentHandle,
                                ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setViString(instrumentHandle,
                                  "INITiate:CDPower:PCDerror",
                                  "",
                                  RSCMUNB_ERROR_INITIATE_FAILED
                                  ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Abort Peak Code Domain Error Measurement                        */
/* Purpose:  This function aborts the running peak code domain error         */
/*           measurement and switchs off.                                    */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_AborPcd (ViSession instrumentHandle,
                                ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setViString(instrumentHandle,
                                  "ABORt:CDPower:PCDerror",
                                  "",
                                  RSCMUNB_ERROR_ABORT_FAILED
                                  ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Stop Peak Code Domain Error Measurement                         */
/* Purpose:  This function stops the peak code domain error measurement      */
/*           after current stat. cycle.                                      */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_StopPcd (ViSession instrumentHandle,
                                ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setViString(instrumentHandle,
                                  "STOP:CDPower:PCDerror",
                                  "",
                                  RSCMUNB_ERROR_STOP_FAILED
                                  ));
    RETMSG(RSCMU_Status);

}

/*===========================================================================*/
/* Function: Continue Peak Code Domain Error Measurement                     */
/* Purpose:  This functions activate the next peak code domain error         */
/*           measurement step.                                               */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_ContPcd (ViSession instrumentHandle,
                                ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setViString(instrumentHandle,
                                  "CONTinue:CDPower:PCDerror",
                                  "",
                                  RSCMUNB_ERROR_CONTINUE_FAILED
                                  ));
    RETMSG(RSCMU_Status);

}

/*===========================================================================*/
/* Function: Get Peak Code Domain Error Measurement Status                   */
/* Purpose:  This function returns the status of the measurement.            */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_FetcScalCdpPcdStat (ViSession instrumentHandle,
                                           ViInt32 *measurementStatus,
                                           ViInt32 *currentStatistics,
                                           ViChar errorMessage[])
{
    ViStatus  RSCMU_Status    = VI_SUCCESS;
    ViChar   tableOfDataValues[100][100];

    GET_RD_BUF( instrumentHandle );
    CHECKRETMSG(RSCMU_getViString(instrumentHandle,
                                  "FETCh:CDPower:PCDerror:STATus?",
                                  rdBuf,
                                  RD_BUFFER_SIZE,
                                  0
                                  ));

    // Split the multiple data values separated by a comma:
    RSCMU_SeparateDataValues (rdBuf, ",", tableOfDataValues);

    // Get the status of the measurement:
    if ((RSCMU_Status = RSCMU_GetMeasStatVal (measurementStatus, tableOfDataValues, 0)) < 0)
        RETMSG(RSCMU_Status);

    // Counting Mode is not supported by this driver
    // // Get the current value of the counting mode counter (the number of the current statistics cycle):
    // if ((RSCMU_Status = RSCMU_GetCountingNo (CountingNo, tableOfDataValues, 1)) < 0)
    //      RETMSG(RSCMU_Status);

    // Get the current evaluation period within a statistics cycle:
    if ((RSCMU_Status = RSCMU_GetStatisticNo (currentStatistics, tableOfDataValues, 2)) < 0)
        RETMSG(RSCMU_Status);

    RETMSG(RSCMU_Status);

}

/*===========================================================================*/
/* Function: Get Scalar Peak Code Domain Error Measurement Results           */
/* Purpose:  This function starts a Peak Code Domain Error measurement and/or*/
/*           returns all scalar measurement results.                         */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_GetScalCdpPcdeResults (ViSession instrumentHandle,
                                              ViUInt32 mode,
                                              ViReal64 arrayOfResults[],
                                              ViChar errorMessage[])
{
    ViStatus  RSCMU_Status    = VI_SUCCESS;
    ViChar   tableOfDataValues[100][100];
    ViChar   cmd[50]          = "";
    ViChar   rdBuf[RD_BUFFER_SIZE]     = "";
    int      i                = 0;

    CHECK_VI_UINT32_RANGE(instrumentHandle, mode, 0, 2, RSCMU_ERROR_PARAMETER2, errorMessage);
    sprintf(cmd,"%s:SCALar:CDPower:PCDerror?",RSCMUNB_Cmd[mode]);

    CHECKRETMSG(RSCMU_getViString(instrumentHandle, cmd, rdBuf, RD_BUFFER_SIZE, 0));

    // Split the multiple data values separated by a comma:
    RSCMU_SeparateDataValues (rdBuf, ",", tableOfDataValues);

    for(i=0; i<18; i++)
    {
        if (strcmp (tableOfDataValues[i], "NAN") != 0)
            sscanf (tableOfDataValues[i], "%le", &arrayOfResults[i]);
        else
            arrayOfResults[i] = RSCMU_NAN;
    }

    RETMSG(RSCMU_Status);

}

/*===========================================================================*/
/* Function: Get Array of Peak Code Domain Error Measurement Results         */
/*           (Power / Channels)                                              */
/* Purpose:  This function starts a Peak Code Domain Error measurement and/or*/
/*           returns the code domain error in all code channels that are     */
/*           considered for the PDCE calculation. The results are obtained   */
/*           with the CPICH slot selected via PCDE CPICH Slot                */
/*           (Nsig_ConfCdpPcdeContCpic) function.                            */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_GetArrCdpPcdeChanResults (ViSession instrumentHandle,
                                                 ViUInt32 mode,
                                                 ViUInt32 statisticMode,
                                                 ViReal64 arrayOfResults[],
                                                 ViChar errorMessage[])
{
    ViStatus  RSCMU_Status    = VI_SUCCESS;
    ViChar   *strValue;
    ViChar   cmd[50]          = "";
    ViChar   rdBuf[RD_BUFFER_SIZE_ARRAY] = "";
    int      i                = 0;

    CHECK_VI_UINT32_RANGE(instrumentHandle, mode, 0, 2, RSCMU_ERROR_PARAMETER2, errorMessage);
    CHECK_VI_UINT32_RANGE(instrumentHandle, statisticMode, 0, 3, RSCMU_ERROR_PARAMETER3, errorMessage);
    if (statisticMode == RSCMU_MINIMUM)
        RETMSG(RSCMU_ERROR_PARAMETER3);

    sprintf(cmd,"%s:ARRay:CDPower:PCDerror:Channels:%s?",RSCMUNB_Cmd[mode], RSCMUNB_StatisticsMode[statisticMode + 3]);

    CHECKRETMSG(RSCMU_getViString(instrumentHandle, cmd, rdBuf, RD_BUFFER_SIZE_ARRAY, 0));

    strValue = strtok(rdBuf, ",");

    while (strValue)
    {
        if (strcmp (strValue, "NAN") != 0)
            sscanf (strValue, "%le", &arrayOfResults[i]);
        else
            arrayOfResults[i] = RSCMU_NAN;

        strValue = strtok (NULL, ",");
        i++;
    }

    RETMSG(RSCMU_Status);

}

/*===========================================================================*/
/* Function: Get Array of Peak Code Domain Error Measurement Results         */
/*           (Power / Slots)                                                 */
/* Purpose:  This function starts a Peak Code Domain Error measurement and/or*/
/*           returns the PCDerror in all 15 CPICH slots. The results are     */
/*           obtained with the code channel number selected via              */
/*           CONFigure:CDPower:PCDerror:CONTrol:CCHannel.                    */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_GetArrCdpPcdeSlotResults (ViSession instrumentHandle,
                                                 ViUInt32 mode,
                                                 ViUInt32 statisticMode,
                                                 ViReal64 arrayOfResults[],
                                                 ViChar errorMessage[])
{
    ViStatus  RSCMU_Status    = VI_SUCCESS;
    ViChar   *strValue;
    ViChar   cmd[50]          = "";
    ViChar   rdBuf[RD_BUFFER_SIZE] = "";
    int      i                = 0;

    CHECK_VI_UINT32_RANGE(instrumentHandle, mode, 0, 2, RSCMU_ERROR_PARAMETER2, errorMessage);
    CHECK_VI_UINT32_RANGE(instrumentHandle, statisticMode, 0, 3, RSCMU_ERROR_PARAMETER3, errorMessage);
    if (statisticMode == RSCMU_MINIMUM)
        RETMSG(RSCMU_ERROR_PARAMETER3);

    sprintf(cmd,"%s:ARRay:CDPower:PCDerror:Slots:%s?",RSCMUNB_Cmd[mode], RSCMUNB_StatisticsMode[statisticMode + 3]);

    CHECKRETMSG(RSCMU_getViString(instrumentHandle, cmd, rdBuf, RD_BUFFER_SIZE, 0));

    strValue = strtok(rdBuf, ",");

    while (strValue)
    {
        if (strcmp (strValue, "NAN") != 0)
            sscanf (strValue, "%le", &arrayOfResults[i]);
        else
            arrayOfResults[i] = RSCMU_NAN;

        strValue = strtok (NULL, ",");
        i++;
    }

    RETMSG(RSCMU_Status);

}

/*===========================================================================*/
/*===========================================================================*/
/* Spectrum ACLR                                                             */
/*===========================================================================*/
/*===========================================================================*/

/*===========================================================================*/
/* Function: ACLR Statistical Depth                                          */
/* Purpose:  This function defines the number of slots forming a statistics  */
/*           cycle.                                                          */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_ConfSpecAclrContStat (ViSession instrumentHandle,
                                             ViInt32 statisticalDepth,
                                             ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    CHECKRETMSG(RSCMU_setViInt32(instrumentHandle,
                                  "CONFigure:SPECtrum:ACLR:CONTrol:STATistics",
                                  statisticalDepth,
                                  1, 1000,
                                  RSCMU_ERROR_PARAMETER2
                                  ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: ACLR Repetition                                                 */
/* Purpose:  This function determines the number of statistics cycles, the   */
/*           stop condition and the stepping mode for the measurement.       */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_ConfSpecAclrContRep (ViSession instrumentHandle,
                                            ViInt32 repetitionMode,
                                            ViInt32 stopCondition,
                                            ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    ViChar    value[42] = "";

    if (RSCMU_invalidViInt32Range (repetitionMode, 0, 1))
        RETMSG(RSCMU_ERROR_PARAMETER2);

    if (RSCMU_invalidViInt32Range (stopCondition, 0, 1))
        RETMSG(RSCMU_ERROR_PARAMETER3);

    sprintf(value, "%s, %s, NONE", RSCMUNB_Repetition[repetitionMode], RSCMUNB_StopCondition[stopCondition]);
    CHECKRETMSG(RSCMU_setViString(instrumentHandle,
                                  "CONFigure:SPECtrum:ACLR:CONTrol:REPetition",
                                  value,
                                  RSCMU_ERROR_PARAMETER2
                                  ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: ACLR Event Reporting                                            */
/* Purpose:  This function defines the event generated when the measurement  */
/*           is terminated or stopped.                                       */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_ConfSpecAclrContErep (ViSession instrumentHandle,
                                             ViInt32 eventReporting,
                                             ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setEnum(instrumentHandle,
                              "CONFigure:SPECtrum:ACLR:EREPorting",
                              eventReporting,
                              RSCMUNB_EventReport,
                              RSCMU_ERROR_PARAMETER2
                             ));
    RETMSG(RSCMU_Status);

}

/*===========================================================================*/
/* Function: ACLR Scanning                                                   */
/* Purpose:  This function enables or disables the ACLR scanning mode. ACLR  */
/*           scanning extends the dynamic range of the instrument.           */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_ConfSpecAclrContScan (ViSession instrumentHandle,
                                             ViInt32 scanning,
                                             ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setEnum(instrumentHandle,
                              "CONFigure:SPECtrum:ACLR:SCANning",
                              scanning,
                              RSCMUNB_mode_On_Off,
                              RSCMU_ERROR_PARAMETER2
                             ));
    RETMSG(RSCMU_Status);

}

/*===========================================================================*/
/* Function: ACLR Measurement Length                                         */
/* Purpose:  This function selects the time interval to be measured. A small */
/*           measurement length accelerates the measurement but does not     */
/*           change the format or length of the output.                      */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_ConfSpecAclrContMlen (ViSession instrumentHandle,
                                             ViInt32 measurementLength,
                                             ViChar errorMessage[])
{

    ViStatus  RSCMU_Status = RSCMU_ERROR_PARAMETER2;
    int i   = 320;

    while (i <= 2560)
    {
        if  (i == measurementLength)
        {
            RSCMU_Status = VI_SUCCESS;
            break;
        }
        i = i<<1;
    }
    if (RSCMU_Status != VI_SUCCESS)
        RETMSG(RSCMU_Status);

    CHECKRETMSG(RSCMU_setViInt32(instrumentHandle,
                                  "CONFigure:SPECtrum:ACLR:CONTrol:MLENgth",
                                  measurementLength,
                                  320, 2560,
                                  RSCMU_ERROR_PARAMETER2
                                  ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Set Limits                                                      */
/* Purpose:  This function defines upper limits for the ACLR measured with   */
/*           the direct filter method.                                       */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_ConfSpecAclrLevLimUpp (ViSession instrumentHandle,
                                             ViInt32 statisticalType,
                                             ViInt32 relativeLimitCheck,
                                             ViInt32 channel,
                                             ViReal64 upperLimit,
                                             ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViChar      cmd1[WR_BUFFER_SIZE]    = "";
    ViChar      cmd2[WR_BUFFER_SIZE]    = "";
    ViString    statType[] = {"CMAX", "AVER", VI_NULL};

    if (RSCMU_invalidViBooleanRange ((ViBoolean)statisticalType))
        RETMSG(RSCMU_ERROR_PARAMETER2);
    if (RSCMU_invalidViBooleanRange ((ViBoolean)relativeLimitCheck))
        RETMSG(RSCMU_ERROR_PARAMETER3);
    if (RSCMU_invalidViInt32Range (channel, 1, 4))
        RETMSG(RSCMU_ERROR_PARAMETER4);
    if (RSCMU_invalidViReal64Range (upperLimit, -80.0, 0))
        RETMSG(RSCMU_ERROR_PARAMETER5);

    sprintf(cmd1, "CONFigure:SPECtrum:ACLR:LEVel:%s:LIMit:SCALar:ASYMmetric:UPPer:CHANnel%ld:VALue %.12f",
                    statType[statisticalType], channel, upperLimit);

    CHECKRETMSG(RSCMU_sendCmd(instrumentHandle, cmd1));

    sprintf(cmd2, "CONFigure:SPECtrum:ACLR:LEVel:%s:LIMit:SCALar:ASYMmetric:UPPer:CHANnel%ld:ENABle %s",
                statType[statisticalType], channel, RSCMUNB_mode_On_Off[relativeLimitCheck]);

    CHECKRETMSG(RSCMU_sendCmd(instrumentHandle, cmd2));

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Initiate ACLR Measurement                                       */
/* Purpose:  This function starts the Adjacent Channel Leakage power Ratio   */
/*           measurement.                                                    */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_InitSpecAclr (ViSession instrumentHandle,
                                     ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setViString(instrumentHandle,
                                  "INITiate:SPECtrum:ACLR",
                                  "",
                                  RSCMUNB_ERROR_INITIATE_FAILED
                                  ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Abort ACLR Measurement                                          */
/* Purpose:  This function aborts running Adjacent Channel Leakage power     */
/*           Ratio measurement and switch off.                               */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_AborSpecAclr (ViSession instrumentHandle,
                                     ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setViString(instrumentHandle,
                                  "ABORt:SPECtrum:ACLR",
                                  "",
                                  RSCMUNB_ERROR_ABORT_FAILED
                                  ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Stop ACLR Measurement                                           */
/* Purpose:  This function stops the Adjacent Channel Leakage power Ratio    */
/*           measurement after current stat. cycle.                          */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_StopSpecAclr (ViSession instrumentHandle,
                                     ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setViString(instrumentHandle,
                                  "STOP:SPECtrum:ACLR",
                                  "",
                                  RSCMUNB_ERROR_STOP_FAILED
                                  ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Continue ACLR Measurement                                       */
/* Purpose:  This function activates the next measurement step (only         */
/*           stepping mode) of the Adjacent Channel Leakage power Ratio      */
/*           measurement.                                                    */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_ContSpecAclr (ViSession instrumentHandle,
                                     ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setViString(instrumentHandle,
                                  "CONTinue:SPECtrum:ACLR",
                                  "",
                                  RSCMUNB_ERROR_CONTINUE_FAILED
                                  ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Get ACLR Measurement Status                                     */
/* Purpose:  This function returns the status of the Adjacent Channel Leakage*/
/*           power Ratio measurement.                                        */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_FetcScalSpecAclrStat (ViSession instrumentHandle,
                                             ViInt32 *measurementStatus,
                                             ViInt32 *currentStatistics,
                                             ViChar errorMessage[])
{
    ViStatus  RSCMU_Status    = VI_SUCCESS;
    ViChar   tableOfDataValues[100][100];

    GET_RD_BUF( instrumentHandle );
    CHECKRETMSG(RSCMU_getViString(instrumentHandle,
                                  "FETCh:SPECtrum:ACLR:STATus?",
                                  rdBuf,
                                  RD_BUFFER_SIZE,
                                  0
                                  ));

    // Split the multiple data values separated by a comma:
    RSCMU_SeparateDataValues (rdBuf, ",", tableOfDataValues);

    // Get the status of the measurement:
    if ((RSCMU_Status = RSCMU_GetMeasStatVal (measurementStatus, tableOfDataValues, 0)) < 0)
        RETMSG(RSCMU_Status);

    // Counting Mode is not supported by this driver
    // // Get the current value of the counting mode counter (the number of the current statistics cycle):
    // if ((RSCMU_Status = RSCMU_GetCountingNo (CountingNo, tableOfDataValues, 1)) < 0)
    //      RETMSG(RSCMU_Status);

    // Get the current evaluation period within a statistics cycle:
    if ((RSCMU_Status = RSCMU_GetStatisticNo (currentStatistics, tableOfDataValues, 2)) < 0)
        RETMSG(RSCMU_Status);

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Get Results of ACLR Measurement                                 */
/* Purpose:  This function start a Adjacent Channel Leakage power Ratio      */
/*           measurement and/or returns all (scalar) measurement results.    */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_GetSpecAclrResults (ViSession instrumentHandle,
                                           ViUInt32 mode, ViUInt32 unit,
                                           ViReal64 arrayOfResults[],
                                           ViChar errorMessage[])
{
    ViStatus  RSCMU_Status    = VI_SUCCESS;
    ViChar   *strValue;
    ViChar   cmd[50]          = "";
    ViChar   rdBuf[RD_BUFFER_SIZE_ARRAY] = "";
    int      i                = 0;

    CHECK_VI_UINT32_RANGE(instrumentHandle, mode, 0, 2, RSCMU_ERROR_PARAMETER2, errorMessage);
    CHECK_VI_UINT32_RANGE(instrumentHandle, unit, 0, 1, RSCMU_ERROR_PARAMETER3, errorMessage);

    sprintf(cmd,"%s:SCALar:SPECtrum:ACLR:%s?",RSCMUNB_Cmd[mode], RSCMUNB_ACLR_Unit[unit]);
//   READ[:SCALar]:SPECtrum:ACLR:ABSolute
//   READ[:SCALar]:SPECtrum:ACLR:RELative

    CHECKRETMSG(RSCMU_getViString(instrumentHandle, cmd, rdBuf, RD_BUFFER_SIZE_ARRAY, 0));

    strValue = strtok(rdBuf, ",");

    while (strValue)
    {
        if (strcmp (strValue, "NAN") != 0)
            sscanf (strValue, "%le", &arrayOfResults[i]);
        else
            arrayOfResults[i] = RSCMU_NAN;

        strValue = strtok (NULL, ",");
        i++;
    }

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Check Limits                                                    */
/* Purpose:  This function indicates whether and in which way the permissible*/
/*           tolerances for the scalar results in the measurement have been  */
/*           exceeded.                                                       */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_CalcSpecAclrMatcLim (ViSession instrumentHandle,
                                           ViInt32 RMSCurrent[],
                                           ViInt32 RMSAverage[],
                                           ViInt32 RMSMaximum[],
                                           ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViChar      tableOfDataValues[100][100];
    ViChar      rdBuf[RD_BUFFER_SIZE]   = "";
    int         i                       = 0;

    CHECKRETMSG(RSCMU_getViString(instrumentHandle, "CALCulate:SCALar:SPECtrum:ACLR:MATChing:LIMit?", rdBuf, RD_BUFFER_SIZE, 0));

    // Split the multiple data values separated by a comma:
    RSCMU_SeparateDataValues (rdBuf, ",", tableOfDataValues);

    for(i=0; i<4; i++)
    {
        if ((RSCMU_Status = RSCMU_GetLimitCheckValue (tableOfDataValues, i, &RMSCurrent[i])) < 0)
            return RSCMU_Status;
    }
    for(i=4; i<8; i++)
    {
        if ((RSCMU_Status = RSCMU_GetLimitCheckValue (tableOfDataValues, i, &RMSAverage[i])) < 0)
            return RSCMU_Status;
    }
    for(i=8; i<12; i++)
    {
        if ((RSCMU_Status = RSCMU_GetLimitCheckValue (tableOfDataValues, i, &RMSMaximum[i])) < 0)
            return RSCMU_Status;
    }

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/*===========================================================================*/
/* Spectrum OBW
/*===========================================================================*/
/*===========================================================================*/

/*===========================================================================*/
/* Function: OBW Result Mode                                                 */
/* Purpose:  This function specifies the type of measured values.            */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_ConfSpecObwContRmod (ViSession instrumentHandle,
                                            ViInt32 resultMode,
                                            ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setEnum(instrumentHandle,
                              "CONFigure:SPECtrum:OBW:CONTrol:RMODe",
                              resultMode,
                              RSCMUNB_Measurement_Result_Mode,
                              RSCMU_ERROR_PARAMETER2
                             ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: OBW Statistical Depth                                           */
/* Purpose:  This function defines the number of slots forming a statistics  */
/*           cycle.                                                          */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_ConfSpecObwContStat (ViSession instrumentHandle,
                                            ViInt32 statisticalDepth,
                                            ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    CHECKRETMSG(RSCMU_setViInt32(instrumentHandle,
                                  "CONFigure:SPECtrum:OBW:CONTrol:STATistics",
                                  statisticalDepth,
                                  1, 1000,
                                  RSCMU_ERROR_PARAMETER2
                                  ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: OBW Repetition                                                  */
/* Purpose:  This function determines the number of statistics cycles, the   */
/*           stop condition and the stepping mode for the measurement.       */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_ConfSpecObwContRep (ViSession instrumentHandle,
                                           ViInt32 repetitionMode,
                                           ViInt32 stopCondition,
                                           ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    ViChar    value[42] = "";

    if (RSCMU_invalidViInt32Range (repetitionMode, 0, 1))
        RETMSG(RSCMU_ERROR_PARAMETER2);

    if (RSCMU_invalidViInt32Range (stopCondition, 0, 1))
        RETMSG(RSCMU_ERROR_PARAMETER3);

    sprintf(value, "%s, %s, NONE", RSCMUNB_Repetition[repetitionMode], RSCMUNB_StopCondition[stopCondition]);
    CHECKRETMSG(RSCMU_setViString(instrumentHandle,
                                  "CONFigure:SPECtrum:OBW:CONTrol:REPetition",
                                  value,
                                  RSCMU_ERROR_PARAMETER2
                                  ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: OBW Event Reporting                                             */
/* Purpose:  This function defines the event generated when the measurement  */
/*           is terminated or stopped.                                       */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_ConfSpecObwContErep (ViSession instrumentHandle,
                                            ViInt32 eventReporting,
                                            ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setEnum(instrumentHandle,
                              "CONFigure:SPECtrum:OBW:EREPorting",
                              eventReporting,
                              RSCMUNB_EventReport,
                              RSCMU_ERROR_PARAMETER2
                             ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Set Limits                                                      */
/* Purpose:  This function defines upper limits for the OBW determined via   */
/*           FFT. The 5th-level keywords distinguish the limits for the      */
/*           Current and Maximum display modes (CMAX) and for the Average    */
/*           display mode (AVERage).                                         */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_ConfSpecObwObanUpp (ViSession instrumentHandle,
                                          ViInt32 statisticalType,
                                          ViInt32 relativeLimitCheck,
                                          ViReal64 upperLimit,
                                          ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViChar      cmd1[WR_BUFFER_SIZE]    = "";
    ViChar      cmd2[WR_BUFFER_SIZE]    = "";
    ViString    statType[] = {"CMAX", "AVER", VI_NULL};

    if (RSCMU_invalidViBooleanRange ((ViBoolean)statisticalType))
        RETMSG(RSCMU_ERROR_PARAMETER2);
    if (RSCMU_invalidViBooleanRange ((ViBoolean)relativeLimitCheck))
        RETMSG(RSCMU_ERROR_PARAMETER3);
    if (RSCMU_invalidViReal64Range (upperLimit, 0.0, 10.0))
        RETMSG(RSCMU_ERROR_PARAMETER4);

    sprintf(cmd1, "CONFigure:SPECtrum:OBW:OBANdwidth:%s:LIMit:SCALar:ASYMmetric:UPPER:VALue %.12f",
                    statType[statisticalType], upperLimit);

    CHECKRETMSG(RSCMU_sendCmd(instrumentHandle, cmd1));

    sprintf(cmd2, "CONFigure:SPECtrum:OBW:OBANdwidth:%s:LIMit:SCALar:ASYMmetric:UPPER:ENABle %s",
                statType[statisticalType], RSCMUNB_mode_On_Off[relativeLimitCheck]);

    CHECKRETMSG(RSCMU_sendCmd(instrumentHandle, cmd2));

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Initiate OBW Measurement                                        */
/* Purpose:  This function starts new measurement.                           */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_InitSpecObw (ViSession instrumentHandle,
                                    ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setViString(instrumentHandle,
                                  "INITiate:SPECtrum:OBW",
                                  "",
                                  RSCMUNB_ERROR_INITIATE_FAILED
                                  ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Abort OBW Measurement                                           */
/* Purpose:  This function aborts the running measurement and sitch off.     */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_AborSpecObw (ViSession instrumentHandle,
                                    ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setViString(instrumentHandle,
                                  "ABORt:SPECtrum:OBW",
                                  "",
                                  RSCMUNB_ERROR_ABORT_FAILED
                                  ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Stop OBW Measurement                                            */
/* Purpose:  This function stops measurement after current stat. cycle.      */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_StopSpecObw (ViSession instrumentHandle,
                                    ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setViString(instrumentHandle,
                                  "STOP:SPECtrum:OBW",
                                  "",
                                  RSCMUNB_ERROR_STOP_FAILED
                                  ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Continue OBW Measurement                                        */
/* Purpose:  This function activates the next measurement step (stepping     */
/*           mode only).                                                     */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_ContSpecObw (ViSession instrumentHandle,
                                    ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setViString(instrumentHandle,
                                  "CONTinue:SPECtrum:OBW",
                                  "",
                                  RSCMUNB_ERROR_CONTINUE_FAILED
                                  ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Get OBW Measurement Status                                      */
/* Purpose:  This function returns the status of the measurement.            */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_FetcScalSpecObwStat (ViSession instrumentHandle,
                                            ViInt32 *measurementStatus,
                                            ViInt32 *counter_for_current_statistics,
                                            ViChar errorMessage[])
{
    ViStatus  RSCMU_Status    = VI_SUCCESS;
    ViChar   tableOfDataValues[100][100];

    GET_RD_BUF( instrumentHandle );
    CHECKRETMSG(RSCMU_getViString(instrumentHandle,
                                  "FETCh:SPECtrum:OBW:STATus?",
                                  rdBuf,
                                  RD_BUFFER_SIZE,
                                  0
                                  ));

    // Split the multiple data values separated by a comma:
    RSCMU_SeparateDataValues (rdBuf, ",", tableOfDataValues);

    // Get the status of the measurement:
    if ((RSCMU_Status = RSCMU_GetMeasStatVal (measurementStatus, tableOfDataValues, 0)) < 0)
        RETMSG(RSCMU_Status);

    // Counting Mode is not supported by this driver
    // // Get the current value of the counting mode counter (the number of the current statistics cycle):
    // if ((RSCMU_Status = RSCMU_GetCountingNo (CountingNo, tableOfDataValues, 1)) < 0)
    //      RETMSG(RSCMU_Status);

    // Get the current evaluation period within a statistics cycle:
    if ((RSCMU_Status = RSCMU_GetStatisticNo (counter_for_current_statistics, tableOfDataValues, 2)) < 0)
        RETMSG(RSCMU_Status);

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: OBW Default                                                     */
/* Purpose:  This function sets all parameters of the subsystem to default   */
/*           values (the setting OFF results in an error message).           */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_DefSpecObwCont (ViSession instrumentHandle,
                                      ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_sendCmd(instrumentHandle,
                                  "DEFault:SPECtrum:OBW:CONTrol ON"));

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Configure OBW SubArray Range                                    */
/* Purpose:  This function configures the READ:SUBarrays...,                 */
/*           FETCh:SUBarrays..., and SAMPle:SUBarrays:SPECtrum... commands.  */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_NSig_ConfSubSpecObw (ViSession instrumentHandle, ViInt32 mode,
                                      ViInt32 number_ofSubranges,
                                      ViInt32 startWFSS[], ViInt32 samplesWFSS[],
                                      ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    ViChar      buffer[WR_BUFFER_SIZE];
    ViChar*     p2buf;
    ViInt32     i;

    if (RSCMU_invalidViInt32Range (number_ofSubranges, 1, 32))
        RETMSG(RSCMU_ERROR_PARAMETER3);

    p2buf = buffer + sprintf (buffer, "CONFigure:SUBarrays:SPECtrum:OBW %s,", RSCMUNB_SubArr[mode]);

    for (i = 0; i < number_ofSubranges; i++)
        p2buf += sprintf(p2buf, "%ld,%ld,", startWFSS[i], samplesWFSS[i]);

    /* remove comma */
    *--p2buf = '\0';

    CHECKRETMSG(RSCMU_sendCmd(instrumentHandle, buffer));

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Get Scalar Result of OBW Measurement                            */
/* Purpose:  This function starts a SPECtrum:OBW measurement and/or return   */
/*           all (scalar) measurement results.                               */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_GetScalSpecOBWResults (ViSession instrumentHandle,
                                              ViUInt32 mode,
                                              ViReal64 arrayOfResults[],
                                              ViChar errorMessage[])
{
    ViStatus  RSCMU_Status    = VI_SUCCESS;
    ViChar   *strValue;
    ViChar   cmd[50]          = "";
    ViChar   rdBuf[RD_BUFFER_SIZE_ARRAY] = "";
    int      i                = 0;

    CHECK_VI_UINT32_RANGE(instrumentHandle, mode, 0, 2, RSCMU_ERROR_PARAMETER2, errorMessage);

    sprintf(cmd,"%s:SCALar:SPECtrum:OBW?",RSCMUNB_Cmd[mode]);
//   READ[:SCALar]:SPECtrum:OBW

    CHECKRETMSG(RSCMU_getViString(instrumentHandle, cmd, rdBuf, RD_BUFFER_SIZE_ARRAY, 0));

    strValue = strtok(rdBuf, ",");

    while (strValue)
    {
        if (strcmp (strValue, "NAN") != 0)
            sscanf (strValue, "%le", &arrayOfResults[i]);
        else
            arrayOfResults[i] = RSCMU_NAN;

        strValue = strtok (NULL, ",");
        i++;
    }

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Get Array of OBW Measurement Results                            */
/* Purpose:  This function returns the ACLR at fixed, equidistant test points*/
/*           with a 20 kHz raster. The number n of measured values is        */
/*           n =1250, corresponding to a frequency range from 12.50 MHz to  */
/*           +12.48 MHz.                                                     */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_GetArrSpecOBWResults (ViSession instrumentHandle,
                                             ViUInt32 mode,
                                             ViReal64 arrayOfResults[],
                                             ViChar errorMessage[])
{
    ViStatus  RSCMU_Status    = VI_SUCCESS;
    ViChar   *strValue;
    ViChar   cmd[50]          = "";
    ViChar   rdBuf[RD_BUFFER_SIZE_ARRAY] = "";
    int      i                = 0;

    CHECK_VI_UINT32_RANGE(instrumentHandle, mode, 0, 2, RSCMU_ERROR_PARAMETER2, errorMessage);

    sprintf(cmd,"%s:ARRay:SPECtrum:OBW:CURRent?",RSCMUNB_Cmd[mode]);
//   READ:ARRay:SPECtrum:OBW:CURRent?

    CHECKRETMSG(RSCMU_getViString(instrumentHandle, cmd, rdBuf, RD_BUFFER_SIZE_ARRAY, 0));

    strValue = strtok(rdBuf, ",");

    while (strValue)
    {
        if (strcmp (strValue, "NAN") != 0)
            sscanf (strValue, "%le", &arrayOfResults[i]);
        else
            arrayOfResults[i] = RSCMU_NAN;

        strValue = strtok (NULL, ",");
        i++;
    }

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Get Subarray of OBW Measurement Results                         */
/* Purpose:  This function returns the ACLR in the subranges defined by      */
/*           means of the CONFigure:SUBarrays:SPECtrum:OBW... commands.      */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_GetSubArrSpecOBWResults (ViSession instrumentHandle,
                                               ViUInt32 mode,
                                               ViReal64 arrayOfResults[],
                                               ViChar errorMessage[])
{
    ViStatus  RSCMU_Status    = VI_SUCCESS;
    ViChar   *strValue;
    ViChar   cmd[50]          = "";
    ViChar   rdBuf[RD_BUFFER_SIZE_ARRAY] = "";
    int      i                = 0;

    CHECK_VI_UINT32_RANGE(instrumentHandle, mode, 0, 2, RSCMU_ERROR_PARAMETER2, errorMessage);

    sprintf(cmd,"%s:SUBarrays:SPECtrum:OBW:CURRent?",RSCMUNB_Cmd[mode]);

    CHECKRETMSG(RSCMU_getViString(instrumentHandle, cmd, rdBuf, RD_BUFFER_SIZE_ARRAY, 0));

    strValue = strtok(rdBuf, ",");

    while (strValue)
    {
        if (strcmp (strValue, "NAN") != 0)
            sscanf (strValue, "%.12f", &arrayOfResults[i]);
        else
            arrayOfResults[i] = RSCMU_NAN;

        strValue = strtok (NULL, ",");
        i++;
    }

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Check Limits                                                    */
/* Purpose:  This function indicates whether and in which way the permissible*/
/*           tolerances for the scalar results in the measurement have been  */
/*           exceeded.                                                       */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_CalcSpecObwMatcLim (ViSession instrumentHandle,
                                          ViInt32 OBWCurrent[],
                                          ViInt32 OBWAverage[],
                                          ViInt32 OBWMaximum[],
                                          ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViChar      tableOfDataValues[100][100];
    ViChar      rdBuf[RD_BUFFER_SIZE]   = "";
    int         i                       = 0;

    CHECKRETMSG(RSCMU_getViString(instrumentHandle, "CALCulate:SCALar:SPECtrum:OBW:MATChing:LIMit?", rdBuf, RD_BUFFER_SIZE, 0));

    // Split the multiple data values separated by a comma:
    RSCMU_SeparateDataValues (rdBuf, ",", tableOfDataValues);

    for(i=0; i<4; i++)
    {
        if ((RSCMU_Status = RSCMU_GetLimitCheckValue (tableOfDataValues, i, &OBWCurrent[i])) < 0)
            return RSCMU_Status;
    }
    for(i=4; i<8; i++)
    {
        if ((RSCMU_Status = RSCMU_GetLimitCheckValue (tableOfDataValues, i, &OBWAverage[i])) < 0)
            return RSCMU_Status;
    }
    for(i=8; i<12; i++)
    {
        if ((RSCMU_Status = RSCMU_GetLimitCheckValue (tableOfDataValues, i, &OBWMaximum[i])) < 0)
            return RSCMU_Status;
    }

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: EMASK Result Mode                                               */
/* Purpose:  This function specifies the type of measured values.            */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_ConfSpecEmaskContRmod (ViSession instrumentHandle,
                                            ViInt32 resultMode,
                                            ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setEnum(instrumentHandle,
                              "CONFigure:SPECtrum:EMASk:CONTrol:RMODe",
                              resultMode,
                              RSCMUNB_Measurement_Result_Mode,
                              RSCMU_ERROR_PARAMETER2
                             ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: EMASK Statistical Depth                                         */
/* Purpose:  This function defines the number of slots forming a statistics  */
/*           cycle.                                                          */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_ConfSpecEmaskContStat (ViSession instrumentHandle,
                                            ViInt32 statisticalDepth,
                                            ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    CHECKRETMSG(RSCMU_setViInt32(instrumentHandle,
                                  "CONFigure:SPECtrum:EMASk:CONTrol:STATistics",
                                  statisticalDepth,
                                  1, 1000,
                                  RSCMU_ERROR_PARAMETER2
                                  ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: EMASK Repetition                                                */
/* Purpose:  This function determines the number of statistics cycles, the   */
/*           stop condition and the stepping mode for the measurement.       */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_ConfSpecEmaskContRep (ViSession instrumentHandle,
                                            ViInt32 repetitionMode,
                                            ViInt32 stopCondition, ViInt32 stepMode,
                                            ViChar _VI_FAR errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    ViChar    value[42] = "";

    if (RSCMU_invalidViInt32Range (repetitionMode, 0, 1))
        RETMSG(RSCMU_ERROR_PARAMETER2);

    if (RSCMU_invalidViInt32Range (stopCondition, 0, 1))
        RETMSG(RSCMU_ERROR_PARAMETER3);

    if (RSCMU_invalidViInt32Range (stepMode, 0, 1))
        RETMSG(RSCMU_ERROR_PARAMETER4);

    sprintf(value, "%s,%s,%s", RSCMUNB_Repetition[repetitionMode], RSCMUNB_StopCondition[stopCondition],RSCMUNB_StepMode[stepMode]);
    CHECKRETMSG(RSCMU_setViString(instrumentHandle,
                                  "CONFigure:SPECtrum:EMASk:CONTrol:REPetition",
                                  value,
                                  RSCMU_ERROR_PARAMETER2
                                  ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: EMASK Event Reporting                                           */
/* Purpose:  This function defines the event generated when the measurement  */
/*           is terminated or stopped.                                       */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_ConfSpecEmaskContErep (ViSession instrumentHandle,
                                            ViInt32 eventReporting,
                                            ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setEnum(instrumentHandle,
                              "CONFigure:SPECtrum:EMASk:EREPorting",
                              eventReporting,
                              RSCMUNB_EventReport,
                              RSCMU_ERROR_PARAMETER2
                             ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: EMASK Select Spectrum Limit                                     */
/* Purpose:  This function defines the event generated when the measurement  */
/*           is terminated or stopped.                                       */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_ConfSpecEmasLimSel (ViSession instrumentHandle,
                                            ViInt32 emissionMask,
                                            ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setEnum(instrumentHandle,
                              "CONFigure:SPECtrum:EMASk:LIMit:SELect",
                              emissionMask,
                              RSCMUNB_EmissionMask,
                              RSCMU_ERROR_PARAMETER2
                             ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: EMASK Limit Lines                                               */
/* Purpose:  This function defines and selects the values for the Current    */
/*           and Max. and for the Average spectrum emission mask,            */
/*           respectively. Each limit value at the points A to J of the      */
/*           emission mask is defined either as an absolute level or relative*/
/*           to the maximum output power of the Node B. Besides, setting two */
/*           consecutive points to OFF disables the limit check in the       */
/*           section confined by these points.                               */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_ConfSpecEmasAverLimLinAsymUpp (ViSession instrumentHandle,
                                                     ViInt32 displayMode,
                                                     ViInt32 _VI_FAR level[],
                                                     ViReal64 _VI_FAR levelValue[],
                                                     ViBoolean _VI_FAR checkLimit[],
                                                     ViChar _VI_FAR errorMessage[])
{
    ViChar*     p2buf;
    ViChar      wrtBuf[WR_BUFFER_SIZE];
    ViInt32     i;

    ViString statisticsMode[] = {"CMAX","AVERage",VI_NULL};

    p2buf = wrtBuf + sprintf (wrtBuf, "CONFigure:SPECtrum:EMASk:%s:LIMit:LINE:ASYMmetric:UPPer ", statisticsMode[displayMode]);

    for ( i=0; i<10; i++)
    {
        if (checkLimit[i])
            p2buf += sprintf (p2buf,"%s,%.12f,", RSCMUNB_ACLR_Unit[level[i]], levelValue[i]);
        else
            p2buf += sprintf (p2buf,"%s,OFF,", RSCMUNB_ACLR_Unit[level[i]]);
    }

    /* remove comma */
    *--p2buf = '\0';

    RETMSG(RSCMU_sendCmd (instrumentHandle, wrtBuf));
}

/*===========================================================================*/
/* Function: EMASK Default Limit Line                                        */
/* Purpose:  This function sets all parameters of the subsystem to           */
/*           default values (the setting OFF results in an error message).   */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_DefSpecEmasLimLine (ViSession instrumentHandle,
                                     ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setViString(instrumentHandle,
                                  "DEFault:SPECtrum:EMASk:LIMit:LINE ON",
                                  "",
                                  RSCMUNB_ERROR_ABORT_FAILED
                                  ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Initiate EMASK Measurement                                      */
/* Purpose:  This function starts new measurement.                           */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_InitSpecEmask (ViSession instrumentHandle,
                                    ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setViString(instrumentHandle,
                                  "INITiate:SPECtrum:EMASk",
                                  "",
                                  RSCMUNB_ERROR_INITIATE_FAILED
                                  ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Abort EMASK Measurement                                         */
/* Purpose:  This function aborts the running measurement and sitch off.     */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_AborSpecEmask (ViSession instrumentHandle,
                                    ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setViString(instrumentHandle,
                                  "ABORt:SPECtrum:EMASk",
                                  "",
                                  RSCMUNB_ERROR_ABORT_FAILED
                                  ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Stop EMASK Measurement                                          */
/* Purpose:  This function stops measurement after current stat. cycle.      */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_StopSpecEmask (ViSession instrumentHandle,
                                    ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setViString(instrumentHandle,
                                  "STOP:SPECtrum:EMASk",
                                  "",
                                  RSCMUNB_ERROR_STOP_FAILED
                                  ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Continue EMASK Measurement                                      */
/* Purpose:  This function activates the next measurement step (stepping     */
/*           mode only).                                                     */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_ContSpecEmask (ViSession instrumentHandle,
                                    ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setViString(instrumentHandle,
                                  "CONTinue:SPECtrum:EMASk",
                                  "",
                                  RSCMUNB_ERROR_CONTINUE_FAILED
                                  ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Get EMASK Measurement Status                                    */
/* Purpose:  This function returns the status of the measurement.            */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_FetcScalSpecEmaskStat (ViSession instrumentHandle,
                                            ViInt32 *measurementStatus,
                                            ViInt32 *counter_for_current_statistics,
                                            ViChar errorMessage[])
{
    ViStatus  RSCMU_Status    = VI_SUCCESS;
    ViChar   tableOfDataValues[100][100];

    GET_RD_BUF( instrumentHandle );
    CHECKRETMSG(RSCMU_getViString(instrumentHandle,
                                  "FETCh:SPECtrum:EMASk:STATus?",
                                  rdBuf,
                                  RD_BUFFER_SIZE,
                                  0
                                  ));

    // Split the multiple data values separated by a comma:
    RSCMU_SeparateDataValues (rdBuf, ",", tableOfDataValues);

    // Get the status of the measurement:
    if ((RSCMU_Status = RSCMU_GetMeasStatVal (measurementStatus, tableOfDataValues, 0)) < 0)
        RETMSG(RSCMU_Status);

    // Counting Mode is not supported by this driver
    // // Get the current value of the counting mode counter (the number of the current statistics cycle):
    // if ((RSCMU_Status = RSCMU_GetCountingNo (CountingNo, tableOfDataValues, 1)) < 0)
    //      RETMSG(RSCMU_Status);

    // Get the current evaluation period within a statistics cycle:
    if ((RSCMU_Status = RSCMU_GetStatisticNo (counter_for_current_statistics, tableOfDataValues, 2)) < 0)
        RETMSG(RSCMU_Status);

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Get Scalar Result of EMASK Measurement                          */
/* Purpose:  This function starts a SPECtrum:EMASk measurement and/or return */
/*           all (scalar) measurement results.                               */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_GetScalSpecEmaskResults (ViSession instrumentHandle,
                                              ViUInt32 mode,
                                              ViReal64 arrayOfResults[],
                                              ViChar errorMessage[])
{
    ViStatus  RSCMU_Status    = VI_SUCCESS;
    ViChar   *strValue;
    ViChar   cmd[50]          = "";
    ViChar   rdBuf[RD_BUFFER_SIZE_ARRAY] = "";
    int      i                = 0;

    CHECK_VI_UINT32_RANGE(instrumentHandle, mode, 0, 2, RSCMU_ERROR_PARAMETER2, errorMessage);

    sprintf(cmd,"%s:SCALar:SPECtrum:EMASk?",RSCMUNB_Cmd[mode]);
//   READ[:SCALar]:SPECtrum:EMASk

    CHECKRETMSG(RSCMU_getViString(instrumentHandle, cmd, rdBuf, RD_BUFFER_SIZE_ARRAY, 0));

    strValue = strtok(rdBuf, ",");

    while (strValue)
    {
        if (strcmp (strValue, "NAN") != 0)
            sscanf (strValue, "%le", &arrayOfResults[i]);
        else
            arrayOfResults[i] = RSCMU_NAN;

        strValue = strtok (NULL, ",");
        i++;
    }

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Get Array of EMASK Measurement Results                          */
/* Purpose:  This function returns the spectrum emission at fixed test       */
/*           points. The array contains a total of 1556 values which are     */
/*           distributed over the 11 sections of the emission mask.          */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_GetArrSpecEmaskResults (ViSession instrumentHandle,
                                                ViUInt32 mode,
                                                ViUInt32 statisticMode,
                                                ViReal64 arrayOfResults[],
                                                ViChar errorMessage[])
{
    ViStatus  RSCMU_Status    = VI_SUCCESS;
    ViChar   *strValue;
    ViChar   cmd[100]          = "";
    ViChar   rdBuf[RD_BUFFER_SIZE_ARRAY] = "";
    int      i                = 0;

    CHECK_VI_UINT32_RANGE(instrumentHandle, mode, 0, 2, RSCMU_ERROR_PARAMETER2, errorMessage);
    CHECK_VI_UINT32_RANGE(instrumentHandle, statisticMode, 0, 2, RSCMU_ERROR_PARAMETER3, errorMessage);

    sprintf(cmd,"%s:ARRay:SPECtrum:EMASk:%s?",RSCMUNB_Cmd[mode],RSCMUNB_StatisticsMode[statisticMode + 3]);
//   READ:ARRay:SPECtrum:EMASk:CURRent?

    CHECKRETMSG(RSCMU_getViString(instrumentHandle, cmd, rdBuf, RD_BUFFER_SIZE_ARRAY, 0));

    strValue = strtok(rdBuf, ",");

    while (strValue)
    {
        if (strcmp (strValue, "NAN") != 0)
            sscanf (strValue, "%le", &arrayOfResults[i]);
        else
            arrayOfResults[i] = RSCMU_NAN;

        strValue = strtok (NULL, ",");
        i++;
    }

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
/* M O D U L A T I O N                                                       */
/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/

/*===========================================================================*/
/*===========================================================================*/
/* Modulation Overview                                                       */
/*===========================================================================*/
/*===========================================================================*/

/*===========================================================================*/
/* Function: Mod Overview Statistical Depth                                  */
/* Purpose:  This function defines the number of slots forming a statistics  */
/*           cycle.                                                          */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_ConfModOvwContStat (ViSession instrumentHandle,
                                           ViInt32 statisticalDepth,
                                           ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    CHECKRETMSG(RSCMU_setViInt32(instrumentHandle,
                                  "CONFigure:MODulation:OVERview:WCDMa:CONTrol:STATistics",
                                  statisticalDepth,
                                  1, 1000,
                                  RSCMU_ERROR_PARAMETER2
                                  ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Mod Overview Repetition                                         */
/* Purpose:  This function determines the number of statistics cycles, the   */
/*           stop condition and the stepping mode for the measurement.       */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_ConfModOvwContRep (ViSession instrumentHandle,
                                          ViInt32 repetitionMode,
                                          ViInt32 stopCondition,
                                          ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    ViChar    value[42] = "";

    if (RSCMU_invalidViInt32Range (repetitionMode, 0, 1))
        RETMSG(RSCMU_ERROR_PARAMETER2);

    if (RSCMU_invalidViInt32Range (stopCondition, 0, 1))
        RETMSG(RSCMU_ERROR_PARAMETER3);

    sprintf(value, "%s, %s, NONE", RSCMUNB_Repetition[repetitionMode], RSCMUNB_StopCondition[stopCondition]);
    CHECKRETMSG(RSCMU_setViString(instrumentHandle,
                                  "CONFigure:MODulation:OVERview:WCDMa:CONTrol:REPetition",
                                  value,
                                  RSCMU_ERROR_PARAMETER2
                                  ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Mod Overview Event Reporting                                    */
/* Purpose:  This function defines the events generated when the measurement */
/*           is terminated or stopped.                                       */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_ConfModOvwContErep (ViSession instrumentHandle,
                                           ViInt32 eventReporting,
                                           ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setEnum(instrumentHandle,
                              "CONFigure:MODulation:OVERview:WCDMa:EREPorting",
                              eventReporting,
                              RSCMUNB_EventReport,
                              RSCMU_ERROR_PARAMETER2
                             ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Mod Overview CPICH Slot                                         */
/* Purpose:  This function selects the CPICH slot no. for the modulation     */
/*           measurement.                                                    */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_ConfModOvwContCpic (ViSession instrumentHandle,
                                           ViInt32 CPICHSlot,
                                           ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    CHECKRETMSG(RSCMU_setViInt32(instrumentHandle,
                                  "CONFigure:MODulation:OVERview:WCDMa:CONTrol:CPIChslot",
                                  CPICHSlot,
                                  0, 14,
                                  RSCMU_ERROR_PARAMETER2
                                  ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Initiate Mod Overview Measurement                               */
/* Purpose:  This function starts new modulation measurement.                */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_InitModOvw (ViSession instrumentHandle,
                                   ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setViString(instrumentHandle,
                                  "INITiate:MODulation:OVERview:WCDMa",
                                  "",
                                  RSCMUNB_ERROR_INITIATE_FAILED
                                  ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Abort Mod Overview Measurement                                  */
/* Purpose:  This function aborts running modulation measurement and switch  */
/*           off.                                                            */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_AborModOvw (ViSession instrumentHandle,
                                   ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setViString(instrumentHandle,
                                  "ABORt:MODulation:OVERview:WCDMa",
                                  "",
                                  RSCMUNB_ERROR_ABORT_FAILED
                                  ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Stop Mod Overview Measurement                                   */
/* Purpose:  This function stops modulation measurement after current stat.  */
/*           cycle.                                                          */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_StopModOvw (ViSession instrumentHandle,
                                   ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setViString(instrumentHandle,
                                  "STOP:MODulation:OVERview:WCDMa",
                                  "",
                                  RSCMUNB_ERROR_STOP_FAILED
                                  ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Continue Mod Overview Measurement                               */
/* Purpose:  This function activates the next modulation measurement step    */
/*           (only stepping mode).                                           */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_ContModOvw (ViSession instrumentHandle,
                                   ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setViString(instrumentHandle,
                                  "CONTinue:MODulation:OVERview:WCDMa",
                                  "",
                                  RSCMUNB_ERROR_CONTINUE_FAILED
                                  ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Get Mod Overview Measurement Status                             */
/* Purpose:  This function returns the status of the modulation measurement. */
/*           (See operating manual for status system details).               */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_FetcScalModOvwStat (ViSession instrumentHandle,
                                           ViInt32 *measurementStatus,
                                           ViInt32 *counter_for_current_statistics,
                                           ViChar errorMessage[])
{
    ViStatus  RSCMU_Status    = VI_SUCCESS;
    ViChar   tableOfDataValues[100][100];

    GET_RD_BUF( instrumentHandle );
    CHECKRETMSG(RSCMU_getViString(instrumentHandle,
                                  "FETCh:MODulation:OVERview:WCDMa:STATus?" ,
                                  rdBuf,
                                  RD_BUFFER_SIZE,
                                  0
                                  ));

    // Split the multiple data values separated by a comma:
    RSCMU_SeparateDataValues (rdBuf, ",", tableOfDataValues);

    // Get the status of the measurement:
    if ((RSCMU_Status = RSCMU_GetMeasStatVal (measurementStatus, tableOfDataValues, 0)) < 0)
        RETMSG(RSCMU_Status);

    // Counting Mode is not supported by this driver
    // // Get the current value of the counting mode counter (the number of the current statistics cycle):
    // if ((RSCMU_Status = RSCMU_GetCountingNo (CountingNo, tableOfDataValues, 1)) < 0)
    //      RETMSG(RSCMU_Status);

    // Get the current evaluation period within a statistics cycle:
    if ((RSCMU_Status = RSCMU_GetStatisticNo (counter_for_current_statistics, tableOfDataValues, 2)) < 0)
        RETMSG(RSCMU_Status);

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Get scalar results of modulation measurement                    */
/* Purpose:  This function starts a modulation measurement and/or returns    */
/*           all scalar measurement results.                                 */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_GetScalModOvwResults (ViSession instrumentHandle,
                                             ViUInt32 mode,
                                             ViReal64 arrayOfResults[],
                                             ViChar errorMessage[])
{
    ViStatus  RSCMU_Status    = VI_SUCCESS;
    ViChar   *strValue;
    ViChar   cmd[50]          = "";
    ViChar   rdBuf[RD_BUFFER_SIZE_ARRAY] = "";
    int      i                = 0;

    CHECK_VI_UINT32_RANGE(instrumentHandle, mode, 0, 2, RSCMU_ERROR_PARAMETER2, errorMessage);

    sprintf(cmd,"%s:MODulation:OVERview:WCDMa?",RSCMUNB_Cmd[mode]);
//   READ[:SCALar]:MODulation:OVERview:WCDMa

    CHECKRETMSG(RSCMU_getViString(instrumentHandle, cmd, rdBuf, RD_BUFFER_SIZE_ARRAY, 0));

    strValue = strtok(rdBuf, ",");

    while (strValue)
    {
        if (strcmp (strValue, "NAN") != 0)
            sscanf (strValue, "%le", &arrayOfResults[i]);
        else
            arrayOfResults[i] = RSCMU_NAN;

        strValue = strtok (NULL, ",");
        i++;
    }

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/*===========================================================================*/
/* Error Vector Magnitude                                                    */
/*===========================================================================*/
/*===========================================================================*/

/*===========================================================================*/
/* Function: Mod EVM Statistical Depth                                       */
/* Purpose:  This function defines the number of slots forming a statistics  */
/*           cycle.                                                          */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_ConfModEvmContStat (ViSession instrumentHandle,
                                           ViInt32 statisticalDepth,
                                           ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    CHECKRETMSG(RSCMU_setViInt32(instrumentHandle,
                                  "CONFigure:MODulation:EVMagnitude:WCDMa:CONTrol:STATistics",
                                  statisticalDepth,
                                  1, 1000,
                                  RSCMU_ERROR_PARAMETER2
                                  ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Mod EVM Repetition                                              */
/* Purpose:  This function determines the number of statistics cycles, the   */
/*           stop condition and the stepping mode for the measurement.       */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_ConfModEvmContRep (ViSession instrumentHandle,
                                          ViInt32 repetitionMode,
                                          ViInt32 stopCondition,
                                          ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    ViChar    value[42] = "";

    if (RSCMU_invalidViInt32Range (repetitionMode, 0, 1))
        RETMSG(RSCMU_ERROR_PARAMETER2);

    if (RSCMU_invalidViInt32Range (stopCondition, 0, 1))
        RETMSG(RSCMU_ERROR_PARAMETER3);

    sprintf(value, "%s, %s, NONE", RSCMUNB_Repetition[repetitionMode], RSCMUNB_StopCondition[stopCondition]);
    CHECKRETMSG(RSCMU_setViString(instrumentHandle,
                                  "CONFigure:MODulation:EVMagnitude:WCDMa:CONTrol:REPetition",
                                  value,
                                  RSCMU_ERROR_PARAMETER2
                                  ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Mod EVM Event Reporting                                         */
/* Purpose:  This function defines the events generated when the measurement */
/*           is terminated or stopped.                                       */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_ConfModEvmContErep (ViSession instrumentHandle,
                                           ViInt32 eventReporting,
                                           ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setEnum(instrumentHandle,
                              "CONFigure:MODulation:EVMagnitude:WCDMa:EREPorting",
                              eventReporting,
                              RSCMUNB_EventReport,
                              RSCMU_ERROR_PARAMETER2
                             ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Mod EVM CPICH Slot                                              */
/* Purpose:  This function selects the CPICH slot no. for the                */
/*           MODulation:EVMagnitude measurement.                             */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_ConfModEvmContCpic (ViSession instrumentHandle,
                                           ViInt32 CPICHSlot,
                                           ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    CHECKRETMSG(RSCMU_setViInt32(instrumentHandle,
                                  "CONFigure:MODulation:EVMagnitude:WCDMa:CONTrol:CPIChslot",
                                  CPICHSlot,
                                  0, 14,
                                  RSCMU_ERROR_PARAMETER2
                                  ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Initiate Mod EVM Measurement                                    */
/* Purpose:  This function starts the new modulation measurement.            */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_InitModEvm (ViSession instrumentHandle,
                                   ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setViString(instrumentHandle,
                                  "INITiate:MODulation:EVMagnitude:WCDMa",
                                  "",
                                  RSCMUNB_ERROR_INITIATE_FAILED
                                  ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Abort Mod EVM Measurement                                       */
/* Purpose:  This function aborts running modulation measurement and switch  */
/*           off.                                                            */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_AborModEvm (ViSession instrumentHandle,
                                   ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setViString(instrumentHandle,
                                  "ABORt:MODulation:EVMagnitude:WCDMa",
                                  "",
                                  RSCMUNB_ERROR_ABORT_FAILED
                                  ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Stop Mod EVM Measurement                                        */
/* Purpose:  This function stops modulation measurement after current stat.  */
/*           cycle.                                                          */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_StopModEvm (ViSession instrumentHandle,
                                   ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setViString(instrumentHandle,
                                  "STOP:MODulation:EVMagnitude:WCDMa",
                                  "",
                                  RSCMUNB_ERROR_STOP_FAILED
                                  ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Continue Mod EVM Measurement                                    */
/* Purpose:  This function activates the next measurement step (only         */
/*           stepping mode).                                                 */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_ContModEvm (ViSession instrumentHandle,
                                   ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setViString(instrumentHandle,
                                  "CONTinue:MODulation:EVMagnitude:WCDMa",
                                  "",
                                  RSCMUNB_ERROR_CONTINUE_FAILED
                                  ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Get Mod EVM Measurement Status                                  */
/* Purpose:  This function returns the status of the modulation measurement. */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_FetcScalModEvmStat (ViSession instrumentHandle,
                                           ViInt32 *measurementStatus,
                                           ViInt32 *counter_for_current_statistics,
                                           ViChar errorMessage[])
{
    ViStatus  RSCMU_Status    = VI_SUCCESS;
    ViChar   tableOfDataValues[100][100];

    GET_RD_BUF( instrumentHandle );
    CHECKRETMSG(RSCMU_getViString(instrumentHandle,
                                  "FETCh:MODulation:EVMagnitude:WCDMa:STATus?" ,
                                  rdBuf,
                                  RD_BUFFER_SIZE,
                                  0
                                  ));

    // Split the multiple data values separated by a comma:
    RSCMU_SeparateDataValues (rdBuf, ",", tableOfDataValues);

    // Get the status of the measurement:
    if ((RSCMU_Status = RSCMU_GetMeasStatVal (measurementStatus, tableOfDataValues, 0)) < 0)
        RETMSG(RSCMU_Status);

    // Counting Mode is not supported by this driver
    // // Get the current value of the counting mode counter (the number of the current statistics cycle):
    // if ((RSCMU_Status = RSCMU_GetCountingNo (CountingNo, tableOfDataValues, 1)) < 0)
    //      RETMSG(RSCMU_Status);

    // Get the current evaluation period within a statistics cycle:
    if ((RSCMU_Status = RSCMU_GetStatisticNo (counter_for_current_statistics, tableOfDataValues, 2)) < 0)
        RETMSG(RSCMU_Status);

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Get scalar results of Error Vector Magnitude mesurement         */
/* Purpose:  This function starts a MODulation:EVMagnitude measurement       */
/*           and/or return all scalar measurement results.                   */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_GetScalModEvmResults (ViSession instrumentHandle,
                                             ViUInt32 mode,
                                             ViReal64 arrayOfResults[],
                                             ViChar errorMessage[])
{
    ViStatus  RSCMU_Status    = VI_SUCCESS;
    ViChar   *strValue;
    ViChar   cmd[50]          = "";
    ViChar   rdBuf[RD_BUFFER_SIZE_ARRAY] = "";
    int      i                = 0;

    CHECK_VI_UINT32_RANGE(instrumentHandle, mode, 0, 2, RSCMU_ERROR_PARAMETER2, errorMessage);

    sprintf(cmd,"%s:MODulation:EVMagnitude:WCDMa?",RSCMUNB_Cmd[mode]);
//   READ[:SCALar]:MODulation:EVMagnitude:WCDMa

    CHECKRETMSG(RSCMU_getViString(instrumentHandle, cmd, rdBuf, RD_BUFFER_SIZE_ARRAY, 0));

    strValue = strtok(rdBuf, ",");

    while (strValue)
    {
        if (strcmp (strValue, "NAN") != 0)
            sscanf (strValue, "%le", &arrayOfResults[i]);
        else
            arrayOfResults[i] = RSCMU_NAN;

        strValue = strtok (NULL, ",");
        i++;
    }

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Get array of Error Vector Magnitude results (EVM / CodeChannel) */
/* Purpose:  This function returns the error vector magnitude vs. time at    */
/*           fixed, equidistant test points with a 1-chip raster. The results*/
/*           are obtained with the CPICH slot selected via Mod EVM CPICH Slot*/
/*           (Nsig_ConfModEvmContCpic) function; the EVM (RMS) values of all */
/*           CPICH slots can be measured by Get array of Error Vector        */
/*           Magnitude results (EVM / Slot) (Nsig_GetArrModPerrSlotResults)  */
/*           function.                                                       */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_GetArrModEvmChanResults (ViSession instrumentHandle,
                                                ViUInt32 mode,
                                                ViUInt32 statisticMode,
                                                ViReal64 arrayOfResults[],
                                                ViChar errorMessage[])
{
    ViStatus  RSCMU_Status    = VI_SUCCESS;
    ViChar   *strValue;
    ViChar   cmd[100]          = "";
    ViChar   rdBuf[RD_BUFFER_SIZE_ARRAY] = "";
    int      i                = 0;

    CHECK_VI_UINT32_RANGE(instrumentHandle, mode, 0, 2, RSCMU_ERROR_PARAMETER2, errorMessage);
    CHECK_VI_UINT32_RANGE(instrumentHandle, statisticMode, 0, 2, RSCMU_ERROR_PARAMETER3, errorMessage);

    sprintf(cmd,"%s:ARRay:MODulation:EVMagnitude:WCDMa:CHANnels:%s?",RSCMUNB_Cmd[mode],RSCMUNB_StatisticsMode[statisticMode]);
//   READ:ARRay:MODulation:EVMagnitude:WCDMa:CHANnels:CURRent?

    CHECKRETMSG(RSCMU_getViString(instrumentHandle, cmd, rdBuf, RD_BUFFER_SIZE_ARRAY, 0));

    strValue = strtok(rdBuf, ",");

    while (strValue)
    {
        if (strcmp (strValue, "NAN") != 0)
            sscanf (strValue, "%le", &arrayOfResults[i]);
        else
            arrayOfResults[i] = RSCMU_NAN;

        strValue = strtok (NULL, ",");
        i++;
    }

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Get Subarray of Error Vector Magnitude results                  */
/*           (EVM / CodeChannel)                                             */
/* Purpose:  This function returns the error vector magnitude versus time in */
/*           the subranges defined by means of the                           */
/*           CONFigure:SUBarrays:MODulation:EVMagnitude... commands.         */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_GetSubArrModEvmChanResults (ViSession instrumentHandle,
                                                  ViUInt32 mode,
                                                  ViUInt32 statisticMode,
                                                  ViReal64 arrayOfResults[],
                                                  ViChar errorMessage[])
{
    ViStatus  RSCMU_Status    = VI_SUCCESS;
    ViChar   *strValue;
    ViChar   cmd[100]          = "";
    ViChar   rdBuf[RD_BUFFER_SIZE_ARRAY] = "";
    int      i                = 0;

    CHECK_VI_UINT32_RANGE(instrumentHandle, mode, 0, 2, RSCMU_ERROR_PARAMETER2, errorMessage);
    CHECK_VI_UINT32_RANGE(instrumentHandle, statisticMode, 0, 2, RSCMU_ERROR_PARAMETER3, errorMessage);

    sprintf(cmd,"%s:SUBarrays:MODulation:EVMagnitude:WCDMa:CHANnels:%s?",RSCMUNB_Cmd[mode],RSCMUNB_StatisticsMode[statisticMode]);

    CHECKRETMSG(RSCMU_getViString(instrumentHandle, cmd, rdBuf, RD_BUFFER_SIZE_ARRAY, 0));

    strValue = strtok(rdBuf, ",");

    while (strValue)
    {
        if (strcmp (strValue, "NAN") != 0)
            sscanf (strValue, "%.12f", &arrayOfResults[i]);
        else
            arrayOfResults[i] = RSCMU_NAN;

        strValue = strtok (NULL, ",");
        i++;
    }

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Get array of Error Vector Magnitude results (EVM / Slot)        */
/* Purpose:  This function returns the EVM (RMS) values in all 15 CPICH      */
/*           slots of the frame.                                             */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_GetArrModEvmSlotResults (ViSession instrumentHandle,
                                                ViUInt32 mode,
                                                ViUInt32 statisticMode,
                                                ViReal64 arrayOfResults[],
                                                ViChar errorMessage[])
{
    ViStatus  RSCMU_Status    = VI_SUCCESS;
    ViChar   *strValue;
    ViChar   cmd[100]          = "";
    ViChar   rdBuf[RD_BUFFER_SIZE_ARRAY] = "";
    int      i                = 0;

    CHECK_VI_UINT32_RANGE(instrumentHandle, mode, 0, 2, RSCMU_ERROR_PARAMETER2, errorMessage);
    CHECK_VI_UINT32_RANGE(instrumentHandle, statisticMode, 0, 2, RSCMU_ERROR_PARAMETER3, errorMessage);

    sprintf(cmd,"%s:ARRay:MODulation:EVMagnitude:WCDMa:SLOTs:%s?",RSCMUNB_Cmd[mode],RSCMUNB_StatisticsMode[statisticMode]);
//   READ:ARRay:MODulation:EVMagnitude:WCDMa:SLOTs:CURRent?

    CHECKRETMSG(RSCMU_getViString(instrumentHandle, cmd, rdBuf, RD_BUFFER_SIZE_ARRAY, 0));

    strValue = strtok(rdBuf, ",");

    while (strValue)
    {
        if (strcmp (strValue, "NAN") != 0)
            sscanf (strValue, "%le", &arrayOfResults[i]);
        else
            arrayOfResults[i] = RSCMU_NAN;

        strValue = strtok (NULL, ",");
        i++;
    }

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/*===========================================================================*/
/* Magnitude Error                                                           */
/*===========================================================================*/
/*===========================================================================*/

/*===========================================================================*/
/* Function: Mod Magn. Error Statistical Depth                               */
/* Purpose:  This function defines the number of slots forming a statistics  */
/*           cycle.                                                          */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_ConfModMerrContStat (ViSession instrumentHandle,
                                            ViInt32 statisticalDepth,
                                            ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    CHECKRETMSG(RSCMU_setViInt32(instrumentHandle,
                                  "CONFigure:MODulation:MERRor:WCDMa:CONTrol:STATistics",
                                  statisticalDepth,
                                  1, 1000,
                                  RSCMU_ERROR_PARAMETER2
                                  ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Mod Magn. Error Repetition                                      */
/* Purpose:  This function determines the number of statistics cycles, the   */
/*           stop condition and the stepping mode for the measurement.       */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_ConfModMerrContRep (ViSession instrumentHandle,
                                           ViInt32 repetitionMode,
                                           ViInt32 stopCondition,
                                           ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    ViChar    value[42] = "";

    if (RSCMU_invalidViInt32Range (repetitionMode, 0, 1))
        RETMSG(RSCMU_ERROR_PARAMETER2);

    if (RSCMU_invalidViInt32Range (stopCondition, 0, 1))
        RETMSG(RSCMU_ERROR_PARAMETER3);

    sprintf(value, "%s, %s, NONE", RSCMUNB_Repetition[repetitionMode], RSCMUNB_StopCondition[stopCondition]);
    CHECKRETMSG(RSCMU_setViString(instrumentHandle,
                                  "CONFigure:MODulation:MERRor:WCDMa:CONTrol:REPetition",
                                  value,
                                  RSCMU_ERROR_PARAMETER2
                                  ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Mod Magn. Error Event Reporting                                 */
/* Purpose:  This function defines the events generated when the measurement */
/*           is terminated or stopped.                                       */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_ConfModMerrContErep (ViSession instrumentHandle,
                                            ViInt32 eventReporting,
                                            ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setEnum(instrumentHandle,
                              "CONFigure:MODulation:MERRor:WCDMa:EREPorting",
                              eventReporting,
                              RSCMUNB_EventReport,
                              RSCMU_ERROR_PARAMETER2
                             ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Mod Magn. Error CPICH Slot                                      */
/* Purpose:  This function selects the CPICH slot no. for the                */
/*           MODulation:MERRor measurement.                                  */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_ConfModMerrContCpic (ViSession instrumentHandle,
                                            ViInt32 CPICHSlot,
                                            ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    CHECKRETMSG(RSCMU_setViInt32(instrumentHandle,
                                  "CONFigure:MODulation:MERRor:WCDMa:CONTrol:CPIChslot",
                                  CPICHSlot,
                                  0, 14,
                                  RSCMU_ERROR_PARAMETER2
                                  ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Initiate Mod Magn. Error Measurement                            */
/* Purpose:  This function starts the new magnitude error measurement.       */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_InitModMerr (ViSession instrumentHandle,
                                    ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setViString(instrumentHandle,
                                  "INITiate:MODulation:MERRor:WCDMa",
                                  "",
                                  RSCMUNB_ERROR_INITIATE_FAILED
                                  ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Abort Mod Magn. Error Measurement                               */
/* Purpose:  This function aborts running magnitude error measurement and    */
/*           switch off.                                                     */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_AborModMerr (ViSession instrumentHandle,
                                    ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setViString(instrumentHandle,
                                  "ABORt:MODulation:MERRor:WCDMa",
                                  "",
                                  RSCMUNB_ERROR_ABORT_FAILED
                                  ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Stop Mod Magn. Error Measurement                                */
/* Purpose:  This function stops magnitude error measurement after current   */
/*           stat. cycle.                                                    */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_StopModMerr (ViSession instrumentHandle,
                                    ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setViString(instrumentHandle,
                                  "STOP:MODulation:MERRor:WCDMa",
                                  "",
                                  RSCMUNB_ERROR_STOP_FAILED
                                  ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Continue Mod Magn. Error Measurement                            */
/* Purpose:  This function activates the next magnitude error measurement    */
/*           step (only stepping mode).                                      */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_ContModMerr (ViSession instrumentHandle,
                                    ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setViString(instrumentHandle,
                                  "CONTinue:MODulation:MERRor:WCDMa",
                                  "",
                                  RSCMUNB_ERROR_CONTINUE_FAILED
                                  ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Get Mod Magn. Error Measurement Status                          */
/* Purpose:  This function returns the status of the magnitude error         */
/*           measurement.                                                    */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_FetcScalModMerrStat (ViSession instrumentHandle,
                                            ViInt32 *measurementStatus,
                                            ViInt32 *counter_for_current_statistics,
                                            ViChar errorMessage[])
{
    ViStatus  RSCMU_Status    = VI_SUCCESS;
    ViChar   tableOfDataValues[100][100];

    GET_RD_BUF( instrumentHandle );
    CHECKRETMSG(RSCMU_getViString(instrumentHandle,
                                  "FETCh:MODulation:MERRor:WCDMa:STATus?" ,
                                  rdBuf,
                                  RD_BUFFER_SIZE,
                                  0
                                  ));

    // Split the multiple data values separated by a comma:
    RSCMU_SeparateDataValues (rdBuf, ",", tableOfDataValues);

    // Get the status of the measurement:
    if ((RSCMU_Status = RSCMU_GetMeasStatVal (measurementStatus, tableOfDataValues, 0)) < 0)
        RETMSG(RSCMU_Status);

    // Counting Mode is not supported by this driver
    // // Get the current value of the counting mode counter (the number of the current statistics cycle):
    // if ((RSCMU_Status = RSCMU_GetCountingNo (CountingNo, tableOfDataValues, 1)) < 0)
    //      RETMSG(RSCMU_Status);

    // Get the current evaluation period within a statistics cycle:
    if ((RSCMU_Status = RSCMU_GetStatisticNo (counter_for_current_statistics, tableOfDataValues, 2)) < 0)
        RETMSG(RSCMU_Status);

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Configure Magn Error SubArray Range                             */
/* Purpose:  This function configures the READ:SUBarrays...,                 */
/*           FETCh:SUBarrays..., and SAMPle:SUBarrays:MODulation:MERRor...   */
/*           commands.                                                       */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_NSig_ConfSubModMerrWcdm (ViSession instrumentHandle,
                                          ViInt32 mode,
                                          ViInt32 number_ofSubranges,
                                          ViInt32 start[], ViInt32 samples[],
                                          ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViChar      buffer[WR_BUFFER_SIZE];
    ViChar*     p2buf;
    ViInt32     i;

    if (RSCMU_invalidViInt32Range (number_ofSubranges, 1, 32))
        RETMSG(RSCMU_ERROR_PARAMETER3);

    p2buf = buffer + sprintf (buffer, "CONFigure:SUBarrays:MODulation:MERRor:WCDMa %s,", RSCMUNB_SubArr[mode]);

    for (i = 0; i < number_ofSubranges; i++)
        p2buf += sprintf(p2buf, "%ld,%ld,", start[i], samples[i]);

    /* remove comma */
    *--p2buf = '\0';

    CHECKRETMSG(RSCMU_sendCmd(instrumentHandle, buffer));

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Get scalar results of Magn. Error mesurement                    */
/* Purpose:  This function starts a modulation magnitude error measurement   */
/*           and/or returns all scalar measurement results.                  */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_GetScalModMerrResults (ViSession instrumentHandle,
                                              ViUInt32 mode,
                                              ViReal64 arrayOfResults[],
                                              ViChar errorMessage[])
{
    ViStatus  RSCMU_Status    = VI_SUCCESS;
    ViChar   *strValue;
    ViChar   cmd[50]          = "";
    ViChar   rdBuf[RD_BUFFER_SIZE_ARRAY] = "";
    int      i                = 0;

    CHECK_VI_UINT32_RANGE(instrumentHandle, mode, 0, 2, RSCMU_ERROR_PARAMETER2, errorMessage);

    sprintf(cmd,"%s:MODulation:MERRor:WCDMa?",RSCMUNB_Cmd[mode]);
//   READ[:SCALar]:MODulation:MERRor:WCDMa

    CHECKRETMSG(RSCMU_getViString(instrumentHandle, cmd, rdBuf, RD_BUFFER_SIZE_ARRAY, 0));

    strValue = strtok(rdBuf, ",");

    while (strValue)
    {
        if (strcmp (strValue, "NAN") != 0)
            sscanf (strValue, "%le", &arrayOfResults[i]);
        else
            arrayOfResults[i] = RSCMU_NAN;

        strValue = strtok (NULL, ",");
        i++;
    }

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Get array of Magn. Error results (EVM / CodeChannel)            */
/* Purpose:  This function returns the magnitude error vs. time at fixed,    */
/*           equidistant test points with a 1-chip raster. The results are   */
/*           obtained with the CPICH slot selected via Mod Magn. Error CPICH */
/*           Slot (Nsig_ConfModMerrContCpic) function. The Magnitude Error   */
/*           (RMS) values of all CPICH slots can be measured by Get array of */
/*           Magn. Error results (EVM / Slot) Nsig_GetArrModMerrSlotResults) */
/*           function.                                                       */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_GetArrModMerrChanResults (ViSession instrumentHandle,
                                                 ViUInt32 mode,
                                                 ViUInt32 statisticMode,
                                                 ViReal64 arrayOfResults[],
                                                 ViChar errorMessage[])
{
    ViStatus  RSCMU_Status    = VI_SUCCESS;
    ViChar   *strValue;
    ViChar   cmd[100]          = "";
    ViChar   rdBuf[RD_BUFFER_SIZE_ARRAY] = "";
    int      i                = 0;

    CHECK_VI_UINT32_RANGE(instrumentHandle, mode, 0, 2, RSCMU_ERROR_PARAMETER2, errorMessage);
    CHECK_VI_UINT32_RANGE(instrumentHandle, statisticMode, 0, 2, RSCMU_ERROR_PARAMETER3, errorMessage);

    sprintf(cmd,"%s:ARRay:MODulation:MERRor:WCDMa:CHANnels:%s?",RSCMUNB_Cmd[mode],RSCMUNB_StatisticsMode[statisticMode]);
//   READ:ARRay:MODulation:MERRor:WCDMa:CHANnels:CURRent?

    CHECKRETMSG(RSCMU_getViString(instrumentHandle, cmd, rdBuf, RD_BUFFER_SIZE_ARRAY, 0));

    strValue = strtok(rdBuf, ",");

    while (strValue)
    {
        if (strcmp (strValue, "NAN") != 0)
            sscanf (strValue, "%le", &arrayOfResults[i]);
        else
            arrayOfResults[i] = RSCMU_NAN;

        strValue = strtok (NULL, ",");
        i++;
    }

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Get Subarray of Magn Error results (EVM / CodeChannel)          */
/* Purpose:  This function returns the magnitude error versus time in the    */
/*           subranges defined by means of the                               */
/*           CONFigure:SUBarrays:MODulation:MERRor... commands.              */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_GetSubArrModMerrChanResults (ViSession instrumentHandle,
                                                   ViUInt32 mode,
                                                   ViUInt32 statisticMode,
                                                   ViReal64 arrayOfResults[],
                                                   ViChar errorMessage[])
{
    ViStatus  RSCMU_Status    = VI_SUCCESS;
    ViChar   *strValue;
    ViChar   cmd[100]         = "";
    ViChar   rdBuf[RD_BUFFER_SIZE_ARRAY] = "";
    int      i                = 0;

    CHECK_VI_UINT32_RANGE(instrumentHandle, mode, 0, 2, RSCMU_ERROR_PARAMETER2, errorMessage);
    CHECK_VI_UINT32_RANGE(instrumentHandle, statisticMode, 0, 2, RSCMU_ERROR_PARAMETER3, errorMessage);

    sprintf(cmd,"%s:SUBarrays:MODulation:MERRor:WCDMa:CHANnels:%s?",RSCMUNB_Cmd[mode],RSCMUNB_StatisticsMode[statisticMode]);

    CHECKRETMSG(RSCMU_getViString(instrumentHandle, cmd, rdBuf, RD_BUFFER_SIZE_ARRAY, 0));

    strValue = strtok(rdBuf, ",");

    while (strValue)
    {
        if (strcmp (strValue, "NAN") != 0)
            sscanf (strValue, "%.12f", &arrayOfResults[i]);
        else
            arrayOfResults[i] = RSCMU_NAN;

        strValue = strtok (NULL, ",");
        i++;
    }

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Get array of Magn. Error results (EVM / Slot)                   */
/* Purpose:  This function returns the Magnitude Error (RMS) values in all   */
/*           15 CPICH slots of the frame.                                    */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_GetArrModMerrSlotResults (ViSession instrumentHandle,
                                                 ViUInt32 mode,
                                                 ViUInt32 statisticMode,
                                                 ViReal64 arrayOfResults[],
                                                 ViChar errorMessage[])
{
    ViStatus  RSCMU_Status    = VI_SUCCESS;
    ViChar   *strValue;
    ViChar   cmd[100]          = "";
    ViChar   rdBuf[RD_BUFFER_SIZE_ARRAY] = "";
    int      i                = 0;

    CHECK_VI_UINT32_RANGE(instrumentHandle, mode, 0, 2, RSCMU_ERROR_PARAMETER2, errorMessage);
    CHECK_VI_UINT32_RANGE(instrumentHandle, statisticMode, 0, 2, RSCMU_ERROR_PARAMETER3, errorMessage);

    sprintf(cmd,"%s:ARRay:MODulation:MERRor:WCDMa:SLOTs:%s?",RSCMUNB_Cmd[mode],RSCMUNB_StatisticsMode[statisticMode]);
//   READ:ARRay:MODulation:MERRor:WCDMa:SLOTs:CURRent?

    CHECKRETMSG(RSCMU_getViString(instrumentHandle, cmd, rdBuf, RD_BUFFER_SIZE_ARRAY, 0));

    strValue = strtok(rdBuf, ",");

    while (strValue)
    {
        if (strcmp (strValue, "NAN") != 0)
            sscanf (strValue, "%le", &arrayOfResults[i]);
        else
            arrayOfResults[i] = RSCMU_NAN;

        strValue = strtok (NULL, ",");
        i++;
    }

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/*===========================================================================*/
/* Phase Error                                                               */
/*===========================================================================*/
/*===========================================================================*/

/*===========================================================================*/
/* Function: Mod Phase Error Statistical Depth                               */
/* Purpose:  This function defines the number of slots forming a statistics  */
/*           cycle.                                                          */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_ConfModPerrContStat (ViSession instrumentHandle,
                                            ViInt32 statisticalDepth,
                                            ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    CHECKRETMSG(RSCMU_setViInt32(instrumentHandle,
                                  "CONFigure:MODulation:PERRor:WCDMa:CONTrol:STATistics",
                                  statisticalDepth,
                                  1, 1000,
                                  RSCMU_ERROR_PARAMETER2
                                  ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Mod Phase Error Repetition                                      */
/* Purpose:  This function determines the number of statistics cycles, the   */
/*           stop condition and the stepping mode for the measurement.       */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_ConfModPerrContRep (ViSession instrumentHandle,
                                           ViInt32 repetitionMode,
                                           ViInt32 stopCondition,
                                           ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    ViChar    value[42] = "";

    if (RSCMU_invalidViInt32Range (repetitionMode, 0, 1))
        RETMSG(RSCMU_ERROR_PARAMETER2);

    if (RSCMU_invalidViInt32Range (stopCondition, 0, 1))
        RETMSG(RSCMU_ERROR_PARAMETER3);

    sprintf(value, "%s, %s, NONE", RSCMUNB_Repetition[repetitionMode], RSCMUNB_StopCondition[stopCondition]);
    CHECKRETMSG(RSCMU_setViString(instrumentHandle,
                                  "CONFigure:MODulation:PERRor:WCDMa:CONTrol:REPetition",
                                  value,
                                  RSCMU_ERROR_PARAMETER2
                                  ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Mod Phase Error Event Reporting                                 */
/* Purpose:  This function defines the events generated when the measurement */
/*           is terminated or stopped.                                       */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_ConfModPerrContErep (ViSession instrumentHandle,
                                            ViInt32 eventReporting,
                                            ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setEnum(instrumentHandle,
                              "CONFigure:MODulation:PERRor:WCDMa:EREPorting",
                              eventReporting,
                              RSCMUNB_EventReport,
                              RSCMU_ERROR_PARAMETER2
                             ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Mod Phase Error CPICH Slot                                      */
/* Purpose:  This function selects the CPICH slot no. for the                */
/*           MODulation:PERRor measurement.                                  */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_ConfModPerrContCpic (ViSession instrumentHandle,
                                            ViInt32 CPICHSlot,
                                            ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    CHECKRETMSG(RSCMU_setViInt32(instrumentHandle,
                                  "CONFigure:MODulation:PERRor:WCDMa:CONTrol:CPIChslot",
                                  CPICHSlot,
                                  0, 14,
                                  RSCMU_ERROR_PARAMETER2
                                  ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Initiate Mod Phase Error Measurement                            */
/* Purpose:  This function starts the modulation phase error measurement.    */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_InitModPerr (ViSession instrumentHandle,
                                    ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setViString(instrumentHandle,
                                  "INITiate:MODulation:PERRor:WCDMa",
                                  "",
                                  RSCMUNB_ERROR_INITIATE_FAILED
                                  ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Abort Mod Phase Error Measurement                               */
/* Purpose:  This function aborts the running modulation phase error         */
/*           measurement and switch off.                                     */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_AborModPerr (ViSession instrumentHandle,
                                    ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setViString(instrumentHandle,
                                  "ABORt:MODulation:PERRor:WCDMa",
                                  "",
                                  RSCMUNB_ERROR_ABORT_FAILED
                                  ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Stop Mod Phase Error Measurement                                */
/* Purpose:  This function stops the modulation phase error measurement      */
/*           after current stat. cycle.                                      */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_StopModPerr (ViSession instrumentHandle,
                                    ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setViString(instrumentHandle,
                                  "STOP:MODulation:PERRor:WCDMa",
                                  "",
                                  RSCMUNB_ERROR_STOP_FAILED
                                  ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Continue Mod Phase Error Measurement                            */
/* Purpose:  This function activates the next modulation phase error         */
/*           measurement step (only stepping mode).                          */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_ContModPerr (ViSession instrumentHandle,
                                    ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setViString(instrumentHandle,
                                  "CONTinue:MODulation:PERRor:WCDMa",
                                  "",
                                  RSCMUNB_ERROR_CONTINUE_FAILED
                                  ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Get Mod Phase Error Measurement Status                          */
/* Purpose:  This function returns the status of the modulation phase error  */
/*           measurement.                                                    */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_FetcScalModPerrStat (ViSession instrumentHandle,
                                            ViInt32 *measurementStatus,
                                            ViInt32 *counter_for_current_statistics,
                                            ViChar errorMessage[])
{
    ViStatus  RSCMU_Status    = VI_SUCCESS;
    ViChar   tableOfDataValues[100][100];

    GET_RD_BUF( instrumentHandle );
    CHECKRETMSG(RSCMU_getViString(instrumentHandle,
                                  "FETCh:MODulation:PERRor:WCDMa:STATus?" ,
                                  rdBuf,
                                  RD_BUFFER_SIZE,
                                  0
                                  ));

    // Split the multiple data values separated by a comma:
    RSCMU_SeparateDataValues (rdBuf, ",", tableOfDataValues);

    // Get the status of the measurement:
    if ((RSCMU_Status = RSCMU_GetMeasStatVal (measurementStatus, tableOfDataValues, 0)) < 0)
        RETMSG(RSCMU_Status);

    // Counting Mode is not supported by this driver
    // // Get the current value of the counting mode counter (the number of the current statistics cycle):
    // if ((RSCMU_Status = RSCMU_GetCountingNo (CountingNo, tableOfDataValues, 1)) < 0)
    //      RETMSG(RSCMU_Status);

    // Get the current evaluation period within a statistics cycle:
    if ((RSCMU_Status = RSCMU_GetStatisticNo (counter_for_current_statistics, tableOfDataValues, 2)) < 0)
        RETMSG(RSCMU_Status);

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Configure Magn Error SubArray Range                             */
/* Purpose:  This function configures the the READ:SUBarrays...,             */
/*           FETCh:SUBarrays..., and SAMPle:SUBarrays:MODulation:PERRor...   */
/*           commands.                                                       */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_NSig_ConfSubModPerrWcdm (ViSession instrumentHandle,
                                          ViInt32 mode,
                                          ViInt32 number_ofSubranges,
                                          ViInt32 start[], ViInt32 samples[],
                                          ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViChar      buffer[WR_BUFFER_SIZE];
    ViChar*     p2buf;
    ViInt32     i;

    if (RSCMU_invalidViInt32Range (number_ofSubranges, 1, 32))
        RETMSG(RSCMU_ERROR_PARAMETER3);

    p2buf = buffer + sprintf (buffer, "CONFigure:SUBarrays:MODulation:PERRor:WCDMa %s,", RSCMUNB_SubArr[mode]);

    for (i = 0; i < number_ofSubranges; i++)
        p2buf += sprintf(p2buf, "%ld,%ld,", start[i], samples[i]);

    /* remove comma */
    *--p2buf = '\0';

    CHECKRETMSG(RSCMU_sendCmd(instrumentHandle, buffer));

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Get scalar results of Phase Error mesurement                    */
/* Purpose:  This function starts a modulation phase error measurement       */
/*           and/or return all scalar measurement results.                   */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_GetScalModPerrResults (ViSession instrumentHandle,
                                              ViUInt32 mode,
                                              ViReal64 arrayOfResults[],
                                              ViChar errorMessage[])
{
    ViStatus  RSCMU_Status    = VI_SUCCESS;
    ViChar   *strValue;
    ViChar   cmd[50]          = "";
    ViChar   rdBuf[RD_BUFFER_SIZE_ARRAY] = "";
    int      i                = 0;

    CHECK_VI_UINT32_RANGE(instrumentHandle, mode, 0, 2, RSCMU_ERROR_PARAMETER2, errorMessage);

    sprintf(cmd,"%s:MODulation:PERRor:WCDMa?",RSCMUNB_Cmd[mode]);
//   READ[:SCALar]:MODulation:PERRor:WCDMa

    CHECKRETMSG(RSCMU_getViString(instrumentHandle, cmd, rdBuf, RD_BUFFER_SIZE_ARRAY, 0));

    strValue = strtok(rdBuf, ",");

    while (strValue)
    {
        if (strcmp (strValue, "NAN") != 0)
            sscanf (strValue, "%le", &arrayOfResults[i]);
        else
            arrayOfResults[i] = RSCMU_NAN;

        strValue = strtok (NULL, ",");
        i++;
    }

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Get array of Phase Error results (EVM / CodeChannel)            */
/* Purpose:  This function returns the phase error vs. time at fixed,        */
/*           equidistant test points with a 1-chip raster. The results are   */
/*           obtained with the CPICH slot selected via Mod Phase Error CPICH */
/*           Slot (RSCMUNB_Nsig_ConfModPerrContCpic) function. The Phase    */
/*           Error (RMS) values of all CPICH slots can be measured by Get    */
/*           array of Phase Error results (EVM / Slot)                       */
/*           (Nsig_GetArrModPerrSlotResults).                                */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_GetArrModPerrChanResults (ViSession instrumentHandle,
                                                 ViUInt32 mode,
                                                 ViUInt32 statisticMode,
                                                 ViReal64 arrayOfResults[],
                                                 ViChar errorMessage[])
{
    ViStatus  RSCMU_Status    = VI_SUCCESS;
    ViChar   *strValue;
    ViChar   cmd[100]          = "";
    ViChar   rdBuf[RD_BUFFER_SIZE_ARRAY] = "";
    int      i                = 0;

    CHECK_VI_UINT32_RANGE(instrumentHandle, mode, 0, 2, RSCMU_ERROR_PARAMETER2, errorMessage);
    CHECK_VI_UINT32_RANGE(instrumentHandle, statisticMode, 0, 2, RSCMU_ERROR_PARAMETER3, errorMessage);

    sprintf(cmd,"%s:ARRay:MODulation:PERRor:WCDMa:CHANnels:%s?",RSCMUNB_Cmd[mode],RSCMUNB_StatisticsMode[statisticMode]);
//   READ:ARRay:MODulation:PERRor:WCDMa:CHANnels:CURRent?

    CHECKRETMSG(RSCMU_getViString(instrumentHandle, cmd, rdBuf, RD_BUFFER_SIZE_ARRAY, 0));

    strValue = strtok(rdBuf, ",");

    while (strValue)
    {
        if (strcmp (strValue, "NAN") != 0)
            sscanf (strValue, "%le", &arrayOfResults[i]);
        else
            arrayOfResults[i] = RSCMU_NAN;

        strValue = strtok (NULL, ",");
        i++;
    }

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Get Subarray of Phase Error results (EVM / CodeChannel)         */
/* Purpose:  This function returns the phase error versus time in the        */
/*           subranges defined by means of the                               */
/*           CONFigure:SUBarrays:MODulation:PERRor... commands.              */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_GetSubArrModPerrChanResults (ViSession instrumentHandle,
                                                   ViUInt32 mode,
                                                   ViUInt32 statisticMode,
                                                   ViReal64 arrayOfResults[],
                                                   ViChar errorMessage[])
{
    ViStatus  RSCMU_Status    = VI_SUCCESS;
    ViChar   *strValue;
    ViChar   cmd[100]         = "";
    ViChar   rdBuf[RD_BUFFER_SIZE_ARRAY] = "";
    int      i                = 0;

    CHECK_VI_UINT32_RANGE(instrumentHandle, mode, 0, 2, RSCMU_ERROR_PARAMETER2, errorMessage);
    CHECK_VI_UINT32_RANGE(instrumentHandle, statisticMode, 0, 2, RSCMU_ERROR_PARAMETER3, errorMessage);

    sprintf(cmd,"%s:SUBarrays:MODulation:PERRor:WCDMa:CHANnels:%s?",RSCMUNB_Cmd[mode],RSCMUNB_StatisticsMode[statisticMode]);

    CHECKRETMSG(RSCMU_getViString(instrumentHandle, cmd, rdBuf, RD_BUFFER_SIZE_ARRAY, 0));

    strValue = strtok(rdBuf, ",");

    while (strValue)
    {
        if (strcmp (strValue, "NAN") != 0)
            sscanf (strValue, "%.12f", &arrayOfResults[i]);
        else
            arrayOfResults[i] = RSCMU_NAN;

        strValue = strtok (NULL, ",");
        i++;
    }

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Get array of Phase Error results (EVM / Slot)                   */
/* Purpose:  This function returns the Phase Error (RMS) values in all 15    */
/*           CPICH slots of the frame.                                       */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_GetArrModPerrSlotResults (ViSession instrumentHandle,
                                                 ViUInt32 mode,
                                                 ViUInt32 statisticMode,
                                                 ViReal64 arrayOfResults[],
                                                 ViChar errorMessage[])
{
    ViStatus  RSCMU_Status    = VI_SUCCESS;
    ViChar   *strValue;
    ViChar   cmd[100]          = "";
    ViChar   rdBuf[RD_BUFFER_SIZE_ARRAY] = "";
    int      i                = 0;

    CHECK_VI_UINT32_RANGE(instrumentHandle, mode, 0, 2, RSCMU_ERROR_PARAMETER2, errorMessage);
    CHECK_VI_UINT32_RANGE(instrumentHandle, statisticMode, 0, 2, RSCMU_ERROR_PARAMETER3, errorMessage);

    sprintf(cmd,"%s:ARRay:MODulation:PERRor:WCDMa:SLOTs:%s?",RSCMUNB_Cmd[mode],RSCMUNB_StatisticsMode[statisticMode]);
//   READ:ARRay:MODulation:PERRor:WCDMa:SLOTs:CURRent?

    CHECKRETMSG(RSCMU_getViString(instrumentHandle, cmd, rdBuf, RD_BUFFER_SIZE_ARRAY, 0));

    strValue = strtok(rdBuf, ",");

    while (strValue)
    {
        if (strcmp (strValue, "NAN") != 0)
            sscanf (strValue, "%le", &arrayOfResults[i]);
        else
            arrayOfResults[i] = RSCMU_NAN;

        strValue = strtok (NULL, ",");
        i++;
    }

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Modulation Test Model                                           */
/* Purpose:  This function selects the test model f r all MODulation         */
/*           applications.                                                   */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_ConfModTmod(ViSession instrumentHandle,
                                            ViInt32 testModel,
                                            ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setEnum(instrumentHandle,
                              "CONFigure:MODulation:TMODel",
                              testModel,
                              RSCMUNB_TestModel,
                              RSCMU_ERROR_PARAMETER2
                             ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
/* P  O  W  E  R
/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/

/*===========================================================================*/
/* Function: Power Meter Default                                             */
/* Purpose : This function sets all parameters of the Power Control          */
/*           subsystem including all applications to default values (the     */
/*           setting OFF results in an error message).                       */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_DefPowCont (ViSession instrumentHandle,
                                  ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_sendCmd(instrumentHandle,
                                  "DEFault:POWer:CONTrol ON"));

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/*===========================================================================*/
/* Wideband Power Meter
/*===========================================================================*/
/*===========================================================================*/

/*===========================================================================*/
/* Function: Wideband Power Meter Repetition                                 */
/* Purpose:  This function determines the number of statistics cycles, the   */
/*           stop condition and the stepping mode for the measurement.       */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_ConfPowWbanContRep (ViSession instrumentHandle,
                                           ViInt32 repetitionMode,
                                           ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    ViChar    value[42] = "";

    if (RSCMU_invalidViInt32Range (repetitionMode, 0, 1))
        RETMSG(RSCMU_ERROR_PARAMETER2);

    sprintf(value, "%s, NONE, NONE", RSCMUNB_Repetition[repetitionMode] );
    CHECKRETMSG(RSCMU_setViString(instrumentHandle,
                                  "CONFigure:POWer:WBand:CONTrol:REPetition",
                                  value,
                                  RSCMU_ERROR_PARAMETER2
                                  ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Wideband Power Meter Event Reporting                            */
/* Purpose:  This function defines the events generated when the measurement */
/*           is terminated or stopped.                                       */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_ConfPowWbanContErep (ViSession instrumentHandle,
                                            ViInt32 eventReporting,
                                            ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setEnum(instrumentHandle,
                              "CONFigure:POWer:WBand:EREPorting",
                              eventReporting,
                              RSCMUNB_EventReport,
                              RSCMU_ERROR_PARAMETER2
                             ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Initiate Wideband Power Meter Measurement                       */
/* Purpose:  This function starts new power meter wideband measurement.      */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_InitPowWban (ViSession instrumentHandle,
                                    ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setViString(instrumentHandle,
                                  "INITiate:POWer:WBand",
                                  "",
                                  RSCMUNB_ERROR_INITIATE_FAILED
                                  ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Abort Wideband Power Meter Measurement                          */
/* Purpose:  This function aborts running power meter wideband measurement   */
/*           and switch off.                                                 */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_AborPowWban (ViSession instrumentHandle,
                                    ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setViString(instrumentHandle,
                                  "ABORt:POWer:WBand",
                                  "",
                                  RSCMUNB_ERROR_ABORT_FAILED
                                  ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Stop Wideband Power Meter Measurement                           */
/* Purpose:  This function stops the power meter wideband measurement after  */
/*           current stat. cycle.                                            */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_StopPowWban (ViSession instrumentHandle,
                                    ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setViString(instrumentHandle,
                                  "STOP:POWer:WBand",
                                  "",
                                  RSCMUNB_ERROR_STOP_FAILED
                                  ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Continue Wideband Power Meter Measurement                       */
/* Purpose:  This function activates the next power meter wideband           */
/*           measurement step (only stepping mode).                          */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_ContPowWban (ViSession instrumentHandle,
                                    ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setViString(instrumentHandle,
                                  "CONTinue:POWer:WBand",
                                  "",
                                  RSCMUNB_ERROR_CONTINUE_FAILED
                                  ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Get Wideband Power Meter Measurement Status                     */
/* Purpose:  This function returns the status of the power meter wideband    */
/*           measurement.                                                    */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_FetcScalPowWbanStat (ViSession instrumentHandle,
                                            ViInt32 *measurementStatus,
                                            ViChar errorMessage[])
{
    ViStatus  RSCMU_Status    = VI_SUCCESS;
    ViChar   tableOfDataValues[100][100];

    GET_RD_BUF( instrumentHandle );
    CHECKRETMSG(RSCMU_getViString(instrumentHandle,
                                  "FETCh:POWer:WBand:STATus?" ,
                                  rdBuf,
                                  RD_BUFFER_SIZE,
                                  0
                                  ));

    // Split the multiple data values separated by a comma:
    RSCMU_SeparateDataValues (rdBuf, ",", tableOfDataValues);

    // Get the status of the measurement:
    if ((RSCMU_Status = RSCMU_GetMeasStatVal (measurementStatus, tableOfDataValues, 0)) < 0)
        RETMSG(RSCMU_Status);

    // Counting Mode is not supported by this driver
    // // Get the current value of the counting mode counter (the number of the current statistics cycle):
    // if ((RSCMU_Status = RSCMU_GetCountingNo (CountingNo, tableOfDataValues, 1)) < 0)
    //      RETMSG(RSCMU_Status);

    // There is no Statistical Evaluation in this measurement
    // // Get the current evaluation period within a statistics cycle:
    // if ((RSCMU_Status = RSCMU_GetStatisticNo (counter_for_current_statistics, tableOfDataValues, 2)) < 0)
    //      RETMSG(RSCMU_Status);

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Wideband Power Meter Default                                    */
/* Purpose : This function sets all parameters of the subsystem to default   */
/*           values (the setting OFF results in an error message).           */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_DefPowWbanCont (ViSession instrumentHandle,
                                      ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_sendCmd(instrumentHandle,
                                  "DEFault:POWer:WBANd:CONTrol ON"));

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Get scalar result of wideband Power Meter measurement           */
/* Purpose:  This function starts a power meter wideband measurement and/or  */
/*           returns the peak power of the RF signal in a wide frequency     */
/*           range.                                                          */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_GetScalPowWbanResults (ViSession instrumentHandle,
                                              ViUInt32 mode,
                                              ViReal64 *widebandPower,
                                              ViChar errorMessage[])
{
    ViStatus  RSCMU_Status    = VI_SUCCESS;
    ViChar   cmd[50]          = "";
    ViChar   rdBuf[RD_BUFFER_SIZE] = "";

    CHECK_VI_UINT32_RANGE(instrumentHandle, mode, 0, 2, RSCMU_ERROR_PARAMETER2, errorMessage);

    sprintf(cmd,"%s:SCALar:POWer:WBANd?",RSCMUNB_Cmd[mode]);
//   READ[:SCALar]:POWer:WBANd

    CHECKRETMSG(RSCMU_getViString(instrumentHandle, cmd, rdBuf, RD_BUFFER_SIZE, 0));

    if (strcmp (rdBuf, "NAN") != 0)
        sscanf (rdBuf, "%le", widebandPower);
    else
        *widebandPower = RSCMU_NAN;

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/*===========================================================================*/
/* Frequency Selective Power Meter                                           */
/*===========================================================================*/
/*===========================================================================*/

/*===========================================================================*/
/* Function: Frequ. Sel. Power Meter Repetition                              */
/* Purpose:  This function determines the number of statistics cycles, the   */
/*           stop condition and the stepping mode for the power meter        */
/*           frequency selective measurement.                                */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_ConfPowFselContRep (ViSession instrumentHandle,
                                          ViInt32 repetitionMode,
                                          ViInt32 stopCondition, ViInt32 stepMode,
                                          ViChar _VI_FAR errorMessage[])

{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    ViChar    value[42] = "";

    if (RSCMU_invalidViInt32Range (repetitionMode, 0, 1))
        RETMSG(RSCMU_ERROR_PARAMETER2);
    if (RSCMU_invalidViInt32Range (stopCondition, 0, 1))
        RETMSG(RSCMU_ERROR_PARAMETER3);
    if (RSCMU_invalidViInt32Range (stepMode, 0, 1))
        RETMSG(RSCMU_ERROR_PARAMETER4);

    sprintf(value, "%s,%s,%s", RSCMUNB_Repetition[repetitionMode], RSCMUNB_StopCondition[stopCondition],RSCMUNB_StepMode[stepMode]);
    CHECKRETMSG(RSCMU_setViString(instrumentHandle,
                                  "CONFigure:POWer:FSELective:CONTrol:REPetition",
                                  value,
                                  RSCMU_ERROR_PARAMETER2
                                  ));
    RETMSG(RSCMU_Status);

}

/*===========================================================================*/
/* Function: Frequ. Sel. Power Meter Statistical Depth                       */
/* Purpose:  This function defines the number of slots forming a statistics  */
/*           cycle.                                                          */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_ConfPowFselContStat (ViSession instrumentHandle,
                                            ViInt32 statisticalDepth,
                                            ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    CHECKRETMSG(RSCMU_setViInt32(instrumentHandle,
                                  "CONFigure:POWer:FSELective:CONTrol:STATistics",
                                  statisticalDepth,
                                  1, 1000,
                                  RSCMU_ERROR_PARAMETER2
                                  ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Frequ. Sel. Power Meter Event Reporting                         */
/* Purpose:  This function defines the events generated when the power meter */
/*           frequency selective measurement is terminated or stopped.       */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_ConfPowFselContErep (ViSession instrumentHandle,
                                            ViInt32 eventReporting,
                                            ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setEnum(instrumentHandle,
                              "CONFigure:POWer:FSELective:EREPorting",
                              eventReporting,
                              RSCMUNB_EventReport,
                              RSCMU_ERROR_PARAMETER2
                             ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Frequ. Sel. Power Meter Resolution Bandwidth                    */
/* Purpose:  This function defines the bandwidth of the analyzer in the      */
/*           power meter frequency selective application.                    */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_ConfPowFselContBwid (ViSession instrumentHandle,
                                            ViInt32 resolutionBandwidth,
                                            ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setEnum(instrumentHandle,
                              "SENSe:POWer:FSELective:CONTrol:BWIDth:RESolution",
                              resolutionBandwidth,
                              RSCMUNB_ResulutionBandwidth,
                              RSCMU_ERROR_PARAMETER2
                             ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Initiate Frequ. Sel. Power Meter Measurement                    */
/* Purpose:  This function starts a new power meter frequency selective      */
/*           measurement.                                                    */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_InitPowFsel (ViSession instrumentHandle,
                                    ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setViString(instrumentHandle,
                                  "INITiate:POWer:FSELective",
                                  "",
                                  RSCMUNB_ERROR_INITIATE_FAILED
                                  ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Abort Frequ. Sel. Power Meter Measurement                       */
/* Purpose:  This function aborts running power meter frequency selective    */
/*           measurement and switches off.                                   */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_AborPowFsel (ViSession instrumentHandle,
                                    ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setViString(instrumentHandle,
                                  "ABORt:POWer:FSELective",
                                  "",
                                  RSCMUNB_ERROR_ABORT_FAILED
                                  ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Stop Frequ. Sel. Power Meter Measurement                        */
/* Purpose:  This function stops the power meter frequency selective         */
/*           measurement after current stat. cycle.                          */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_StopPowFsel (ViSession instrumentHandle,
                                    ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setViString(instrumentHandle,
                                  "STOP:POWer:FSELective",
                                  "",
                                  RSCMUNB_ERROR_STOP_FAILED
                                  ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Continue Frequ. Sel. Power Meter Measurement                    */
/* Purpose:  This function activates the next power meter frequency          */
/*           selective measurement step.                                     */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_ContPowFsel (ViSession instrumentHandle,
                                    ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setViString(instrumentHandle,
                                  "CONTinue:POWer:FSELective",
                                  "",
                                  RSCMUNB_ERROR_CONTINUE_FAILED
                                  ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Get Frequ. Sel. Power Meter Measurement Status                  */
/* Purpose:  This function returns the status of the power meter frequency   */
/*           selective measurement.                                          */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_FetcScalPowFselStat (ViSession instrumentHandle,
                                            ViInt32 *measurementStatus,
                                            ViInt32 *counter_for_current_statistics,
                                            ViChar errorMessage[])
{
    ViStatus  RSCMU_Status    = VI_SUCCESS;
    ViChar   tableOfDataValues[100][100];

    GET_RD_BUF( instrumentHandle );
    CHECKRETMSG(RSCMU_getViString(instrumentHandle,
                                  "FETCh:POWer:FSELective:STATus?" ,
                                  rdBuf,
                                  RD_BUFFER_SIZE,
                                  0
                                  ));

    // Split the multiple data values separated by a comma:
    RSCMU_SeparateDataValues (rdBuf, ",", tableOfDataValues);

    // Get the status of the measurement:
    if ((RSCMU_Status = RSCMU_GetMeasStatVal (measurementStatus, tableOfDataValues, 0)) < 0)
        RETMSG(RSCMU_Status);

    // Counting Mode is not supported by this driver
    // // Get the current value of the counting mode counter (the number of the current statistics cycle):
    // if ((RSCMU_Status = RSCMU_GetCountingNo (CountingNo, tableOfDataValues, 1)) < 0)
    //      RETMSG(RSCMU_Status);

    // Get the current evaluation period within a statistics cycle:
    if ((RSCMU_Status = RSCMU_GetStatisticNo (counter_for_current_statistics, tableOfDataValues, 2)) < 0)
        RETMSG(RSCMU_Status);

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Frequ Sel Power Meter Default                                   */
/* Purpose : This function sets all parameters of the subsystem to default   */
/*           values (the setting OFF results in an error message).           */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_DefPowFselCont (ViSession instrumentHandle,
                                      ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_sendCmd(instrumentHandle,
                                  "DEFault:POWer:FSELective:CONTrol ON"));

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Get scalar results of Frequ. Sel. Power Meter measurement       */
/* Purpose:  This function starts a power meter frequency selective          */
/*           measurement and/or return all scalar measurement results.       */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_GetScalPowFselResults (ViSession instrumentHandle,
                                              ViUInt32 mode,
                                              ViReal64 arrayOfResults[],
                                              ViChar errorMessage[])
{
    ViStatus  RSCMU_Status    = VI_SUCCESS;
    ViChar   *strValue;
    ViChar   cmd[50]          = "";
    ViChar   rdBuf[RD_BUFFER_SIZE_ARRAY] = "";
    int      i                = 0;

    CHECK_VI_UINT32_RANGE(instrumentHandle, mode, 0, 2, RSCMU_ERROR_PARAMETER2, errorMessage);

    sprintf(cmd,"%s:SCALar:POWer:FSELective?",RSCMUNB_Cmd[mode]);
//   READ[:SCALar]:POWer:FSELective

    CHECKRETMSG(RSCMU_getViString(instrumentHandle, cmd, rdBuf, RD_BUFFER_SIZE_ARRAY, 0));

    strValue = strtok(rdBuf, ",");

    while (strValue)
    {
        if (strcmp (strValue, "NAN") != 0)
            sscanf (strValue, "%le", &arrayOfResults[i]);
        else
            arrayOfResults[i] = RSCMU_NAN;

        strValue = strtok (NULL, ",");
        i++;
    }

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Select User Test Model                                          */
/* Purpose:  This function configures and selects the user test model.       */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_ConfUtmLine (ViSession instrumentHandle, ViInt32 testModel,
                                   ViInt32 line, ViInt32 channel,
                                   ViInt32 symbolRate, ViInt32 chanCode,
                                   ViInt32 timingOffset, ViInt32 pilotBits,
                                   ViInt32 chanStatus,
                                   ViChar _VI_FAR errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    ViChar    wrtBuf[WR_BUFFER_SIZE];

    CHECK_VI_INT32_RANGE(instrumentHandle, testModel, 1, 2, RSCMU_ERROR_PARAMETER2, errorMessage);
    CHECK_VI_INT32_RANGE(instrumentHandle, line, 1, 134, RSCMU_ERROR_PARAMETER3, errorMessage);
    CHECK_VI_INT32_RANGE(instrumentHandle, channel, 0, 9, RSCMU_ERROR_PARAMETER4, errorMessage);
    CHECK_VI_INT32_RANGE(instrumentHandle, symbolRate, 0, 8, RSCMU_ERROR_PARAMETER5, errorMessage);
    CHECK_VI_INT32_RANGE(instrumentHandle, chanCode, 0, 511, RSCMU_ERROR_PARAMETER6, errorMessage);
    CHECK_VI_INT32_RANGE(instrumentHandle, timingOffset, -1, 150, RSCMU_ERROR_PARAMETER7, errorMessage);
    CHECK_VI_INT32_RANGE(instrumentHandle, pilotBits, 0, 6, RSCMU_ERROR_PARAMETER8, errorMessage);

    if (timingOffset >= 0)
    {
        sprintf(wrtBuf, "CONFigure:U%ldTM:LINE%ld %s,%s,%ld,%ld,%s,%s", testModel, line,
        RSCMUNB_UTM_Chan[channel], RSCMUNB_UTM_Srate[symbolRate], chanCode, timingOffset,
        RSCMUNB_UTM_Pilot[pilotBits], RSCMUNB_mode_On_Off[chanStatus]);
    }
    else
    {
        sprintf(wrtBuf, "CONFigure:U%ldTM:LINE%ld %s,%s,%ld,INV,%s,%s", testModel, line,
        RSCMUNB_UTM_Chan[channel], RSCMUNB_UTM_Srate[symbolRate], chanCode,
        RSCMUNB_UTM_Pilot[pilotBits], RSCMUNB_mode_On_Off[chanStatus]);
    }
    CHECKRETMSG(RSCMU_sendCmd(instrumentHandle, wrtBuf));

RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Clear User Test Model                                           */
/* Purpose:  This function clears test model.                                */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_ConfUtmClear (ViSession instrumentHandle,
                                          ViInt32 testModel,
                                          ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    ViChar    wrtBuf[WR_BUFFER_SIZE];

    CHECK_VI_INT32_RANGE(instrumentHandle, testModel, 1, 2, RSCMU_ERROR_PARAMETER2, errorMessage);

    sprintf(wrtBuf, "CONFigure:U%ldTM:CLEar", testModel);
    CHECKRETMSG(RSCMU_sendCmd(instrumentHandle, wrtBuf));

RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/*===========================================================================*/
 /*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
/* Function: HSDPA Start Mode                                                */
/* Purpose:  This function specifies the start mode for the HSDPA            */
/*           measurement.                                                    */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ConfHsdpaStartMode (ViSession instrumentHandle,
                                                         ViInt32 startMode,
                                                         ViInt32 UEID,
                                                         ViInt32 HSFN,
                                                         ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViChar      cmd1[WR_BUFFER_SIZE]    = "";

    if (RSCMU_invalidViInt32Range (startMode, 0, 2))
        RETMSG(RSCMU_ERROR_PARAMETER2);
    if (RSCMU_invalidViInt32Range (UEID, 0, 65535))
        RETMSG(RSCMU_ERROR_PARAMETER3);
    if (RSCMU_invalidViInt32Range (HSFN, 0, 20479))
        RETMSG(RSCMU_ERROR_PARAMETER4);
    
    switch (startMode)
    {
        case RSCMUNB_HSDPA_START_HSFN:
            sprintf(cmd1, "CONFigure:HSDPa:CONTrol:SMODe %s;:CONFigure:HSDPa:CONTrol:HSFN %ld",
                            RSCMUNB_HsdpaMeasMode[startMode], HSFN);
            break;
        case RSCMUNB_HSDPA_START_UEID:
            sprintf(cmd1, "CONFigure:HSDPa:CONTrol:SMODe %s;:CONFigure:HSDPa:CONTrol:UEID %ld",
                            RSCMUNB_HsdpaMeasMode[startMode], UEID);
            break;
        default:
            sprintf(cmd1, "CONFigure:HSDPa:CONTrol:SMODe %s",
                            RSCMUNB_HsdpaMeasMode[startMode]);
            break;
    }
    
    CHECKRETMSG(RSCMU_sendCmd(instrumentHandle, cmd1));
    
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: HSDPA Nr. of UE-IDs                                             */
/* Purpose:  This function specifies the total number of analyzed UE-IDs.    */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ConfHsdpaNrUeids (ViSession instrumentHandle,
                                                ViInt32 nr_ofUEIDs,
                                                ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    
    CHECKRETMSG(RSCMU_setViInt32(instrumentHandle,
                                  "CONFigure:HSDPa:CONTrol:NRUEID",
                                  nr_ofUEIDs,
                                  1, 128,
                                  RSCMU_ERROR_PARAMETER2
                                  ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: HSDPA UE-ID Index Table                                         */
/* Purpose:  This function specifies a single UE-ID value for the UE-ID      */
/*           index table.                                                    */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ConfHsdpaUeidIndexTable (ViSession instrumentHandle,
                                                       ViInt32 nr_ofUEIDs,
                                                       ViInt32 UEIDIndexTable,
                                                       ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViChar      cmd1[WR_BUFFER_SIZE]    = "";

    if (RSCMU_invalidViInt32Range (nr_ofUEIDs, 1, 128))
        RETMSG(RSCMU_ERROR_PARAMETER2);
    if (RSCMU_invalidViInt32Range (UEIDIndexTable, 0, 65535))
        RETMSG(RSCMU_ERROR_PARAMETER3);

    sprintf(cmd1, "CONFigure:HSDPa:CONTrol:UITable%ld %ld", nr_ofUEIDs, UEIDIndexTable);
    
    CHECKRETMSG(RSCMU_sendCmd(instrumentHandle, cmd1));
    
    RETMSG(RSCMU_Status);
}


/*===========================================================================*/
/* Function: HSDPA UE-ID Index Table All                                     */
/* Purpose:  This function specifies the UE-IDs for the UE-ID index table.   */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ConfHsdpaUeidIndexTableAll (ViSession instrumentHandle,
                                                           ViInt32 number_ofEntries,
                                                           ViInt32 UEIDIndexTable[],
                                                           ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    ViInt32   i = 0;
    ViChar    *buffer, *p2buf;
    
    p2buf = buffer = (ViChar*) malloc (number_ofEntries * 11);
    
    for (i = 0; i < number_ofEntries; i++)
        p2buf += sprintf (p2buf, "%ld,", UEIDIndexTable [i]);
    
    *--p2buf = '\n';    
    
    RSCMU_Status = RSCMU_setViString(instrumentHandle,
                                  "CONFigure:HSDPa:CONTrol:UITable:ALL",
                                  buffer,
                                  RSCMU_ERROR_PARAMETER2
                                  );
    free (buffer);
    
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: HS-SCCH Repetition                                              */
/* Purpose:  This function determines the number of statistics cycles, the   */
/*           stop condition and the stepping mode for the measurement.       */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ConfHsscchContRep (ViSession instrumentHandle,
                                           ViInt32 repetitionMode,
                                           ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    ViChar    value[42] = "";

    if (RSCMU_invalidViInt32Range (repetitionMode, 0, 1))
        RETMSG(RSCMU_ERROR_PARAMETER2);

    sprintf(value, "%s, NONE, NONE", RSCMUNB_Repetition[repetitionMode]);
    
    CHECKRETMSG(RSCMU_setViString(instrumentHandle,
                                  "CONFigure:HSDPa:HINFormation:CONTrol:REPetition",
                                  value,
                                  RSCMU_ERROR_PARAMETER2
                                  ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: HS-SCCH Display Mode                                            */
/* Purpose:  This function selects the display mode for the HS-SCCH          */
/*           information table. UEID possibly extends the single shot        */
/*           measurement time.                                               */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ConfHsscchDisplayMode (ViSession instrumentHandle,
                                                      ViInt32 displayMode,
                                                      ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setEnum(instrumentHandle,
                              "CONFigure:HSDPa:HINFormation:CONTrol:DMODe",
                              displayMode,
                              RSCMUNB_HsscchDisplayMode,
                              RSCMU_ERROR_PARAMETER2
                             ));
    RETMSG(RSCMU_Status);
}


/*===========================================================================*/
/* Function: HS-SCCH Event Reporting                                         */
/* Purpose:  This function defines the event generated when the measurement  */
/*           is terminated or stopped.                                       */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ConfHsscchContErep (ViSession instrumentHandle,
                                            ViInt32 eventReporting,
                                            ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setEnum(instrumentHandle,
                              "CONFigure:HSDPa:HINFormation:EREPorting",
                              eventReporting,
                              RSCMUNB_EventReport,
                              RSCMU_ERROR_PARAMETER2
                             ));
    RETMSG(RSCMU_Status);
}
 
 
/*===========================================================================*/
/* Function: Initiate HS-SCCH                                                */
/* Purpose:  This function starts new measurement.                           */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_InitHsscch (ViSession instrumentHandle,
                                    ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setViString(instrumentHandle,
                                  "INITiate:HSDPa:HINFormation",
                                  "",
                                  RSCMUNB_ERROR_INITIATE_FAILED
                                  ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Abort HS-SCCH                                                   */
/* Purpose:  This function aborts running measurement and switch off.        */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_AborHsscch (ViSession instrumentHandle,
                                    ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setViString(instrumentHandle,
                                  "ABORt:HSDPa:HINFormation",
                                  "",
                                  RSCMUNB_ERROR_ABORT_FAILED
                                  ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Stop HS-SCCH                                                    */
/* Purpose:  This function stops the measurement after current stat. cycle.  */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_StopHsscch (ViSession instrumentHandle,
                                    ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setViString(instrumentHandle,
                                  "STOP:HSDPa:HINFormation",
                                  "",
                                  RSCMUNB_ERROR_STOP_FAILED
                                  ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Continue HS-SCCH                                                */
/* Purpose:  This function activates the next measurement step               */
/*           (only stepping mode).                                           */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ContHsscch (ViSession instrumentHandle,
                                    ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setViString(instrumentHandle,
                                  "CONTinue:HSDPa:HINFormation",
                                  "",
                                  RSCMUNB_ERROR_CONTINUE_FAILED
                                  ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Get HS-SCCH Measurement Status                                  */
/* Purpose:  This function returns the status of the measurement.            */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_FetcScalHsscchStat (ViSession instrumentHandle,
                                            ViInt32 *measurementStatus,
                                            ViInt32 *counter_for_current_statistics,
                                            ViChar errorMessage[])
{
    ViStatus  RSCMU_Status    = VI_SUCCESS;
    ViChar   tableOfDataValues[100][100];

    GET_RD_BUF( instrumentHandle );
    CHECKRETMSG(RSCMU_getViString(instrumentHandle,
                                  "FETCh:HSDPa:HINFormation:STATus?" ,
                                  rdBuf,
                                  RD_BUFFER_SIZE,
                                  0
                                  ));

    // Split the multiple data values separated by a comma:
    RSCMU_SeparateDataValues (rdBuf, ",", tableOfDataValues);

    // Get the status of the measurement:
    if ((RSCMU_Status = RSCMU_GetMeasStatVal (measurementStatus, tableOfDataValues, 0)) < 0)
        RETMSG(RSCMU_Status);

    // Counting Mode is not supported by this driver
    // // Get the current value of the counting mode counter (the number of the current statistics cycle):
    // if ((RSCMU_Status = RSCMU_GetCountingNo (CountingNo, tableOfDataValues, 1)) < 0)
    //      RETMSG(RSCMU_Status);

    // Get the current evaluation period within a statistics cycle:
    if ((RSCMU_Status = RSCMU_GetStatisticNo (counter_for_current_statistics, tableOfDataValues, 2)) < 0)
        RETMSG(RSCMU_Status);

    RETMSG(RSCMU_Status);
} 
 

/*===========================================================================*/
/* Function: UE IDs for All HS-SCCHs                                         */
/* Purpose:  This function starts a HS-SCCH Information measurement and      */
/*           return the decoded UE-ID values for all four HS-SCCHs.          */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_GetArrHsscchUeidAllResults
                                                         (ViSession instrumentHandle,
                                                         ViUInt32 mode,
                                                         ViInt32 UEIDs_for_allHSSCCHs[], 
                                                         ViChar  errorMessage[])                                              
{
    ViStatus  RSCMU_Status    = VI_SUCCESS;
    ViChar   *strValue;
    ViChar   cmd[50]          = "";
    ViChar   rdBuf[RD_BUFFER_SIZE_ARRAY] = "";
    int      i                = 0;

    CHECK_VI_UINT32_RANGE(instrumentHandle, mode, 0, 1, RSCMU_ERROR_PARAMETER2, errorMessage);

    sprintf(cmd,"%s:ARRay:HSDPa:HINFormation:ALL?",RSCMUNB_Cmd[mode]);

    CHECKRETMSG(RSCMU_getViString(instrumentHandle, cmd, rdBuf, RD_BUFFER_SIZE_ARRAY, 0));

    strValue = strtok(rdBuf, ",");

    while (strValue)
    {
        if (strcmp (strValue, "NAN") != 0)
            sscanf (strValue, "%ld", &UEIDs_for_allHSSCCHs[i]);
        else
            UEIDs_for_allHSSCCHs[i] = RSCMU_NAN;

        strValue = strtok (NULL, ",");
        i++;
    }

    RETMSG(RSCMU_Status);
}


/*===========================================================================*/
/* Function: Detailed Info                                                   */
/* Purpose:  This function starts a HS-SCCH Information measurement and      */
/*           return the detailed information for selected channel.           */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_GetArrHsscchDetailedInfo (ViSession instrumentHandle,
                                                              ViUInt32 mode,
                                                              ViInt32 channel,
                                                              ViInt32 detailedInfo[],
                                                              ViChar errorMessage[])
{
    ViStatus  RSCMU_Status    = VI_SUCCESS;
    ViChar   *strValue;
    ViChar   cmd[50]          = "";
    ViChar   rdBuf[2*RD_BUFFER_SIZE_ARRAY] = "";
    int      i                = 0;

    CHECK_VI_UINT32_RANGE(instrumentHandle, mode, 0, 1, RSCMU_ERROR_PARAMETER2, errorMessage);
    CHECK_VI_UINT32_RANGE(instrumentHandle, channel, 1, 4, RSCMU_ERROR_PARAMETER3, errorMessage);

    sprintf(cmd,"%s:ARR:HSDP:HINF%ld? DINF",RSCMUNB_Cmd[mode], channel);

    CHECKRETMSG(RSCMU_getViString(instrumentHandle, cmd, rdBuf, 2*RD_BUFFER_SIZE_ARRAY, 0));

    strValue = strtok(rdBuf, ",");
    
    i = 0;
    while (strValue != NULL)
    {
        detailedInfo[i++] = atol (strValue);                       /*HSFN*/
        strValue = strtok (NULL,",");                              
        detailedInfo[i++] = atol (strValue);                       /*UEID*/
        strValue = strtok (NULL,",");
        detailedInfo[i++] = atol (strValue);                       /*New Data Indicator*/
        strValue = strtok (NULL,",");
        detailedInfo[i++] = strcmp (strValue, "FALSE") ? 1 : 0;    /*Retransmission*/
        strValue = strtok (NULL,",");
        detailedInfo[i++] = strcmp (strValue, "QPSK") ? 1 : 0;     /*Modulation Type*/
        strValue = strtok (NULL,",");
        detailedInfo[i++] = atol (strValue);                       /*Redundancy Version*/
        strValue = strtok (NULL,",");
        detailedInfo[i++] = atol (strValue);                       /*s*/
        strValue = strtok (NULL,",");
        detailedInfo[i++] = atol (strValue);                       /*r*/
        strValue = strtok (NULL,",");
        detailedInfo[i++] = atol (strValue);                       /*b*/
        strValue = strtok (NULL,",");
        detailedInfo[i++] = atol (strValue);                       /*HARQ Process Identifier*/
        strValue = strtok (NULL,",");
        detailedInfo[i++] = atol (strValue);                       /*Block size index*/
        strValue = strtok (NULL,",");
        detailedInfo[i++] = atol (strValue);                       /*Block size*/
        strValue = strtok (NULL,",");
        detailedInfo[i++] = atol (strValue);                       /*Chanel code offset*/
        strValue = strtok (NULL,",");
        detailedInfo[i++] = atol (strValue);                       /*Number of channels*/
        strValue = strtok (NULL,",");
        detailedInfo[i++] = strcmp (strValue, "FALSE") ? 1 : 0;    /*CRC error*/
        strValue = strtok (NULL,",");                              
    }

    RETMSG(RSCMU_Status);
}


/*===========================================================================*/
/* Function: HS-SCCH Received Subframes                                      */
/* Purpose:  This function starts a HS-SCCH Information measurement and      */
/*           return the number of received HS-SCCH subframes.                */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_GetScalHsscchRecSubframesResults (ViSession instrumentHandle,
                                                                 ViUInt32 mode,
                                                                 ViInt32 *receivedSubframes,
                                                                 ViChar errorMessage[])
{
    ViStatus  RSCMU_Status    = VI_SUCCESS;
    ViChar   cmd[50]          = "";
    ViChar   rdBuf[RD_BUFFER_SIZE] = "";

    CHECK_VI_UINT32_RANGE(instrumentHandle, mode, 0, 1, RSCMU_ERROR_PARAMETER2, errorMessage);

    sprintf(cmd,"%s:SCALar:HSDPa:HINFormation:RSUBframes?",RSCMUNB_Cmd[mode]);

    CHECKRETMSG(RSCMU_getViString(instrumentHandle, cmd, rdBuf, RD_BUFFER_SIZE, 0));

    if (strcmp (rdBuf, "NAN") != 0)
        sscanf (rdBuf, "%ld", receivedSubframes);
    else
        *receivedSubframes = RSCMU_NAN;

    RETMSG(RSCMU_Status);
}


/*===========================================================================*/
/* Function: HS-PDSCH Repetition                                             */
/* Purpose:  This function determines the number of statistics cycles, the   *
/*           stop condition and the stepping mode for the measurement.       */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ConfHspdschContRep (ViSession instrumentHandle,
                                                   ViInt32 repetitionMode,
                                                   ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    ViChar    value[42] = "";

    if (RSCMU_invalidViInt32Range (repetitionMode, 0, 1))
        RETMSG(RSCMU_ERROR_PARAMETER2);

    sprintf(value, "%s, NONE, NONE", RSCMUNB_Repetition[repetitionMode]);
    
    CHECKRETMSG(RSCMU_setViString(instrumentHandle,
                                  "CONFigure:HSDPa:HTHRoughput:CONTrol:REPetition",
                                  value,
                                  RSCMU_ERROR_PARAMETER2
                                  ));
    RETMSG(RSCMU_Status);
}


/*===========================================================================*/
/* Function: HS-PDSCH Record Subframes                                       */
/* Purpose:  This function specifies the number of subframes recorded in a   */
/*           Continuous or Single Shot measurement.                          */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ConfHspdschRecSubframes (ViSession instrumentHandle,
                                                        ViInt32 mode,
                                                        ViInt32 recordSubframes,
                                                        ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViChar      cmd1[WR_BUFFER_SIZE]    = "";

    if (RSCMU_invalidViInt32Range (mode, 0, 1))
        RETMSG(RSCMU_ERROR_PARAMETER2);
    if (RSCMU_invalidViInt32Range (recordSubframes, 0, 20480))
        RETMSG(RSCMU_ERROR_PARAMETER3);

    if (mode == RSCMU_CONTINUOUS)
        sprintf(cmd1, "CONFigure:HSDPa:HTHRoughput:CONTrol:RSFCont %ld", recordSubframes);
    else        
        sprintf(cmd1, "CONFigure:HSDPa:HTHRoughput:CONTrol:RSFSingle %ld", recordSubframes);
        
    CHECKRETMSG(RSCMU_sendCmd(instrumentHandle, cmd1));
    
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: HS-PDSCH Event Reporting                                        */
/* Purpose:  This function defines the event generated when the measurement  */
/*           is terminated or stopped.                                       */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ConfHspdschErep (ViSession instrumentHandle,
                                                ViInt32 eventReporting,
                                                ViChar _VI_FAR errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setEnum(instrumentHandle,
                              "CONFigure:HSDPa:HTHRoughput:EREPorting",
                              eventReporting,
                              RSCMUNB_EventReport,
                              RSCMU_ERROR_PARAMETER2
                             ));
    RETMSG(RSCMU_Status);
}


/*===========================================================================*/
/* Function: Get HS-SCCH Measurement Status                                  */
/* Purpose:  This function returns the status of the measurement.            */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_FetcScalHspdschStat (ViSession instrumentHandle,
                                                    ViInt32 *measurementStatus,
                                                    ViInt32 *counter_for_current_statistics,
                                                    ViChar errorMessage[])
{
    ViStatus  RSCMU_Status    = VI_SUCCESS;
    ViChar   tableOfDataValues[100][100];

    GET_RD_BUF( instrumentHandle );
    CHECKRETMSG(RSCMU_getViString(instrumentHandle,
                                  "FETCh:HSDPa:HTHRoughput:STATus?" ,
                                  rdBuf,
                                  RD_BUFFER_SIZE,
                                  0
                                  ));

    // Split the multiple data values separated by a comma:
    RSCMU_SeparateDataValues (rdBuf, ",", tableOfDataValues);

    // Get the status of the measurement:
    if ((RSCMU_Status = RSCMU_GetMeasStatVal (measurementStatus, tableOfDataValues, 0)) < 0)
        RETMSG(RSCMU_Status);

    // Counting Mode is not supported by this driver
    // // Get the current value of the counting mode counter (the number of the current statistics cycle):
    // if ((RSCMU_Status = RSCMU_GetCountingNo (CountingNo, tableOfDataValues, 1)) < 0)
    //      RETMSG(RSCMU_Status);

    // Get the current evaluation period within a statistics cycle:
    if ((RSCMU_Status = RSCMU_GetStatisticNo (counter_for_current_statistics, tableOfDataValues, 2)) < 0)
        RETMSG(RSCMU_Status);

    RETMSG(RSCMU_Status);
} 

 
/*===========================================================================*/
/* Function: Initiate HS-PDSCH Measurement                                   */
/* Purpose:  This function starts new measurement.                           */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_InitHspdsch (ViSession instrumentHandle,
                                    ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setViString(instrumentHandle,
                                  "INITiate:HSDPa:HTHRoughput",
                                  "",
                                  RSCMUNB_ERROR_INITIATE_FAILED
                                  ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Abort HS-PDSCH Measurement                                      */
/* Purpose:  This function aborts running measurement and switch off.        */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_AborHspdsch (ViSession instrumentHandle,
                                    ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setViString(instrumentHandle,
                                  "ABORt:HSDPa:HTHRoughput",
                                  "",
                                  RSCMUNB_ERROR_ABORT_FAILED
                                  ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Stop HS-PDSCH Measurement                                       */
/* Purpose:  This function stops the measurement after current stat. cycle.  */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_StopHspdsch (ViSession instrumentHandle,
                                    ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setViString(instrumentHandle,
                                  "STOP:HSDPa:HTHRoughput",
                                  "",
                                  RSCMUNB_ERROR_STOP_FAILED
                                  ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Continue HS-PDSCH Measurement                                   */
/* Purpose:  This function activates the next measurement step               */
/*           (only stepping mode).                                           */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ContHspdsch (ViSession instrumentHandle,
                                    ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setViString(instrumentHandle,
                                  "CONTinue:HSDPa:HTHRoughput",
                                  "",
                                  RSCMUNB_ERROR_CONTINUE_FAILED
                                  ));
    RETMSG(RSCMU_Status);
} 

/*===========================================================================*/
/* Function: HS-PDSCH Data Rate                                              */
/* Purpose:  This function start a HS-PDSCH Throughput measurement and return*/
/*           the current, average, and maximum data rates for all UE-IDs in  */
/*           the index table.                                                */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_GetArrHspdschDrate (ViSession instrumentHandle,
                                                   ViUInt32 mode,
                                                   ViInt32 dataRate[],
                                                   ViChar errorMessage[])
{
    ViStatus  RSCMU_Status    = VI_SUCCESS;
    ViChar   *strValue;
    ViChar   cmd[50]          = "";
    ViChar   rdBuf[RD_BUFFER_SIZE_ARRAY] = "";
    int      i                = 0;

    CHECK_VI_UINT32_RANGE(instrumentHandle, mode, 0, 1, RSCMU_ERROR_PARAMETER2, errorMessage);

    sprintf(cmd,"%s:ARRay:HSDPa:HTHRoughput:DRATe?",RSCMUNB_Cmd[mode]);

    CHECKRETMSG(RSCMU_getViString(instrumentHandle, cmd, rdBuf, RD_BUFFER_SIZE_ARRAY, 0));

    strValue = strtok(rdBuf, ",");

    while (strValue)
    {
        if (strcmp (strValue, "NAN") != 0)
            sscanf (strValue, "%ld", &dataRate[i]);
        else
            dataRate[i] = RSCMU_NAN;

        strValue = strtok (NULL, ",");
        i++;
    }

    RETMSG(RSCMU_Status);
    
}


/*===========================================================================*/
/* Function: Get Array of HS-PDSCH All Datarate Meas                         */
/* Purpose:  This function starts a HS-PDSCH Throughput measurement and      */
/*           return the current, average, and maximum data rates for all     */
/*           UE-IDs in the index table.                                      */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_GetArrHspdschDrateThrMeas (ViSession instrumentHandle,
                                                          ViUInt32 mode,
                                                          ViUInt32 statisticMode,
                                                          ViReal64 arrayOfResults[],
                                                          ViChar errorMessage[])
{
    ViStatus  RSCMU_Status    = VI_SUCCESS;
    ViChar   *strValue;
    ViChar   cmd[100]         = "";
    ViChar   rdBuf[RD_BUFFER_SIZE_ARRAY] = "";
    int      i                = 0;

    CHECK_VI_UINT32_RANGE(instrumentHandle, mode, 0, 2, RSCMU_ERROR_PARAMETER2, errorMessage);
    CHECK_VI_UINT32_RANGE(instrumentHandle, statisticMode, 0, 2, RSCMU_ERROR_PARAMETER3, errorMessage);

    sprintf(cmd,"%s:ARRay:HSDPa:HTHRoughput:DRATe:%s?",RSCMUNB_Cmd[mode],RSCMUNB_StatisticsMode[statisticMode + 3]);

    CHECKRETMSG(RSCMU_getViString(instrumentHandle, cmd, rdBuf, RD_BUFFER_SIZE_ARRAY, 0));

    strValue = strtok(rdBuf, ",");

    while (strValue)
    {
        if (strcmp (strValue, "NAN") != 0)
            sscanf (strValue, "%le", &arrayOfResults[i]);
        else
            arrayOfResults[i] = RSCMU_NAN;

        strValue = strtok (NULL, ",");
        i++;
    }

    RETMSG(RSCMU_Status);
}


/*===========================================================================*/
/* Function: HS-PDSCH Throughput                                             */
/* Purpose:  This function starts a HS-PDSCH Throughput measurement and      */
/*           return the current, average, minimum, and maximum throughputs   */
/*           for all UE-IDs in the index table.                              */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_GetArrHspdschThroughput (ViSession instrumentHandle,
                                                        ViUInt32 mode,
                                                        ViReal64 throughput[],
                                                        ViChar errorMessage[])
{
    ViStatus  RSCMU_Status    = VI_SUCCESS;
    ViChar   *strValue;
    ViChar   cmd[50]          = "";
    ViChar   rdBuf[RD_BUFFER_SIZE_ARRAY] = "";
    int      i                = 0;

    CHECK_VI_UINT32_RANGE(instrumentHandle, mode, 0, 1, RSCMU_ERROR_PARAMETER2, errorMessage);

    sprintf(cmd,"%s:ARRay:HSDPa:HTHRoughput:THRoughput?",RSCMUNB_Cmd[mode]);

    CHECKRETMSG(RSCMU_getViString(instrumentHandle, cmd, rdBuf, RD_BUFFER_SIZE_ARRAY, 0));

    strValue = strtok(rdBuf, ",");

    while (strValue)
    {
        if (strcmp (strValue, "NAN") != 0)
            sscanf (strValue, "%le", &throughput[i]);
        else
            throughput[i] = RSCMU_NAN;

        strValue = strtok (NULL, ",");
        i++;
    }

    RETMSG(RSCMU_Status);
    
}

/*===========================================================================*/
/* Function: Get Array of HS-PDSCH All Throughput Meas                       */
/* Purpose:  This function starts a HS-PDSCH Throughput measurement and      */
/*           return the current, average, minimum, and maximum throughputs   */
/*           for all UE-IDs in the index table.                              */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_GetArrHspdschThroughputMeas (ViSession instrumentHandle,
                                                          ViUInt32 mode,
                                                          ViUInt32 statisticMode,
                                                          ViReal64 arrayOfResults[],
                                                          ViChar errorMessage[])

{
    ViStatus  RSCMU_Status    = VI_SUCCESS;
    ViChar   *strValue;
    ViChar   cmd[100]         = "";
    ViChar   rdBuf[RD_BUFFER_SIZE_ARRAY] = "";
    int      i                = 0;

    CHECK_VI_UINT32_RANGE(instrumentHandle, mode, 0, 2, RSCMU_ERROR_PARAMETER2, errorMessage);
    CHECK_VI_UINT32_RANGE(instrumentHandle, statisticMode, 0, 3, RSCMU_ERROR_PARAMETER3, errorMessage);

    sprintf(cmd,"%s:ARRay:HSDPa:HTHRoughput:THRoughput:%s?",RSCMUNB_Cmd[mode],RSCMUNB_StatisticsMode[statisticMode + 3]);

    CHECKRETMSG(RSCMU_getViString(instrumentHandle, cmd, rdBuf, RD_BUFFER_SIZE_ARRAY, 0));

    strValue = strtok(rdBuf, ",");

    while (strValue)
    {
        if (strcmp (strValue, "NAN") != 0)
            sscanf (strValue, "%le", &arrayOfResults[i]);
        else
            arrayOfResults[i] = RSCMU_NAN;

        strValue = strtok (NULL, ",");
        i++;
    }

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Total Data Rate and Throughput                                  */
/* Purpose:  This function starts a HS-PDSCH Throughput measurement and      */
/*           return all total throughputs (7 values).                        */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_GetArrHspdschTotalDrate (ViSession instrumentHandle,
                                                        ViUInt32 mode,
                                                        ViReal64 totalDataRateAndThroughput[],
                                                        ViChar errorMessage[])
{

    ViStatus  RSCMU_Status    = VI_SUCCESS;
    ViChar   cmd[50]          = "";
    ViChar   rdBuf[RD_BUFFER_SIZE] = "";

    CHECK_VI_UINT32_RANGE(instrumentHandle, mode, 0, 1, RSCMU_ERROR_PARAMETER2, errorMessage);

    sprintf(cmd,"%s:SCALar:HSDPa:HTHRoughput:TOTal?",RSCMUNB_Cmd[mode]);

    CHECKRETMSG(RSCMU_getViString(instrumentHandle, cmd, rdBuf, RD_BUFFER_SIZE, 0));

    if (strcmp (rdBuf, "NAN") != 0)
        sscanf (rdBuf, "%le", totalDataRateAndThroughput);
    else
        *totalDataRateAndThroughput = RSCMU_NAN;

    RETMSG(RSCMU_Status);

}

/*===========================================================================*/
/* Function: NBSignal Main UE-ID                                             */
/* Purpose:  This function specifies the first entry in the UE-ID Index Table*/
/*           (CONFigure:HSDPa:CONTrol:UITable. The main UE-ID is also used to*/
/*           generate the HS-UE-ID Trigger events.                           */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_NbsigMainUeid (ViSession instrumentHandle,
                                              ViInt32 transmitDistance,
                                              ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setViInt32(instrumentHandle,
                              "CONFigure:NBSignal:HSDPa:MUID",
                              transmitDistance,
                              0,
                              65535,
                              RSCMU_ERROR_PARAMETER2));
    RETMSG(RSCMU_Status);    
}


/*===========================================================================*/
/* Function: NBSignal HSDPA UE-ID Index Table                                */
/* Purpose:  This function selects the channelization codes for up to four   */
/*           HS-SCCHs in the Node B signal numbered <nr> = 1 to 4. Channels  */
/*           above the specified no. of HS-SCCHs  are not decoded.           */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_NbsigHsscchChCode (ViSession instrumentHandle,
                                                  ViInt32 nodeBSignalNumber,
                                                  ViInt32 transmitDistance,
                                                  ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViChar      cmd1[WR_BUFFER_SIZE]    = "";

    if (RSCMU_invalidViInt32Range (nodeBSignalNumber, 1, 4))
        RETMSG(RSCMU_ERROR_PARAMETER2);
    if (RSCMU_invalidViInt32Range (transmitDistance, 0, 127))
        RETMSG(RSCMU_ERROR_PARAMETER3);

    sprintf(cmd1, "CONFigure:NBSignal:HSDPa:HSSCch%ld %ld", nodeBSignalNumber, transmitDistance);
    
    CHECKRETMSG(RSCMU_sendCmd(instrumentHandle, cmd1));
    
    RETMSG(RSCMU_Status);
}


/*===========================================================================*/
/* Function: NBSignal Number of HS-SCCHs                                     */
/* Purpose:  This function selects the total number of decoded HS-SCCHs.     */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_NbsigNumberHsscch (ViSession instrumentHandle,
                                                  ViInt32 nodeBSignalNumber,
                                                  ViChar errorMessage[])

{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setViInt32(instrumentHandle,
                              "CONFigure:NBSignal:HSDPa:HSSCch:NUMBer",
                              nodeBSignalNumber,
                              1,
                              4,
                              RSCMU_ERROR_PARAMETER2));
    RETMSG(RSCMU_Status);    
}









/*===========================================================================*/
/* Function: RSCMUNB_GetResourceName                                         */
/* Purpose:  This function returns resource descriptor for WCDMA NodeB option.*/
/*           Session parameter must contain at least 256 elements ViChar[256]*/
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_GetResourceName (ViSession instrSession,
                                           ViSession instrSessionBase,
                                           ViChar _VI_FAR sessionName[])
{
  ViStatus    RSCMU_Status    = VI_SUCCESS;
  ViInt32     comp;

  struct RSCMU_Base_Globals  *p_globalvarBase;
  struct RSCMU_RF_NSig_Globals  *p_globalvarNB;

  /* Get a copy of the global data for base session. */
  CHECKRET(viGetAttribute( instrSessionBase
                                     , VI_ATTR_USER_DATA
                                     , (ViAddr)&p_globalvarBase
                                     ));
  /* Get a copy of the global data for this session. */
  CHECKRET(viGetAttribute( instrSession
                                     , VI_ATTR_USER_DATA
                                     , (ViAddr)&p_globalvarNB
                                     ));

  // Hardware Connection
  comp = strncmp("GPIB", p_globalvarBase->RSCMU_tGlobal.RSCMU_SCPIConnection, 4);

  if( comp == 0 )
  {
    sprintf( sessionName
            , "%s::%ld::%ld::INSTR"
            , p_globalvarBase->RSCMU_tGlobal.RSCMU_SCPIConnection
            , p_globalvarBase->RSCMU_tGlobal.RSCMU_primary_address
            , p_globalvarNB->RSCMU_tGlobal.RSCMU_secondary_address
           );
  }
  else
  {
    sprintf( sessionName
            , "%s::INSTR"
            , p_globalvarBase->RSCMU_tGlobal.RSCMU_SCPIConnection
           );
  }

  return RSCMU_Status;
}

/* ----------------------------------------------------------------------------
 * WCDMA MEASUREMENTS WITH SIGNALLING
 *
 * In the Signalling mode, the R&S CMU can synchronize to the DL
 * signal from the Node B and is able to transmit and receive
 * signalling information and data.
 */

/* ----------------------------------------------------------------------------
 * Connection Control
 * ----------------------------------------------------------------------------
 */
/*===========================================================================*/
/*===========================================================================*/
/* Connection                                                                */
/*===========================================================================*/
/*===========================================================================*/

/*===========================================================================*/
/* Function: Signalling Control                                              */
/* Purpose:  This function changes between the different signalling states   */
/*           of the CMU.                                                     */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ProcSignAct (ViSession instrumentHandle, ViInt32 state,
                                  ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setEnum(instrumentHandle,
                                ":PROCedure:SIGNalling:ACTion",
                                state,
                                RSCMUNB_Sig_Act,
                                RSCMU_ERROR_PARAMETER2));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Signalling State                                                */
/* Purpose:  This function returns the current signalling state. The query   */
/*           can be refined using [SENSe:]SIGNalling:STO?                    */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_SensSignStat (ViSession instrumentHandle,
                                   ViInt32 *state,
                                   ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_getEnum(instrumentHandle,
                                ":SENSe:SIGNalling:STATe?",
                                state,
                                RSCMUNB_Sig_Stat,
                                RSCMU_ERROR_INSTRUMENT_ERROR));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Sync Monitor                                                    */
/* Purpose:  This function returns two values, representing the current      */
/*           status of CPICH and BCH synchronization.                        */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_SensSignSto (ViSession instrumentHandle,
                                  ViInt32 *CPICHState,
                                  ViInt32 *BCHState,
                                  ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    ViChar    buffer[RD_BUFFER_SIZE];
    ViChar    *pbuffer;

    if (!CPICHState)
        RETMSG (RSCMU_ERROR_PARAMETER2);
    if (!BCHState)
        RETMSG (RSCMU_ERROR_PARAMETER3);

    CHECKRETMSG(RSCMU_getViString(instrumentHandle,
                                ":SENSe:SIGNalling:STO?",
                                buffer,
                                RD_BUFFER_SIZE,
                                RSCMU_ERROR_PARAMETER2));

    pbuffer=buffer;
    pbuffer=strtok(buffer,",");
    CHECKRETMSG (RSCMU_getIndexFromString (instrumentHandle, pbuffer, RSCMUNB_Sig_Sto_CPICH, CPICHState));
    pbuffer=strtok(NULL,",");
    CHECKRETMSG (RSCMU_getIndexFromString (instrumentHandle, pbuffer, RSCMUNB_Sig_Sto_BCH, BCHState));

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: CPICH Power                                                     */
/* Purpose:  This function returns the average CPICH power measured while the*/
/*           R&S CMU synchronizes to the Node B signal.                      */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_SensSignMinfCpow (ViSession instrumentHandle,
                                       ViReal64 *power,
                                       ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_getViReal64(instrumentHandle,
                                ":SENSe:SIGNalling:MINFo:CPOWer?",
                                power,
                                RSCMU_ERROR_PARAMETER2));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: CC8 SIR Signal                                                  */
/* Purpose:  This function returns the CC8-SIR detected while the R&S CMU    */
/*           synchronizes to the Node B signal.                              */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_SensSignMinfCsir (ViSession instrumentHandle,
                                       ViReal64 *signal,
                                       ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_getViReal64(instrumentHandle,
                                ":SENSe:SIGNalling:MINFo:CSIR?",
                                signal,
                                RSCMU_ERROR_PARAMETER2));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Frequency Shift                                                 */
/* Purpose:  This function returns the frequency shift measured while the    */
/*           R&S CMU synchronizes to the Node B signal.                      */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_SensSignMinfFsh (ViSession instrumentHandle,
                                      ViReal64 *frequencyShift,
                                      ViChar errorMessage[])
{
     ViStatus  RSCMU_Status = VI_SUCCESS;
    CHECKRETMSG(RSCMU_getViReal64(instrumentHandle,
                                ":SENSe:SIGNalling:MINFo:FSHift?",
                                frequencyShift,
                                RSCMU_ERROR_PARAMETER2));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Node B Power                                                    */
/* Purpose:  This function returns returns the average Node B power measured */
/*           while the R&S CMU synchronizes to the Node B signal.            */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_SensSignMinfNpow (ViSession instrumentHandle,
                                       ViReal64 *power,
                                       ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_getViReal64(instrumentHandle,
                                ":SENSe:SIGNalling:MINFo:NPOWer?",
                                power,
                                RSCMU_ERROR_PARAMETER2));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: System Frame Number                                             */
/* Purpose:  This function returns the SFN used/detected while the R&S CMU   */
/*           synchronizes to the Node B signal.                              */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_SensSignMinfSfn (ViSession instrumentHandle,
                                      ViReal64 *systemFrameNumber,
                                      ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    CHECKRETMSG(RSCMU_getViReal64(instrumentHandle,
                                ":SENSe:SIGNalling:MINFo:SFN?",
                                systemFrameNumber,
                                RSCMU_ERROR_PARAMETER2));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Operating Band                                                  */
/* Purpose:  This function sets the Operating Band of the UE.                */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ConfSignSelBand (ViSession instrumentHandle,
                                      ViInt32 band,
                                      ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    ViChar    wrtBuf[WR_BUFFER_SIZE];

    CHECK_VI_INT32_RANGE(instrumentHandle, band, 1, 6, RSCMU_ERROR_PARAMETER2, errorMessage);

    sprintf(wrtBuf, ":CONFigure:SIGNalling:SELect:BAND OB%ld", band);
    CHECKRETMSG(RSCMU_sendCmd(instrumentHandle, wrtBuf));

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/*===========================================================================*/
/* Node-B Setting                                                            */
/*===========================================================================*/
/*===========================================================================*/

/*===========================================================================*/
/* Function: Node B Signal Default                                           */
/* Purpose:  This function sets all parameters of the subsystem to default   */
/*           values (the setting OFF results in an error message).           */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_DefNbs (ViSession instrumentHandle, ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_sendCmd(instrumentHandle, "DEFault:NBSignal ON"));

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: RF Channel                                                      */
/* Purpose:  This function defines the frequency of the Node B signal.       */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ConfNbsChan (ViSession instrumentHandle, ViInt32 unit,
                                  ViReal64 RFChannel, ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    ViReal64  min, max;

    max = 2112.4E+0;
    min = 871.4E+0;

    switch (unit){
        case RSCMU_HZ:
            max = 2167.6E+6;
            min = 871.4E+6;
        break;
        case RSCMU_KHZ:
            max = 2167.6E+3;
            min = 871.4E+3;
        break;
        case RSCMU_MHZ:
            max = 2167.6E+0;
            min = 871.4E+0;
        break;
        case RSCMU_GHZ:
            max = 2167.6E-3;
            min = 871.4E-3;
        break;
        case RSCMU_CHANNEL:
            max = 0;
            min = 0;
        break;
   }

    CHECKRETMSG(RSCMU_setEnum(instrumentHandle,
                                "UNIT:NBSignal:CHANnel",
                                unit,
                                RSCMUNB_Unit_NBS_Freq,
                                RSCMU_ERROR_PARAMETER2));

    CHECKRETMSG(RSCMU_setViReal64(instrumentHandle,
                                "CONFigure:NBSignal:CHANnel",
                                RFChannel,
                                min,
                                max,
                                RSCMU_ERROR_PARAMETER2));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Frequency Unit                                                  */
/* Purpose:  This function defines whether the frequency of the RF signal    */
/*           analyzed is specified in frequency units or as an UARFCN.       */
/*           Frequency units must be used to select input signals that are   */
/*           outside the designated channel range.                           */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_UnitNbsChan (ViSession instrumentHandle,
                                  ViInt32 unit,
                                  ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setEnum(instrumentHandle,
                                "UNIT:NBSignal:CHANnel",
                                unit,
                                RSCMUNB_Unit_NBS_Freq,
                                RSCMU_ERROR_PARAMETER2));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Frequency Offset                                                */
/* Purpose:  This function defines an offset for the RF analyzer frequency.  */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ConfNbsFoff (ViSession instrumentHandle,
                                  ViReal64 frequencyOffset,
                                  ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setViReal64(instrumentHandle,
                                ":CONFigure:NBSignal:FOFFset",
                                frequencyOffset,
                                -100.0E+3,
                                100.0E+3,
                                RSCMU_ERROR_PARAMETER2));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: RX-TX Frequency Separation                                      */
/* Purpose:  This function displays the frequency gap between the downlink   */
/*           and uplink bands.                                               */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ConfNbsRtfs (ViSession instrumentHandle,
                                  ViReal64 *RXTXFrequencySeparation,
                                  ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_getViReal64(instrumentHandle,
                                ":CONFigure:NBSignal:RTFSeparate?",
                                RXTXFrequencySeparation,
                                RSCMU_ERROR_PARAMETER2));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: DL Operation Mode                                               */
/* Purpose:  This function defines whether dedicated channel of the Node B   */
/*           signal is configured as a downlink 3GPP reference channel       */
/*           according to 3GPP TS 25.101 or as a downlink dedicated physical */
/*           channel.                                                        */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ConfNbsDlopMode (ViSession instrumentHandle,
                                      ViInt32 DLOperationMode,
                                      ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setEnum(instrumentHandle,
                                ":CONFigure:NBSignal:DLOPeration:MODE",
                                DLOperationMode,
                                RSCMUNB_DedicatedChannel_Mode,
                                RSCMU_ERROR_PARAMETER2));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: DL Scrambling Code (DPCH)                                       */
/* Purpose:  This function specifies the scrambling code for the DPCH in the */
/*           Node B signal.                                                  */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ConfNbsDlscDpch (ViSession instrumentHandle,
                                      ViString DLScramblingCode,
                                      ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    ViChar    buffer[50];
    
    sprintf (buffer, "'%s'", DLScramblingCode);

    CHECKRETMSG(RSCMU_setViString(instrumentHandle,
                                ":CONFigure:NBSignal:DLSCode:DPCH",
                                buffer,
                                RSCMU_ERROR_PARAMETER2));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: DL Scrambling Code (CPICH)                                      */
/* Purpose:  This function specifies the scrambling code for the CPICH in    */
/*           the Node B signal.                                              */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ConfNbsDlscCpic (ViSession instrumentHandle,
                                      ViString DLScramblingCode,
                                      ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    ViChar    buffer[50];
    
    sprintf (buffer, "'%s'", DLScramblingCode);

    CHECKRETMSG(RSCMU_setViString(instrumentHandle,
                                ":CONFigure:NBSignal:DLSCode:CPICh",
                                buffer,
                                RSCMU_ERROR_PARAMETER2));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/*===========================================================================*/
/* Dedicated Physical Channel                                                           */
/*===========================================================================*/
/*===========================================================================*/

/*===========================================================================*/
/* Function: CPICH-DL DPCH Offset                                            */
/* Purpose:  This function specifies the timing delay of the start of the DL */
/*           DPCH relative to the P-CPICH timing.                            */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ConfNbsDdpcDldp (ViSession instrumentHandle,
                                      ViInt32 CPICHDLDPCHOffset,
                                      ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setViInt32(instrumentHandle,
                                ":CONFigure:NBSignal:DDPChannel:DLDPoffset",
                                CPICHDLDPCHOffset,
                                0,
                                38400,
                                RSCMU_ERROR_PARAMETER2));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Channelization Code Number                                      */
/* Purpose:  This function specifies the channelization code number of the   */
/*           downlink dedicated physical channel.                            */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ConfNbsDdpcChcn (ViSession instrumentHandle,
                                      ViInt32 channelizationCodeNumber,
                                      ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setViInt32(instrumentHandle,
                                ":CONFigure:NBSignal:DDPChannel:CHCNumber",
                                channelizationCodeNumber,
                                0,
                                511,
                                RSCMU_ERROR_PARAMETER2));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Symbol Rate                                                     */
/* Purpose:  This function returns the symbol rate corresponding to the DL   */
/*           slot format.                                                    */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ConfNbsDdpcSrat (ViSession instrumentHandle,
                                      ViReal64 *symbolRate,
                                      ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_getViReal64(instrumentHandle,
                                ":CONFigure:NBSignal:DDPChannel:SRATe?",
                                symbolRate,
                                RSCMU_ERROR_PARAMETER2));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Slot Format                                                     */
/* Purpose:  This function specifies the slot format of the downlink         */
/*           dedicated physical channel.                                     */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ConfNbsDdpcSfor (ViSession instrumentHandle,
                                      ViInt32 slotFormat,
                                      ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setViInt32(instrumentHandle,
                                ":CONFigure:NBSignal:DDPChannel:SFORmat",
                                slotFormat,
                                0,
                                16,
                                RSCMU_ERROR_PARAMETER2));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/*===========================================================================*/
/* 3GPP DL Reference Channel                                                 */
/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
/* Function: Reference Channel Type                                          */
/* Purpose:  This function sets the channel type (information bit rate) of   */
/*           the 3GPP DL reference channel.                                  */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ConfNbsGpprCtyp (ViSession instrumentHandle,
                                      ViInt32 referenceChannelType,
                                      ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setEnum(instrumentHandle,
                                ":CONFigure:NBSignal:GPPReference:CTYPe",
                                referenceChannelType,
                                RSCMUNB_Ref_Chan_Type,
                                RSCMU_ERROR_PARAMETER2));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: CPICH-DL DPCH Offset                                            */
/* Purpose:  This function specifies the timing delay of the start of the DL */
/*           DPCH relative to the P-CPICH timing.                            */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ConfNbsGpprDldp (ViSession instrumentHandle,
                                      ViInt32 CPICHDLDPCHOffset,
                                      ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setViInt32(instrumentHandle,
                                ":CONFigure:NBSignal:GPPReference:DLDPoffset",
                                CPICHDLDPCHOffset,
                                0,
                                38400,
                                RSCMU_ERROR_PARAMETER2));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Channelization Code Number                                      */
/* Purpose:  This function specifies specifies the channelization code number*/
/*           of the 3GPP DL reference channel.                               */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ConfNbsGpprChcn (ViSession instrumentHandle,
                                      ViInt32 channelizationCodeNumber,
                                      ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setViInt32(instrumentHandle,
                                ":CONFigure:NBSignal:GPPReference:CHCNumber",
                                channelizationCodeNumber,
                                0,
                                511,
                                RSCMU_ERROR_PARAMETER2));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/*===========================================================================*/
/* UE Signal                                                                 */
/*===========================================================================*/
/*===========================================================================*/

/*===========================================================================*/
/*===========================================================================*/
/* Control                                                                   */
/*===========================================================================*/
/*===========================================================================*/

/*===========================================================================*/
/* Function: Initiate                                                        */
/* Purpose:  This function starts the RF generator measurement.              */
/*           values (the setting OFF results in an error message).           */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_InitUes (ViSession instrumentHandle, ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_sendCmd(instrumentHandle, "INITiate:UESignal"));

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Abort                                                           */
/* Purpose:  This function aborts running RF generator measurement and       */
/*           switches off.                                                   */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_AborUes (ViSession instrumentHandle, ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_sendCmd(instrumentHandle, "ABORt:UESignal"));

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Measurement Status                                              */
/* Purpose:  This function starts the RF generator measurement.              */
/*           values (the setting OFF results in an error message).           */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_FetcUesStat (ViSession instrumentHandle,
                                  ViInt32 *measurementStatus,
                                  ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_getEnum(instrumentHandle,
                              "FETCh:UESignal:STATus?",
                              measurementStatus,
                              RSCMUNB_MeasStatus,
                              RSCMU_ERROR_INSTRUMENT_ERROR
                             ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: UE Signal Default                                               */
/* Purpose:  This function sets all parameters of the subsystem to default   */
/*           values (the setting OFF results in an error message).           */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_DefUes (ViSession instrumentHandle, ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_sendCmd(instrumentHandle, "DEFault:UESignal ON"));

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/*===========================================================================*/
/* Channel Independent Settings                                              */
/*===========================================================================*/
/*===========================================================================*/

/*===========================================================================*/
/* Function: Dedicated Channel Mode                                          */
/* Purpose:  This function sets all parameters of the subsystem to default   */
/*           values (the setting OFF results in an error message).           */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ConfUesUlopMode (ViSession instrumentHandle,
                                      ViInt32 dedicatedChannelMode,
                                      ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setEnum(instrumentHandle,
                                ":CONFigure:UESignal:ULOPeration:MODE",
                                dedicatedChannelMode,
                                RSCMUNB_DedicatedChannel_Mode,
                                RSCMU_ERROR_PARAMETER2));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Channel Number                                                  */
/* Purpose:  This function sets the frequency of the generated UE signal.    */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ConfUESChan (ViSession instrumentHandle,
                                  ViInt32 unit,
                                  ViReal64 frequencyChannelNumber,
                                  ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    ViReal64  min, max;

    max = 2112.4E+0;
    min = 871.4E+0;

    switch (unit){
        case RSCMU_HZ:
            max = 1977.6E+6;
            min = 826.4E+6;
        break;
        case RSCMU_KHZ:
            max = 1977.6E+3;
            min = 826.4E+3;
        break;
        case RSCMU_MHZ:
            max = 1977.6E+0;
            min = 826.4E+0;
        break;
        case RSCMU_GHZ:
            max = 1977.6E-3;
            min = 826.4E-3;
        break;
        case RSCMU_CHANNEL:
            max = 9888;
            min = 4132;
        break;
    }

    CHECKRETMSG(RSCMU_setEnum(instrumentHandle,
                          "UNIT:UESignal:CHANnel",
                          unit,
                          RSCMUNB_Unit_NBS_Freq,
                          RSCMU_ERROR_PARAMETER2
                         ));

    CHECKRETMSG(RSCMU_setViReal64(instrumentHandle,
                                ":CONFigure:UESignal:CHANnel",
                                frequencyChannelNumber,
                                min,
                                max,
                                RSCMU_ERROR_PARAMETER2));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: UL Scrambling Code                                              */
/* Purpose:  This function selects the long scrambling code for the uplink   */
/*           RF generator signal.                                            */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ConfUesUlsc (ViSession instrumentHandle,
                                  ViString scramblingCode,
                                  ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    ViChar    buffer[50];
    
    sprintf (buffer, "'%s'", scramblingCode);

    CHECKRETMSG(RSCMU_setViString(instrumentHandle,
                                "CONFigure:UESignal:ULSCode",
                                buffer,
                                RSCMU_ERROR_PARAMETER2));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: CPICH-DL DPCH Offset                                            */
/* Purpose:  This function defines the timing delay of the start of the      */
/*           UL DPCH relative to the CPICH.                                  */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ConfUesUldp (ViSession instrumentHandle,
                                     ViInt32 CPICHULDPCHOffset,
                                     ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setViInt32(instrumentHandle,
                                "CONFigure:UESignal:ULDPoffset",
                                CPICHULDPCHOffset,
                                0,
                                38400,
                                RSCMU_ERROR_PARAMETER2));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: TPC Step Size                                                   */
/* Purpose:  This function defines the power difference between two          */
/*           consecutive slots where the R&S CMU is commanded to change its  */
/*           output power.                                                   */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ConfUesUlpcTpcs (ViSession instrumentHandle,
                                      ViInt32 TPCStepSize,
                                      ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setViInt32(instrumentHandle,
                                "CONFigure:UESignal:ULPControl:TPCSsize",
                                TPCStepSize,
                                1,
                                2,
                                RSCMU_ERROR_PARAMETER2));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: CPICH-DL DPCH Offset                                            */
/* Purpose:  This function command defines the algorithm for uplink power    */
/*           control.                                                        */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ConfUesUlpcMode (ViSession instrumentHandle,
                                      ViInt32 mode,
                                      ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setEnum(instrumentHandle,
                              ":CONFigure:UESignal:ULPControl:MODE",
                              mode,
                              RSCMUNB_PowerControl_Mode,
                              RSCMU_ERROR_PARAMETER2
                             ));

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Frequency Offset                                                */
/* Purpose:  This function defines an offset for the RF analyzer frequency   */
/*           set with the function RSCMUNB_Sig_ConfUesChan.                  */
/*           The offset frequency must be in multiples of 1 Hz.              */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ConfUesFoffs (ViSession instrumentHandle,
                                   ViReal64 frequencyOffset,
                                   ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setViReal64(instrumentHandle,
                              ":CONFigure:UESignal:FOFFset",
                              frequencyOffset,
                              -100.000E+3,
                              100.000E+3,
                              RSCMU_ERROR_PARAMETER2
                             ));

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/*===========================================================================*/
/* 3GPP Reference Channel                                                    */
/*===========================================================================*/
/*===========================================================================*/

/*===========================================================================*/
/* Function: Reference Channel Type                                          */
/* Purpose:  This function sets the information bit rate of the 3GPP UL      */
/*           reference channel.                                              */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ConfUesGpprCtyp (ViSession instrumentHandle,
                                      ViInt32 referenceChannelType,
                                      ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setEnum(instrumentHandle,
                                ":CONFigure:UESignal:GPPReference:CTYPe",
                                referenceChannelType,
                                RSCMUNB_Ref_Chan_Type,
                                RSCMU_ERROR_PARAMETER2));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Channel Data Source DTCH                                        */
/* Purpose:  This function specifies the bit pattern transmitted on the      */
/*           Dedicated Traffic Channel part of the 3GPP UL reference channel.*/
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ConfUesGpprCdsDtch (ViSession instrumentHandle,
                                         ViInt32 sequence,
                                         ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    CHECKRETMSG(RSCMU_setEnum(instrumentHandle,
                              ":CONFigure:UESignal:GPPReference:CDSource:DTCH",
                              sequence,
                              RSSMUNB_SEQ_PR,
                              RSCMU_ERROR_PARAMETER2
                             ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: TFCI Bits                                                       */
/* Purpose:  This function defines the Transport Format Combination Indicator*/
/*           (TFCI) bits to be transmitted on the 3GPP UL reference channel. */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ConfUesGpprTfc (ViSession instrumentHandle,
                                     ViUInt32 TFCIBits,
                                     ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    ViChar    TFCIBitsBuffer[35] = "";

    RSCMU_Status = RSCMU_Dec2Bin(TFCIBits, TFCIBitsBuffer, "\"");
    RETMSG(RSCMU_setViString(instrumentHandle,
                             ":CONFigure:UESignal:GPPReference:TFCibits",
                             TFCIBitsBuffer,
                             RSCMU_ERROR_PARAMETER2
                            ));
}

/*===========================================================================*/
/* Function: BER BLER Verification Mode                                      */
/* Purpose:  This function enables or disables the BER or BLER verification  */
/*           mode. The BER or BLER of the signal is defined separately.      */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_SourRfgGpprBver (ViSession instrumentHandle,
                                      ViInt32 BERBLERVerificationMode,
                                      ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setEnum(instrumentHandle,
                              ":CONFigure:UESignal:GPPReference:BVER:MODE",
                              BERBLERVerificationMode,
                              RSCMUNB_BerBlerVerifMode,
                              RSCMU_ERROR_PARAMETER2
                             ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: BER BLER Value                                                  */
/* Purpose:  This function defines the Bit Error Rate (BER) or Block Error   */
/*           Rate (BLER) of the uplink signal that the CMU generates if the  */
/*           BER or BLER verification mode is enabled                        */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_SourRfgGpprBverBler (ViSession instrumentHandle,
                                          ViInt32 verificationMode,
                                          ViReal64 value,
                                          ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECK_VI_INT32_RANGE(instrumentHandle, verificationMode, 1, 2, RSCMU_ERROR_PARAMETER2, errorMessage);

    switch (verificationMode){
        case RSCMUNB_3GPP_VERIFICATION_BER:
            CHECKRETMSG(RSCMU_setViReal64(instrumentHandle,
                            ":CONFigure:UESignal:GPPReference:BVER:BER",
                            value,
                            0.0, 100.0,
                            RSCMU_ERROR_PARAMETER3));
        break;
        case RSCMUNB_3GPP_VERIFICATION_BLER:
            CHECKRETMSG(RSCMU_setViReal64(instrumentHandle,
                            ":CONFigure:UESignal:GPPReference:BVER:BLER",
                            value,
                            0.0, 100.0,
                            RSCMU_ERROR_PARAMETER3));
    }

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/*===========================================================================*/
/* Dedicated Physical Channel                                                */
/*===========================================================================*/
/*===========================================================================*/

/*===========================================================================*/
/*===========================================================================*/
/* Physical Channel DPCCH                                                    */
/*===========================================================================*/
/*===========================================================================*/

/*===========================================================================*/
/* Function: Slot Format                                                     */
/* Purpose:  This function specifies the slot format of the UL DPDCH.        */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ConfUesUdpcDpccSfor (ViSession instrumentHandle,
                                          ViInt32 *slotFormat,
                                          ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_getViInt32(instrumentHandle,
                              ":CONFigure:UESignal:UDPChannel:DPCCh:SFORmat?",
                              slotFormat,
                              RSCMU_ERROR_PARAMETER2));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Symbol Rate                                                     */
/* Purpose:  This function specifies the slot format of the UL DPDCH.        */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ConfUesUdpcDpccSrat (ViSession instrumentHandle,
                                          ViInt32 *symbolRate,
                                          ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    
    GET_RD_BUF (instrumentHandle);

    CHECKRETMSG(RSCMU_getViString(instrumentHandle,
                                  "CONFigure:UESignal:UDPChannel:DPCCh:SRATe?",
                                  rdBuf,
                                  RD_BUFFER_SIZE,
                                  RSCMU_ERROR_PARAMETER1));

    sscanf (rdBuf, "S%ld", symbolRate);
    
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/*===========================================================================*/
/* TFCI Field Setting                                                        */
/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
/* Function: TFCI Code Word                                                  */
/* Purpose:  This function defines the Transport Format Combination Indicator*/
/*           (TFCI) code word to be transmitted on the uplink physical       */
/*           channel.                                                        */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ConfUesUdpcDpccTfciCWor (ViSession instrumentHandle,
                                              ViUInt32 TFCICodeWord,
                                              ViChar errorMessage[])
{
    ViChar    TFCICodeWordBuffer[11] = "";

    sprintf(TFCICodeWordBuffer, "\"%lX\"", TFCICodeWord);
    RETMSG(RSCMU_setViString(instrumentHandle,
                             ":CONFigure:UESignal:UDPChannel:DPCCh:TFCI:CWORd",
                             TFCICodeWordBuffer,
                             RSCMU_ERROR_PARAMETER2
                            )
                );
}

/*===========================================================================*/
/*===========================================================================*/
/* TPC Field Setting                                                         */
/*===========================================================================*/
/*===========================================================================*/

/*===========================================================================*/
/* Function: TPC Pattern                                                     */
/* Purpose:  This function defines the single TPC bit pattern transmitted on */
/*           the uplink physical channel.                                    */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ConfUesUdpcDpccTpcPatt (ViSession instrumentHandle,
                                             ViInt32 TPCPatternLength,
                                             ViUInt32 TPCPattern,
                                             ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    ViChar    patternBuffer[35] = "";

    CHECKRETMSG(RSCMU_setViInt32(instrumentHandle,
                                ":CONFigure:UESignal:UDPChannel:DPCCh:TPC:PLENgth",
                                TPCPatternLength,
                                0, 32,
                                RSCMU_ERROR_PARAMETER2));

    RSCMU_Status = RSCMU_Dec2Bin(TPCPattern, patternBuffer, "\"");
    RETMSG(RSCMU_setViString(instrumentHandle,
                             ":CONFigure:UESignal:UDPChannel:DPCCh:TPC:PATTern",
                             patternBuffer,
                             RSCMU_ERROR_PARAMETER3
                            ));
}

/*===========================================================================*/
/* Function: Read Out Mode                                                   */
/* Purpose:  This function defines the rule for constructing a TPC pattern   */
/*           of infinite length to be transferred to Node B. The actual      */
/*           pattern transferred depends on the finite single pattern        */
/*           defined via CONFigure:UESignal:UDPChannel:DPCCh:TPC:PATTern     */
/*           and the read out mode.                                          */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ConfUesUdpsDpccTpcRom (ViSession instrumentHandle,
                                            ViInt32 readOutMode,
                                            ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setEnum(instrumentHandle,
                              ":CONFigure:UESignal:UDPChannel:DPCCh:TPC:ROMode",
                              readOutMode,
                              RSCMUNB_PCFieldSettings_ReadOutMode,
                              RSCMU_ERROR_PARAMETER2
                             ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/*===========================================================================*/
/* Physical Channel DPDCH                                                    */
/*===========================================================================*/
/*===========================================================================*/

/*===========================================================================*/
/* Function: Channel Number                                                  */
/* Purpose:  This function specifies the number of code channels for slot    */
/*           format 6. Slot format 6 must be selected with the command       */
/*           (CONFigure:UESignal:UDPChannel:DPDCh:TOTal:SFORmat 6) before    */
/*           this command can be used.                                       */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ConfUesUdpcDpdcCchn (ViSession instrumentHandle,
                                          ViInt32 channel_Number,
                                          ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setViInt32(instrumentHandle,
                              ":CONFigure:UESignal:UDPChannel:DPDCh:CCHNumber",
                              channel_Number,
                              1,
                              6,
                              RSCMU_ERROR_PARAMETER2));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Channel Data Source                                             */
/* Purpose:  This function specifies the bit pattern transmitted on the      */
/*           UL DPDCH.                                                       */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ConfUesUdpcDpdcCds (ViSession instrumentHandle,
                                         ViInt32 channelDataSource,
                                         ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    CHECKRETMSG(RSCMU_setEnum(instrumentHandle,
                              ":CONFigure:UESignal:UDPChannel:DPDCh:CDSource",
                              channelDataSource,
                              RSCMUNB_ChannelDataSource,
                              RSCMU_ERROR_PARAMETER2
                             ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Slot Format                                                     */
/* Purpose:  This function specifies the slot format of the UL DPDCH.        */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ConfUesUdpcDpdcSfor (ViSession instrumentHandle,
                                          ViInt32 slotFormat,
                                          ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setViInt32(instrumentHandle,
                              ":CONFigure:UESignal:UDPChannel:DPDCh:SFORmat",
                              slotFormat,
                              0,
                              6,
                              RSCMU_ERROR_PARAMETER2));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Symbol Rate                                                     */
/* Purpose:  This function returns the symbol rate of the UL DPDCH.          */
/*           For slot format 6 (symbol rate 960 kbps) a higher effective     */
/*           symbol rate can be achieved by using several code channels      */
/*           (CONFigure:UESignal:UDPChannel:DPDCh:CCHNumber).                */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ConfUesUdpcDpdcSrat (ViSession instrumentHandle,
                                             ViInt32 *symbolRate,
                                             ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    GET_RD_BUF (instrumentHandle);

    CHECKRETMSG(RSCMU_getViString(instrumentHandle,
                                  "CONFigure:UESignal:UDPChannel:DPDCh:SRATe?",
                                  rdBuf,
                                  RD_BUFFER_SIZE,
                                  RSCMU_ERROR_PARAMETER1));

    sscanf (rdBuf, "S%ld", symbolRate);
    
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/*===========================================================================*/
/* Power                                                                     */
/*===========================================================================*/
/*===========================================================================*/

/*===========================================================================*/
/* Function: AWGN Contribution                                               */
/* Purpose:  This function switches the Additional White Gaussian Noise      */
/*           contribution to the RF generator signal on or off. The AWGN     */
/*           level is defined separately (CONFigure:UESignal:UPOWer:NLEVel). */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ConfUesUpowAwgn (ViSession instrumentHandle,
                                      ViInt32 awgn,
                                      ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setEnum(instrumentHandle,
                              "CONFigure:UESignal:UPOWer:AWGN",
                              awgn,
                              RSCMUNB_mode_On_Off,
                              RSCMU_ERROR_PARAMETER2));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: AWGN Noise Level                                                */
/* Purpose:  This function sets the level of the Additional White Gaussian   */
/*           Noise contribution to the RF generator signal. The setting takes*/
/*           effect if the AWGN is enabled only                              */
/*           (CONFigure:UESignal:UPOWer:AWGN ON).                            */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ConfUesUpowNlev (ViSession instrumentHandle,
                                      ViReal64 noiseLevel,
                                      ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setViReal64(instrumentHandle,
                              "CONFigure:UESignal:UPOWer:NLEVel",
                              noiseLevel,
                              -137.0,
                              -23.0,
                              RSCMU_ERROR_PARAMETER2));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: DPCCH Level                                                     */
/* Purpose:  This function sets the level of the Dedicated Physical Control  */
/*           Channel (DPCCH). The DPCCH/DPDCH power ratio and the number of  */
/*           DPDCHs is considered to be fixed for all channel configurations,*/
/*           so the command overwrites the value set with command            */
/*           CONFigure:UESignal:UPOWer:TTPower and vice versa.               */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ConfUesUpowDpcc (ViSession instrumentHandle,
                                      ViReal64 level,
                                      ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setViReal64(instrumentHandle,
                              "CONFigure:UESignal:UPOWer:DPCCh",
                              level,
                              -137.0,
                              -10.0,
                              RSCMU_ERROR_PARAMETER2));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: DPDCH Level                                                     */
/* Purpose:  This function returns the level of the Dedicated Physical Data  */
/*           Channel (DPDCH). The DPDCH power depends on the channel         */
/*           configuration and the Total TX Power                            */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ConfUesUpowDpdc (ViSession instrumentHandle,
                                      ViReal64 *level,
                                      ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_getViReal64(instrumentHandle,
                              "CONFigure:UESignal:UPOWer:DPDCh?",
                              level,
                              RSCMU_ERROR_PARAMETER2));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Power Ratio                                                     */
/* Purpose:  This function sets the DPCCH/DPDCH power ratio for physical     */
/*           channels in the uplink WCDMA signal. The value can be set in all*/
/*           instrument states; it takes effect after a physical uplink      */
/*           channel is selected (CONFigure:UESignal:ULOPeration:MODE).      */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ConfUesUpowRcdPhys (ViSession instrumentHandle,
                                         ViInt32 DPDCHState,
                                         ViReal64 powerRatio,
                                         ViChar errorMessage[])
{

    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECK_VI_INT32_RANGE(instrumentHandle, DPDCHState, 0, 1, RSCMU_ERROR_PARAMETER2, errorMessage);

    if (DPDCHState)
        CHECKRETMSG(RSCMU_setViReal64(instrumentHandle,
                              "CONFigure:UESignal:UPOWer:RCD:PHYSical",
                              powerRatio,
                              -23.52,
                              23.52,
                              RSCMU_ERROR_PARAMETER2));
    else
        // DPDCH switched off
        CHECKRETMSG(RSCMU_sendCmd(instrumentHandle,
                              "CONFigure:UESignal:UPOWer:RCD:PHYSical INF"));

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Total Power                                                     */
/* Purpose:  This function sets the Total TX Power in absolute units.        */
/*           The DPCCH/DPDCH power ratio and the number of DPDCHs is         */
/*           considered to be fixed for all channel configurations, so the   */
/*           command overwrites the value of CONFigure:UESignal:UPOWer:DPCCh */
/*           and vice versa.                                                 */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ConfUesUpowTtp (ViSession instrumentHandle,
                                     ViReal64 totalPower,
                                     ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setViReal64(instrumentHandle,
                              "CONFigure:UESignal:UPOWer:TTPower",
                              totalPower,
                              -137.0,
                              -0.0,
                              RSCMU_ERROR_PARAMETER2));
    RETMSG(RSCMU_Status);
}



/*===========================================================================*/
/*===========================================================================*/
/*  HSDPA Channel
/*===========================================================================*/
/*===========================================================================*/

/*===========================================================================*/
/* Function: HSDPA Signal                                                    */
/* Purpose:  This function switches the HS-DPCCH channel in the UL WCDMA     */
/*           signal on or off. Enabling the HSDPA signal requires BCH/SFN    */
/*           synchronization.                                                */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ConfHsdpaSignal (ViSession instrumentHandle,
                                               ViInt32 signalState,
                                               ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setEnum(instrumentHandle,
                              "CONFigure:UESignal:HSDPa:SIGNal:ENABle",
                              signalState,
                              RSCMUNB_mode_On_Off,
                              RSCMU_ERROR_PARAMETER2));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: HSDPA Configuration Type                                        */
/* Purpose:  This function activates both the ACK/NACK & CQI slots or one    */
/*           slot type only (ACK/NACK with DTX in the CQI slots or CQI with  */
/*           DTX in the ACK/NACK slots).                                     */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ConfHsdpaType (ViSession instrumentHandle,
                                             ViInt32 HSDPCCHConfType,
                                             ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setEnum(instrumentHandle,
                              "CONFigure:UESignal:HSDPa:CTYPe",
                              HSDPCCHConfType,
                              RSCMUNB_HsdpaType,
                              RSCMU_ERROR_PARAMETER2));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: HSDPA ACK Transmit Mode                                         */
/* Purpose:  This function enables ACK/NACK transmission every n-th HSDPA    */
/*           subframe where n is the ACK Transmit Distance                   */
/*           (CONFigure:UESignal:HSDPa:ACK:TDIStance) or single ACK/NACK     */
/*           transmission each time the main UE-ID                           */
/*           (CONFigure:NBSignal:HSDPa:MUID) is received on one              */
/*           of the DL HS-SCCHs.                                             */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ConfHsdpaAckTransmitMode (ViSession instrumentHandle,
                                                   ViInt32 ACKTransmitMode,
                                                   ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setEnum(instrumentHandle,
                              "CONFigure:UESignal:HSDPa:ACK:TMODe",
                              ACKTransmitMode,
                              RSCMUNB_ACKTransmitMode,
                              RSCMU_ERROR_PARAMETER2));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: HSDPA Transmit Distance                                         */
/* Purpose:  This function defines the distance between two active ACK/NACK  */
/*           or two active pairs of CQI slots.                               */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ConfHsdpaTransmitDistance (ViSession instrumentHandle,
                                                       ViInt32 slot,
                                                       ViInt32 transmitDistance,
                                                       ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    ViChar    cmd[WR_BUFFER_SIZE]     = "";

    if (RSCMU_invalidViInt32Range (slot, 1, 3))
       RETMSG(RSCMU_ERROR_PARAMETER2);
    if (RSCMU_invalidViInt32Range (transmitDistance, 0, 4095))
       RETMSG(RSCMU_ERROR_PARAMETER3);

    sprintf(cmd, "CONFigure:UESignal:HSDPa:%s:TDIStance %ld",
    RSCMUNB_HsdpaTransmitMode[slot], transmitDistance);

    CHECKRETMSG(RSCMU_sendCmd(instrumentHandle, cmd));

    RETMSG(RSCMU_Status);
}


/*===========================================================================*/
/* Function: HSDPA Power Delta                                               */
/* Purpose:  This function defines the ratio between the HS-DPCCH gain       */
/*           factors for ACK, NACK or CQI slots and the gain factor for the  */
/*           DPCCHs. According to standard TS 25.214, the ratio is equal to  */
/*           20*log(delta ACK), 20*log(delta NACK) or 20*log(delta CQI).     */                          
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ConfHsdpaPowerDelta (ViSession instrumentHandle,
                                                   ViInt32 slot,
                                                   ViReal64 powerDelta,
                                                   ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    ViChar    cmd[WR_BUFFER_SIZE]     = "";

    if (RSCMU_invalidViInt32Range (slot, 1, 3))
       RETMSG(RSCMU_ERROR_PARAMETER2);
    if (RSCMU_invalidViReal64Range (powerDelta, -23.52, 6.02))
       RETMSG(RSCMU_ERROR_PARAMETER3);

    sprintf(cmd, "CONFigure:UESignal:HSDPa:%s:PDELta %f",
    RSCMUNB_HsdpaTransmitMode[slot], powerDelta);

    CHECKRETMSG(RSCMU_sendCmd(instrumentHandle, cmd));

    RETMSG(RSCMU_Status);
}


/*===========================================================================*/
/* Function: HSDPA Power Delta                                               */
/* Purpose:  This function defines the HSDPA System Frame Number and defines */
/*           whether HS-DPCCH generation starts immediately after the HSDPA  */
/*           signal is switched on or after a definite HSFN Number.          */                          
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ConfHsdpaHsfnActivation (ViSession instrumentHandle,
                                                       ViInt32 slot,
                                                       ViInt32 activationHSFNMode,
                                                       ViInt32 activationHSFNNumber,
                                                       ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    ViChar    cmd[WR_BUFFER_SIZE]     = "";

    if (RSCMU_invalidViInt32Range (slot, 1, 3))
       RETMSG(RSCMU_ERROR_PARAMETER2);

    sprintf(cmd, "CONFigure:UESignal:HSDPa:%s:HSFN:MODE",RSCMUNB_HsdpaTransmitMode[slot]);
    CHECKRETMSG (RSCMU_setEnum (instrumentHandle, cmd, activationHSFNMode, RSCMUNB_activationHSFNMode, RSCMU_ERROR_PARAMETER3));

    if (activationHSFNMode == RSCMUNB_UES_HSDPA_ACK_HSFN_MODE_HNUM)
    {
        sprintf(cmd, "CONFigure:UESignal:HSDPa:%s:HSFN:NUMBer", RSCMUNB_HsdpaTransmitMode[slot]);
        CHECKRETMSG (RSCMU_setViInt32 (instrumentHandle, cmd, activationHSFNNumber, 0, 4095, RSCMU_ERROR_PARAMETER4));
    }

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: HSDPA Table Length                                              */
/* Purpose:  This function determines the length of the Table.               */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ConfHsdpaTabLength (ViSession instrumentHandle,
                                                  ViInt32 slot,
                                                  ViInt32 no_ofACKsCQIs_inTable,
                                                  ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    ViChar    cmd[WR_BUFFER_SIZE]     = "";

    if (RSCMU_invalidViInt32Range (slot, 1, 3))
       RETMSG(RSCMU_ERROR_PARAMETER2);
    if (RSCMU_invalidViInt32Range (no_ofACKsCQIs_inTable, 1, 64))
       RETMSG(RSCMU_ERROR_PARAMETER3);

    sprintf(cmd, "CONFigure:UESignal:HSDPa:%s:NUMBer %ld",
    RSCMUNB_HsdpaTransmitMode[slot], no_ofACKsCQIs_inTable);

    CHECKRETMSG(RSCMU_sendCmd(instrumentHandle, cmd));

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: HSDPA Single ACK Table Value                                    */
/* Purpose:  This function specifies a single entry for the ACK table.       */                                                  
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ConfHsdpaSingAckTabValue (ViSession instrumentHandle,
                                                        ViInt32 ACKTableIndex,
                                                        ViInt32 ACKTableValue,
                                                        ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViChar      cmd1[WR_BUFFER_SIZE]    = "";
    
    if (RSCMU_invalidViInt32Range (ACKTableValue, 0, 2))
        RETMSG(RSCMU_ERROR_PARAMETER3);

    sprintf(cmd1, "CONFigure:UESignal:HSDPa:ACK:TABLe%ld %s",
            ACKTableIndex, RSCMUNB_HsdpaTransmitMode[ACKTableValue]);
    
    CHECKRETMSG(RSCMU_sendCmd(instrumentHandle, cmd1));
    
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: HSDPA All ACK Table Values                                      */
/* Purpose:  This function specifies all entries for the ACK table.          */                                                  
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ConfHsdpaAllAckTabValues (ViSession instrumentHandle,
                                                        ViInt32 number_ofValues,
                                                        ViInt32 allACKTableValues[],
                                                        ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    ViInt32   i = 0;
    ViChar    *buffer, *p2buf;
    
    p2buf = buffer = (ViChar*) malloc (number_ofValues * 5 + 10);
    
    for (i = 0; i < number_ofValues; i++)
    {
        if (RSCMU_invalidViInt32Range (allACKTableValues[i], 0, 2))
        {
            free (buffer);
            RETMSG(RSCMU_ERROR_PARAMETER2);
        }
        p2buf += sprintf (p2buf, "%s,", RSCMUNB_HsdpaTransmitMode[allACKTableValues[i]]);
    }    
    
    *--p2buf = '\n';    
    
    RSCMU_Status = RSCMU_setViString(instrumentHandle,
                                  "CONFigure:UESignal:HSDPa:ACK:TABLe:ALL",
                                  buffer,
                                  RSCMU_ERROR_PARAMETER2
                                  );
    free (buffer);
    
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: HSDPA Single CQI Table Value                                    */
/* Purpose:  This function specifies a single entry for the CQI table.       */                                                  
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ConfHsdpaSingCqiTabValue (ViSession instrumentHandle,
                                                        ViInt32 ACKTableIndex,
                                                        ViInt32 CQITableValue,
                                                        ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViChar      cmd1[WR_BUFFER_SIZE]    = "";
    
    if (RSCMU_invalidViInt32Range (CQITableValue, 1, 30))
        RETMSG(RSCMU_ERROR_PARAMETER3);

    sprintf(cmd1, "CONFigure:UESignal:HSDPa:CQI:TABLe%ld %ld", ACKTableIndex, CQITableValue);
    
    CHECKRETMSG(RSCMU_sendCmd(instrumentHandle, cmd1));
    
    RETMSG(RSCMU_Status);
}


/*===========================================================================*/
/* Function: HSDPA All CQI Table Values                                      */
/* Purpose:  This function specifies all entries for the CQI table.          */                                                  
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ConfHsdpaAllCqiTabValues (ViSession instrumentHandle,
                                                        ViInt32 number_ofValues,
                                                        ViInt32 allCQITableValues[],
                                                        ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    ViInt32   i = 0;
    ViChar    *buffer, *p2buf;
    
    p2buf = buffer = (ViChar*) malloc (number_ofValues * 11);
    
    for (i = 0; i < number_ofValues; i++)
    {
        if (RSCMU_invalidViInt32Range (allCQITableValues[i], 1, 30))
        {
            free (buffer);
            RETMSG(RSCMU_ERROR_PARAMETER2);
        }
        
        p2buf += sprintf (p2buf, "%ld,", allCQITableValues [i]);
    }
    
    *--p2buf = '\n';    
    
    RSCMU_Status = RSCMU_setViString(instrumentHandle,
                                  "CONFigure:UESignal:HSDPa:CQI:TABLe:ALL",
                                  buffer,
                                  RSCMU_ERROR_PARAMETER2
                                  );
    free (buffer);
    
    RETMSG(RSCMU_Status);
}
              
/*===========================================================================*/
/*===========================================================================*/
/* Trigger                                                                   */
/*===========================================================================*/
/*===========================================================================*/

/*===========================================================================*/
/* Function: Default Trigger                                                 */
/* Purpose:  This function sets all parameters of the TRIGger subsystem to   */
/*           default values. The setting OFF results in an error message.    */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_DefTrig (ViSession instrumentHandle, ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_sendCmd(instrumentHandle, "DEFault:TRIGger ON"));

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Trigger Source                                                  */
/* Purpose:  This function selects the source of the trigger events.         */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_TrigSour (ViSession instrumentHandle,
                               ViInt32 triggerSource,
                               ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    CHECKRETMSG(RSCMU_setEnum(instrumentHandle,
                              "TRIGger:SOURce",
                              triggerSource,
                              RSCMUNB_Trigger_Source,
                              RSCMU_ERROR_PARAMETER2));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Trigger Slope                                                   */
/* Purpose:  This function qualifies whether the trigger event occurs on the */
/*           rising edge or on the falling edge of the external meas. trigger*/
/*           signal.                                                         */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_TrigSlop (ViSession instrumentHandle,
                               ViInt32 triggerSlope,
                               ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setEnum(instrumentHandle,
                              "TRIGger:SLOPe",
                              triggerSlope,
                              RSCMUNB_Trig_Slope,
                              RSCMU_ERROR_PARAMETER2));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Chip Delay                                                      */
/* Purpose:  This function defines an integer number of chip periods between */
/*           the external meas. trigger time and the start of the measurement*/
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_TrigEtc (ViSession instrumentHandle,
                              ViReal64 chipDelay,
                              ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setViReal64(instrumentHandle,
                              "TRIGger:ETCoffset",
                              chipDelay,
                              0.0,
                              38399.75,
                              RSCMU_ERROR_PARAMETER2));
    RETMSG(RSCMU_Status);
}



/*===========================================================================*/
/* Function: Start SFN                                                       */
/* Purpose:  This function defines an integer number as Start SFN for        */
/*           the Internal Superframe Trigger.                                */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_TrigIsts (ViSession instrumentHandle,
                                        ViInt32 startSFNNumber,
                                        ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setViInt32(instrumentHandle,
                              "TRIG:ISTS",
                              startSFNNumber,
                              0,
                              4095,
                              RSCMU_ERROR_PARAMETER2));
    RETMSG(RSCMU_Status);    
}

/*===========================================================================*/
/* Function: Trigger Period                                                  */
/* Purpose:  This function defines Trigger Periods for the Superframe        */
/*           Trigger.                                                        */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_TrigIstp (ViSession instrumentHandle,
                                        ViInt32 period, ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    
    GET_WRT_BUF (instrumentHandle);
    
    if (RSCMU_invalidViInt32Range (period, 2, 4096))
        RETMSG(RSCMU_ERROR_PARAMETER2);
       
    sprintf (wrtBuf, "TRIG:ISTP P%ld", period);
    CHECKRETMSG(RSCMU_sendCmd(instrumentHandle, wrtBuf));
    
    RETMSG(RSCMU_Status);     
}

 
/*===========================================================================*/
/* Function: Pre-Trigger Time                                                */
/* Purpose:  This function defines the pre-trigger time for the HS UE-ID     */
/*           trigger signals.                                                */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_PreTrigTime (ViSession instrumentHandle,
                                           ViInt32 preTriggerTime,
                                           ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    
    GET_WRT_BUF (instrumentHandle);
    
    if (RSCMU_invalidViInt32Range (preTriggerTime, 0, 256))
        RETMSG(RSCMU_ERROR_PARAMETER2);
       
    sprintf (wrtBuf, "TRIGger:SEQuence:HUTPre %ld", preTriggerTime);
    CHECKRETMSG(RSCMU_sendCmd(instrumentHandle, wrtBuf));
    
    RETMSG(RSCMU_Status);     
}


/*===========================================================================*/
/* Function: Output Trigger Signal                                           */
/* Purpose:  This function assigns the different trigger signals (or no      */
/*           signal) to pins 2 to 5 (<nr> = 2 to 5) of the AUX 3 connector.  */
/*           The settings are only valid for Signalling trigger source.      */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_TrigOutpPinSig (ViSession instrumentHandle,
                                              ViInt32 pin, 
                                              ViInt32 signal,
                                              ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    
    GET_WRT_BUF (instrumentHandle);
    
    if (RSCMU_invalidViInt32Range (pin, 2, 5))
        RETMSG(RSCMU_ERROR_PARAMETER2);
        
    sprintf (wrtBuf, "TRIG:OUTP:PIN%ld:SIGN", pin);
    CHECKRETMSG(RSCMU_setEnum(instrumentHandle,
                              wrtBuf,
                              signal,
                              RSCMUNB_trigOutpPinSig,
                              RSCMU_ERROR_PARAMETER3));
    RETMSG(RSCMU_Status);     
}

/*===========================================================================*/
/* Function: Output Trigger Offset                                           */
/* Purpose:  This function qualifies whether the frame, slot or superframe   */
/*           trigger signal at pins 2 to 5 (<nr> = 2 to 5) of the AUX 3      */
/*           connector are delayed by the specified offset.                  */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_TrigOutpOffs (ViSession instrumentHandle,
                                            ViInt32 pin, 
                                            ViInt32 state,
                                            ViInt32 offset,
                                            ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    ViChar  *p2buff;
    
    GET_WRT_BUF (instrumentHandle);
    
    if (RSCMU_invalidViInt32Range (pin, 2, 5))
        RETMSG(RSCMU_ERROR_PARAMETER2);
    if (RSCMU_invalidViInt32Range (state, 0, 1))
        RETMSG(RSCMU_ERROR_PARAMETER3);
    if (RSCMU_invalidViInt32Range (offset, 1, 38399))
        RETMSG(RSCMU_ERROR_PARAMETER4);
        
    p2buff = wrtBuf;
    p2buff += sprintf (p2buff, "TRIG:OUTP:PIN%ld:OFFS:ENAB %s", pin, RSCMUNB_mode_On_Off[state]);
    
    if (state)
        sprintf (p2buff, ";:TRIG:OUTP:OFFS %ld", offset);
    CHECKRETMSG(RSCMU_sendCmd(instrumentHandle, wrtBuf));
    
    RETMSG(RSCMU_Status);     
}

/*===========================================================================*/
/*===========================================================================*/
/* BCH Monitor                                                               */
/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
/* Control                                                                   */
/*===========================================================================*/
/*===========================================================================*/


/*===========================================================================*/
/* Function: Initiate BCH Measurement                                        */
/* Purpose:  This function starts the BCH Monitor measurement.               */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_InitBchm (ViSession instrumentHandle, ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_sendCmd(instrumentHandle, "INITiate:BCHMonitor"));

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Abort BCH Measurement                                           */
/* Purpose:  This function aborts the BCH Monitor measurement.               */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_AborBchm (ViSession instrumentHandle, ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_sendCmd(instrumentHandle, "ABORt:BCHMonitor"));

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/*===========================================================================*/
/* Result                                                                    */
/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
/* Function: Read SIB Contents                                               */
/* Purpose:  This function retrieves the contents of the last received SIB   */
/*           of the type selected via the 'Type' parameter.                  */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_SensBchmSibr (ViSession instrumentHandle, ViInt32 type,
                                   ViInt32 *number_ofReturnedValues,
                                   ViInt32 *completeStringSize,
                                   ViInt32 indentationDepth[],
                                   ViChar returnedStrings[],
                                   ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    ViChar    buffer[RD_BUFFER_SIZE_ARRAY];
    ViChar    cmd[WR_BUFFER_SIZE] = "";
    ViChar    *pbuffer, *p2ret_str;
    ViInt32   num,i;

    CHECK_VI_INT32_RANGE(instrumentHandle, type, RSCMUNB_SIB_TYPE_MIB, RSCMUNB_SIB_TYPE_SB2, RSCMU_ERROR_PARAMETER2, errorMessage);

    if (!number_ofReturnedValues) RETMSG(RSCMU_ERROR_PARAMETER3);
    if (!completeStringSize) RETMSG(RSCMU_ERROR_PARAMETER4);

    sprintf(cmd,"BCHM:SIB? %s",RSCMUNB_SIB_Type[type]);
    CHECKRETMSG(RSCMU_getViString(instrumentHandle, cmd, buffer, RD_BUFFER_SIZE_ARRAY, RSCMU_ERROR_INSTRUMENT_ERROR));

    pbuffer = strtok(buffer,",");
    num = atoi(pbuffer);
    *number_ofReturnedValues = num;
    *completeStringSize = 0;
    if (indentationDepth || returnedStrings)
    {
        p2ret_str=returnedStrings;
        for (i=0;i<num;i++)
        {
            pbuffer = strtok(NULL,",");
            indentationDepth[i]= atoi(pbuffer);
            pbuffer = strtok(NULL,",");
            p2ret_str += sprintf (p2ret_str,"%s,", pbuffer);
            *completeStringSize += strlen(pbuffer);
        }
        if (i)
        {
            p2ret_str--;
            p2ret_str='\0';
        }
    }
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Read SIB                                                        */
/* Purpose:  This function returns the last received SIB.                    */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_SensBchmSibrec (ViSession instrumentHandle,
                                              ViInt32 *type,
                                              ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_getEnum(instrumentHandle,
                                  "BCHM:SIBR?",
                                  type,
                                  RSCMUNB_SIB_Type,
                                  RSCMU_ERROR_INTERPRETING_RESPONSE));
    
    RETMSG (RSCMU_Status);
}

/*===========================================================================*/
/* Function: Change Destination                                              */
/* Purpose:  This function changes the default file name and path for        */
/*           logging files in the current storage                            */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_BchmCdes (ViSession instrumentHandle,
                                        ViChar fileName[], 
                                        ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    GET_WRT_BUF (instrumentHandle);

    if (!fileName)
        RETMSG(RSCMU_ERROR_PARAMETER2);
        
    sprintf (wrtBuf, "MMEM:BCHM:CDES \"%s\"", fileName);

    CHECKRETMSG(RSCMU_sendCmd(instrumentHandle, wrtBuf));
    
    RETMSG(RSCMU_Status);
    
}

/*===========================================================================*/
/* Function: Save to File                                                    */
/* Purpose:  This function saves the current ring buffer content to          */
/*           the default logging file or to the specified file and storage   */
/*           device.                                                         */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_BchmSave (ViSession instrumentHandle,
                                        ViChar fileName[], 
                                        ViInt32 storageDevice,
                                        ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    
    GET_WRT_BUF( instrumentHandle );

    if (RSCMU_invalidViInt32Range (storageDevice, 0, 1))
        RETMSG(RSCMU_ERROR_PARAMETER3);
        
    sprintf (wrtBuf, "MMEM:BCHM:SAVE \"%s\",%s" , fileName, RSCMUNB_STOR_DEV[storageDevice]);

    CHECKRETMSG(RSCMU_sendCmd(instrumentHandle, wrtBuf));
    
    RETMSG(RSCMU_Status);    
}

/*===========================================================================*/
/* Function: Buffer Writting                                                 */
/* Purpose:  This function activates/deactivates the data recording into the */
/*           ring buffer.                                                    */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_BchmBwr (ViSession instrumentHandle,
                                       ViInt32 state, 
                                       ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    CHECKRETMSG(RSCMU_setEnum(instrumentHandle,
                              "MMEM:BCHM:BWR",
                              state,
                              RSCMUNB_mode_On_Off,
                              RSCMU_ERROR_PARAMETER2
                             ));
    RETMSG(RSCMU_Status);    
}

/*===========================================================================*/
/* Function: Get Buffer Empty State                                          */
/* Purpose:  This function returns whether or not the buffer is empty.       */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_BchmEmp (ViSession instrumentHandle,
                                       ViInt32 *state, 
                                       ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_getEnum(instrumentHandle,
                                  "MMEM:BCHM:BEMP?",
                                  state,
                                  RSCMUNB_bchmEmpBuff,
                                  RSCMU_ERROR_INTERPRETING_RESPONSE));
    
    RETMSG (RSCMU_Status);
       
}

/*===========================================================================*/
/*===========================================================================*/
/* RACH Test                                                                 */
/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
/* RACH Common                                                               */
/*===========================================================================*/
/*===========================================================================*/

/*===========================================================================*/
/* Function: AWGN Contribution                                               */
/* Purpose:  This function switches the Additional White Gaussian Noise      */
/*           contribution to the RF generator signal on or off.              */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ConfRachCommContAwgn (ViSession instrumentHandle,
                                                    ViInt32 awgn,
                                                    ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    CHECKRETMSG(RSCMU_setEnum(instrumentHandle,
                              "CONF:RACH:COMM:CONT:AWGN",
                              awgn,
                              RSCMUNB_mode_On_Off,
                              RSCMU_ERROR_PARAMETER2
                             ));
    RETMSG(RSCMU_Status);    
}

/*===========================================================================*/
/* Function: Noise Level                                                     */
/* Purpose:  This function sets the level of the Additional White Gaussian   */
/*           Noise contribution to the RF generator signal.                  */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ConfRachCommContNpow (ViSession instrumentHandle,
                                                    ViReal64 noiseLevel,
                                                    ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    CHECKRETMSG(RSCMU_setViReal64(instrumentHandle,
                                "CONF:RACH:COMM:CONT:NPOW",
                                noiseLevel,
                                -80.0, -20.0,
                                RSCMU_ERROR_PARAMETER2));
    RETMSG(RSCMU_Status);    
}

/*===========================================================================*/
/* Function: AICH Transmission Timing                                        */
/* Purpose:  This function sets the timing parameter which adjusts           */
/*           the distance between preamble and message part.                 */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ConfRachCommContAtt (ViSession instrumentHandle,
                                                   ViInt32 transmissionTiming,
                                                   ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    CHECKRETMSG(RSCMU_setViInt32(instrumentHandle,
                                "CONF:RACH:COMM:CONT:ATT",
                                transmissionTiming,
                                0, 1,
                                RSCMU_ERROR_PARAMETER2));
    RETMSG(RSCMU_Status);    
}


/*===========================================================================*/
/* Function: Initiate RACH Measurement                                       */
/* Purpose:  This function starts the RACH Preamble measurement.             */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_InitRachPre (ViSession instrumentHandle,
                                           ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    
    CHECKRETMSG(RSCMU_sendCmd(instrumentHandle,
                                "INIT:RACH:PRE"));
    RETMSG(RSCMU_Status);       
}

/*===========================================================================*/
/* Function: Abort RACH Measurement                                          */
/* Purpose:  This function aborts the RACH Preamble measurement.             */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_AborRachPre (ViSession instrumentHandle,
                                           ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    
    CHECKRETMSG(RSCMU_sendCmd(instrumentHandle,
                                "ABOR:RACH:PRE"));
    RETMSG(RSCMU_Status);    
}

/*===========================================================================*/
/* Function: Stop RACH Measurement                                           */
/* Purpose:  This function stops the RACH Preamble measurement.              */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_StopRachPre (ViSession instrumentHandle,
                                           ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    
    CHECKRETMSG(RSCMU_sendCmd(instrumentHandle,
                                "STOP:RACH:PRE"));
    RETMSG(RSCMU_Status);    
}

/*===========================================================================*/
/* Function: RACH Event Reporting                                            */
/* Purpose:  This function defines the events generated when the measurement */
/*           is terminated or stopped                                        */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ConfRachPreErep (ViSession instrumentHandle,
                                               ViInt32 eventReporting,
                                               ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setEnum(instrumentHandle,
                              "CONF:RACH:PRE:EREP",
                              eventReporting,
                              RSCMUNB_EventReport,
                              RSCMU_ERROR_PARAMETER2
                             ));
    RETMSG(RSCMU_Status);    
}

/*===========================================================================*/
/* Function: Get RACH Measurement Status                                     */
/* Purpose:  This function returns the status of the measurement             */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_FetcRachPreStat (ViSession instrumentHandle,
                                               ViInt32 *status,
                                               ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_getEnum(instrumentHandle,
                              "FETC:RACH:PRE:STAT?",
                              status,
                              RSCMUNB_MeasStatus,
                              RSCMU_ERROR_INSTRUMENT_ERROR
                             ));
    RETMSG(RSCMU_Status);    
}
/*===========================================================================*/
/*===========================================================================*/
/* RACH Preamble                                                             */
/*===========================================================================*/
/*===========================================================================*/

/*===========================================================================*/
/* Function: Preamble Power                                                  */
/* Purpose:  This function defines the Output Power of the RACH preambles.   */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ConfRachPreContPow (ViSession instrumentHandle,
                                                  ViReal64 totalPower,
                                                  ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    
    CHECKRETMSG(RSCMU_setViReal64(instrumentHandle,
                                "CONF:RACH:PRE:CONT:POW",
                                totalPower,
                                -137.0, 9.0,
                                RSCMU_ERROR_PARAMETER2));
    RETMSG(RSCMU_Status);    
}

/*===========================================================================*/
/* Function: Signal To Noise Ration                                          */
/* Purpose:  This function defines the Signal to Noise Ratio of the RACH     */
/*           preambles.                                                      */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ConfRachPreContEcn (ViSession instrumentHandle,
                                                  ViReal64 signalToNoiseRatio,
                                                  ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    
    CHECKRETMSG(RSCMU_setViReal64(instrumentHandle,
                                "CONF:RACH:PRE:CONT:ECN",
                                signalToNoiseRatio,
                                -137.0, 29.0,
                                RSCMU_ERROR_PARAMETER2));
    RETMSG(RSCMU_Status);     
}

/*===========================================================================*/
/* Function: Scrambling Code                                                 */
/* Purpose:  This function selects the scrambling code for the uplink RF     */
/*           generator signal (RACH).                                        */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ConfRachPreContScod (ViSession instrumentHandle,
                                                   ViInt32 scramblingCode,
                                                   ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    
    CHECKRETMSG(RSCMU_setViInt32(instrumentHandle,
                                "CONF:RACH:PRE:CONT:SCOD",
                                scramblingCode,
                                0, 16777215,
                                RSCMU_ERROR_PARAMETER2));
    RETMSG(RSCMU_Status);     
}


/*===========================================================================*/
/* Function: Number of Preambles                                             */
/* Purpose:  This function defines the number of preambles that will be send.*/
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ConfRachPreContNrpr (ViSession instrumentHandle,
                                                   ViInt32 number_ofPreambles,
                                                   ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    
    CHECKRETMSG(RSCMU_setViInt32(instrumentHandle,
                                "CONF:RACH:PRE:CONT:NRPR",
                                number_ofPreambles,
                                0, 16777216,
                                RSCMU_ERROR_PARAMETER2));
    RETMSG(RSCMU_Status);      
}

/*===========================================================================*/
/* Function: AICH Spreading Code Number                                      */
/* Purpose:  This function defines the AICH Spreading Code Number.           */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ConfRachPreContAscn (ViSession instrumentHandle,
                                                   ViInt32 codeNumber,
                                                   ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    
    CHECKRETMSG(RSCMU_setViInt32(instrumentHandle,
                                "CONF:RACH:PRE:CONT:ASCN",
                                codeNumber,
                                0, 255,
                                RSCMU_ERROR_PARAMETER2));
    RETMSG(RSCMU_Status);         
}

/*===========================================================================*/
/* Function: SFN Number                                                      */
/* Purpose:  This function defines the Activation System Frame Number of the */
/*           first preamble to be sent.                                      */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ConfRachPreContSfnn (ViSession instrumentHandle,
                                                   ViInt32 SFNNumber,
                                                   ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    
    CHECKRETMSG(RSCMU_setViInt32(instrumentHandle,
                                "CONF:RACH:PRE:CONT:SFNN",
                                SFNNumber,
                                0, 4095,
                                RSCMU_ERROR_PARAMETER2));
    RETMSG(RSCMU_Status);      
}

/*===========================================================================*/
/* Function: SFN Mode                                                        */
/* Purpose:  This function defines the Start Mode of the Rach Preamble Test. */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ConfRachPreContSfnm (ViSession instrumentHandle,
                                                   ViInt32 SFNMode,
                                                   ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    
    CHECKRETMSG(RSCMU_setEnum(instrumentHandle,
                                "CONF:RACH:PRE:CONT:SFNM",
                                SFNMode,
                                RSCMUNB_rachPreSfnm,
                                RSCMU_ERROR_PARAMETER2));
    RETMSG(RSCMU_Status);      
}

/*===========================================================================*/
/* Function: Number of Access Slots in List                                  */
/* Purpose:  This function defines the number of predefined preambles in     */
/*           a cycle ( using the Access Slot List).                          */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ConfRachPreContNasl (ViSession instrumentHandle,
                                                   ViInt32 number_ofAccessSlots,
                                                   ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    
    CHECKRETMSG(RSCMU_setViInt32(instrumentHandle,
                                "CONF:RACH:PRE:CONT:NASL",
                                number_ofAccessSlots,
                                1,
                                64,
                                RSCMU_ERROR_PARAMETER2));
    RETMSG(RSCMU_Status);          
}

/*===========================================================================*/
/* Function: Access Slots List                                               */
/* Purpose:  This function defines an Access Frame Number, an Access Slot    */
/*           and the Signature for Access Slot Table line number.            */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ConfRachPreContAst (ViSession instrumentHandle,
                                                  ViInt32 lineNumber,
                                                  ViInt32 frame, 
                                                  ViInt32 slot,
                                                  ViInt32 signature,
                                                  ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    
    GET_WRT_BUF (instrumentHandle);
    
    if (RSCMU_invalidViInt32Range (lineNumber, 1, 64))
        RETMSG(RSCMU_ERROR_PARAMETER2);
    if (RSCMU_invalidViInt32Range (frame, 0, 7))
        RETMSG(RSCMU_ERROR_PARAMETER3);
    switch (frame){
        case 0:
        case 2:
        case 4:
        case 6:
               if (RSCMU_invalidViInt32Range (slot, 0, 7))
                    RETMSG(RSCMU_ERROR_PARAMETER4);    
        break;
        case 1:
        case 3:
        case 5:
        case 7:
               if (RSCMU_invalidViInt32Range (slot, 8, 14))
                    RETMSG(RSCMU_ERROR_PARAMETER4);    
        break;                    
    }
    if (RSCMU_invalidViInt32Range (signature, 1, 64))
        RETMSG(RSCMU_ERROR_PARAMETER5);

    sprintf (wrtBuf, "CONF:RACH:PRE:CONT:AST%ld %ld,%ld,%ld",
            lineNumber, frame, slot, signature);
    CHECKRETMSG(RSCMU_sendCmd(instrumentHandle,
                                wrtBuf));
                                
    RETMSG(RSCMU_Status);     
}

/*===========================================================================*/
/* Function: Default Settings                                                */
/* Purpose:  This function sets all parameters of the subsystem to default   */
/*           values.                                                         */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_DefRachCont (ViSession instrumentHandle,
                                           ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_sendCmd(instrumentHandle,
                                "DEF:RACH:CONT ON"));
                                
    RETMSG(RSCMU_Status);         
}

/*===========================================================================*/
/* Function: Get Scalar RACH Preamble Results                                */
/* Purpose:  This function starts a RACH Preamble test and return            */
/*           the measurement results.                                        */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_GetScalRachPreResults (ViSession instrumentHandle,
                                                     ViInt32 mode,
                                                     ViReal64 arrayOfResults[],
                                                     ViChar errorMessage[])
{
    ViStatus  RSCMU_Status    = VI_SUCCESS;
    ViChar   *strValue;
    int      i                = 0;
    ViChar   wrtBuf[WR_BUFFER_SIZE];
    
    GET_RD_BUF (instrumentHandle);
    
    if (RSCMU_invalidViInt32Range (mode, 0, 1))
        RETMSG(RSCMU_ERROR_PARAMETER2);

    sprintf(wrtBuf,"%s:RACH:PRE?",RSCMUNB_Cmd[mode]);

    CHECKRETMSG(RSCMU_getViString(instrumentHandle, wrtBuf, rdBuf, RD_BUFFER_SIZE_ARRAY, 0));

    strValue = strtok(rdBuf, ",");

    while (strValue)
    {
        if (strcmp (strValue, "NAN") != 0)
            sscanf (strValue, "%le", &arrayOfResults[i]);
        else
            arrayOfResults[i] = RSCMU_NAN;

        strValue = strtok (NULL, ",");
        i++;
    }

    RETMSG(RSCMU_Status);
    
}

/*===========================================================================*/
/*===========================================================================*/
/* DL BER Test                                                               */
/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
/* Configuration                                                             */
/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
/* Function: Repetition                                                      */
/* Purpose:  This function determines the number of statistics cycles, the   */
/*           stop condition and the stepping mode for the measurement.       */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ConfDlbBerContRep (ViSession instrumentHandle,
                                        ViInt32 repetitionMode,
                                        ViInt32 stopCondition,
                                        ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    ViChar    value[WR_BUFFER_SIZE] = "";

    if (RSCMU_invalidViInt32Range (repetitionMode, 0, 1))
        RETMSG(RSCMU_ERROR_PARAMETER2);

    if (RSCMU_invalidViInt32Range (stopCondition, 0, 1))
        RETMSG(RSCMU_ERROR_PARAMETER3);

    sprintf(value, "CONFigure:DLBer:BER:CONTrol:REPetition %s,%s,NONE", RSCMUNB_Repetition[repetitionMode], RSCMUNB_StopCondition[stopCondition]);
    CHECKRETMSG(RSCMU_sendCmd(instrumentHandle, value));

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Event Reporting                                                 */
/* Purpose:  This function defines the events generated when the measurement */
/*           is terminated or stopped (event reporting).                     */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ConfDlbBerErep (ViSession instrumentHandle,
                                     ViInt32 eventReporting,
                                     ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setEnum(instrumentHandle,
                              "CONFigure:DLBer:BER:EREPorting",
                              eventReporting,
                              RSCMUNB_EventReport,
                              RSCMU_ERROR_PARAMETER2));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Transport Blocks                                                */
/* Purpose:  This function defines the number of transport blocks to be used */
/*           for averaging in continuous measurements or to be sent in each  */
/*           single shot measurement cycle.                                  */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ConfDlbBerContTb (ViSession instrumentHandle,
                                       ViInt32 mode,
                                       ViInt32 transportBlocks,
                                       ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    if (RSCMU_invalidViInt32Range (mode, 0, 1))
        RETMSG(RSCMU_ERROR_PARAMETER2);

    switch (mode){
        case RSCMU_CONTINUOUS:
            CHECKRETMSG(RSCMU_setViInt32(instrumentHandle,
                              "CONFigure:DLBer:BER:CONTrol:TBContinuous",
                              transportBlocks,
                              10,
                              10000,
                              RSCMU_ERROR_PARAMETER2));
        break;
        case RSCMU_SINGLE_SHOT:
            CHECKRETMSG(RSCMU_setViInt32(instrumentHandle,
                              "CONFigure:DLBer:BER:CONTrol:TBSingleshot",
                              transportBlocks,
                              10,
                              100000,
                              RSCMU_ERROR_PARAMETER2));
        break;
    }
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: AGC Holdoff Time                                                */
/* Purpose:  This function defines the hold off time during which the        */
/*           R&S CMU can adapt itself to the new RF level at the beginning   */
/*           of the receiver quality measurement.                            */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ConfDlbBerContAhot (ViSession instrumentHandle,
                                         ViReal64 AGCHoldoffTime,
                                         ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setViReal64(instrumentHandle,
                              "CONFigure:DLBer:BER:CONTrol:AHOTime",
                              AGCHoldoffTime,
                              0.0,
                              100.0,
                              RSCMU_ERROR_PARAMETER2));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Sync Holdoff Time                                               */
/* Purpose:  This function defines the maximum time during which the R&S CMU */
/*           can synchronize to the received bit stream.                     */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ConfDlbBerContShot (ViSession instrumentHandle,
                                         ViReal64 syncHoldoffTime,
                                         ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setViReal64(instrumentHandle,
                              "CONFigure:DLBer:BER:CONTrol:SHOTime",
                              syncHoldoffTime,
                              0.0,
                              100.0,
                              RSCMU_ERROR_PARAMETER2));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: DL BER Default                                                  */
/* Purpose:  This function sets all parameters of the subsystem to default   */
/*           values (the setting OFF results in an error message).           */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_DefDlbBerCont (ViSession instrumentHandle,
                                    ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_sendCmd(instrumentHandle, "DEFault:DLBer:BER:CONTrol ON"));

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: DL BER Automatic Resync                                         */
/* Purpose:  This function defines if an automatic resynchronization to PRBS */
/*           pattern should be performed in case of a loss of synchronization*/
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ConfDlbBerContPar (ViSession instrumentHandle,
                                                ViInt32 automaticResyncState,
                                                ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setEnum(instrumentHandle,
                              "CONF:DLB:BER:CONT:PAR",
                              automaticResyncState,
                              RSCMUNB_mode_On_Off,
                              RSCMU_ERROR_INSTRUMENT_ERROR));
    RETMSG(RSCMU_Status);      
}


/*===========================================================================*/
/*===========================================================================*/
/* Control                                                                   */
/*===========================================================================*/
/*===========================================================================*/

/*===========================================================================*/
/* Function: Initiate                                                        */
/* Purpose:  This function starts a new receiver quality measurement.        */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_InitDlbBer (ViSession instrumentHandle,
                                 ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_sendCmd(instrumentHandle, "INITiate:DLBer:BER"));

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Abort                                                           */
/* Purpose:  This function aborts running receiver quality measurement and   */
/*           switches off.                                                   */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_AborDlbBer (ViSession instrumentHandle,
                                 ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_sendCmd(instrumentHandle, "ABORt:DLBer:BER"));

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Stop                                                            */
/* Purpose:  This function stops the receiver quality measurement.           */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_StopDlbBer (ViSession instrumentHandle,
                                 ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_sendCmd(instrumentHandle, "STOP:DLBer:BER    "));

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Measurement Status                                              */
/* Purpose:  This function returns the status of measurement.                */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_FetcDlbBerStat (ViSession instrumentHandle,
                                     ViInt32 *measurementStatus,
                                     ViInt32 *reserved,
                                     ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    ViChar*   p2buf;
    
    GET_RD_BUF( instrumentHandle );
    

    CHECKRETMSG(RSCMU_getViString(instrumentHandle,
                                  "FETCh:DLBer:BER:STATus?",
                                  rdBuf,
                                  RD_BUFFER_SIZE,
                                  RSCMU_ERROR_INTERPRETING_RESPONSE));
    
    
    p2buf = strtok (rdBuf, ",");
    
    RETMSG (RSCMU_getIndexFromString (instrumentHandle, p2buf, RSCMUNB_MeasStatus, measurementStatus));
}

/*===========================================================================*/
/*===========================================================================*/
/* Results                                                                   */
/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
/* Function: Measurement Status                                              */
/* Purpose:  This function starts a single shot measurement and return       */
/*           results.                                                        */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_GetDlbBer (ViSession instrumentHandle,
                                ViUInt32 mode,
                                ViReal64 arrayOfResults[],
                                ViChar errorMessage[])
{
    ViStatus RSCMU_Status    = VI_SUCCESS;
    ViChar   cmd[WR_BUFFER_SIZE]       = "";
    ViChar   rdBuf[RD_BUFFER_SIZE]     = "";

    if (RSCMU_invalidViInt32Range (mode, 0, 1))
        RETMSG(RSCMU_ERROR_PARAMETER2);

    if (!arrayOfResults)
        RETMSG(RSCMU_ERROR_PARAMETER3);

    sprintf(cmd,"%s:SCALar:DLBer:BER?",RSCMUNB_Cmd[mode]);

    CHECKRETMSG(RSCMU_getViString(instrumentHandle, cmd, rdBuf, RD_BUFFER_SIZE, 0));

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Check Limits                                                    */
/* Purpose:  This function indicates whether and in which way the permissible*/
/*           tolerances for the scalar results in the measurement have been  */
/*           exceeded.                                                       */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_CalcDlbBerMatcLim (ViSession instrumentHandle,
                                        ViInt32 *BER,
                                        ViInt32 *BLER,
                                        ViInt32 *DBLER,
                                        ViChar errorMessage[])
{
    ViStatus  RSCMU_Status    = VI_SUCCESS;
    ViChar    rdBuf[RD_BUFFER_SIZE]     = "";
    ViChar    tableOfDataValues[100][100];
    ViInt32   values[3] = {0,0,0};
    ViInt32   i;

    if (!BER)
        RETMSG(RSCMU_ERROR_PARAMETER2);
    if (!BLER)
        RETMSG(RSCMU_ERROR_PARAMETER3);
    if (!DBLER)
        RETMSG(RSCMU_ERROR_PARAMETER4);

    CHECKRETMSG(RSCMU_getViString(instrumentHandle, "CALCulate:DLBer:BER:MATChing:LIMit?", rdBuf, RD_BUFFER_SIZE, 0));
    // Split the multiple data values separated by a comma:
    RSCMU_SeparateDataValues (rdBuf, ",", tableOfDataValues);

    for (i=0;i<3;i++)
        CHECKRET (RSCMU_GetLimitCheckValue (tableOfDataValues, i, &values[i]));

    *BER = values[0];
    *BLER = values[1];
    *DBLER = values[2];

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/*===========================================================================*/
/* UL and DL Signal Configuration                                            */
/*===========================================================================*/
/*===========================================================================*/

/*===========================================================================*/
/* Function: Total Transmit Power (BER)                                      */
/* Purpose:  This function sets the Total TX Power in absolute units.        */
/*           The DPCCH/DPDCH power ratio and the number of DPDCHs is         */
/*           considered to be fixed for all channel configurations, so the   */
/*           command overwrites the value of                                 */
/*           CONFigure:DLBer:BER:UESignal:DPCCh                              */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ConfDlbBerUesTtp (ViSession instrumentHandle,
                                       ViReal64 totalTransmitPower,
                                       ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setViReal64(instrumentHandle,
                              "CONFigure:DLBer:BER:UESignal:TTPower",
                              totalTransmitPower,
                              -137.0,
                              0.0,
                              RSCMU_ERROR_PARAMETER2));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: DPCCH Level (BER)                                               */
/* Purpose:  This function sets the level of the Dedicated Physical Control  */
/*           Channel (DPCCH). The DPCCH/DPDCH power ratio and the number of  */
/*           DPDCHs is considered to be fixed for all channel configurations,*/
/*           so the command overwrites the value of                          */
/*           CONFigure:UESignal:LEVel:TTPower.                               */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ConfDlbBerUesDpcc (ViSession instrumentHandle,
                                        ViReal64 DPCCHLevel_BER,
                                        ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setViReal64(instrumentHandle,
                              "CONFigure:DLBer:BER:UESignal:DPCCh",
                              DPCCHLevel_BER,
                              -137.0,
                              -10.0,
                              RSCMU_ERROR_PARAMETER2));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Noise Level (BER)                                               */
/* Purpose:  This function the level of the Additional White Gaussian Noise  */
/*           contribution to the RF generator signal.                        */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ConfDlbBerUesNpow (ViSession instrumentHandle,
                                                 ViReal64 noiseLevel,
                                                 ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setViReal64(instrumentHandle,
                              "CONF:DLB:BER:UES:NLEV",
                              noiseLevel,
                              -137.0,
                              -23.0,
                              RSCMU_ERROR_PARAMETER2));
    RETMSG(RSCMU_Status);    
}

/*===========================================================================*/
/* Function: AWGN Contribution (BER)                                         */
/* Purpose:  This function switches the Additional White Gaussian Noise      */
/*           contribution to the RF generator signal on or off.              */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ConfDlbBerUesAwgn (ViSession instrumentHandle,
                                                 ViInt32 awgn,
                                                 ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setEnum(instrumentHandle,
                              "CONF:DLB:BER:UES:AWGN",
                              awgn,
                              RSCMUNB_mode_On_Off,
                              RSCMU_ERROR_PARAMETER2));
    RETMSG(RSCMU_Status);       
}

/*===========================================================================*/
/* Function: Channel Data Source DTCH                                        */
/* Purpose:  This function specifies the PRBS data sequence that the R&S CMU */
/*           transmits on the UL DTCH.                                       */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ConfDlbBerUesDtch (ViSession instrumentHandle,
                                        ViInt32 sequence,
                                        ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    CHECKRETMSG(RSCMU_setEnum(instrumentHandle,
                              "CONFigure:DLBer:BER:UESignal:DTCH",
                              sequence,
                              RSSMUNB_SEQ_PR,
                              RSCMU_ERROR_PARAMETER2
                             ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: NB Channel Data Source DTCH                                     */
/* Purpose:  This function specifies the PRBS data sequence that the R&S CMU */
/*           expects to be transmitted on the DL DTCH. In loopback mode the  */
/*           PRBS sequence is the same as the UL sequence selected via       */
/*           CONFigure:DLBer:BER:UESignal:DTCH.                              */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ConfDlbBerNbsDtch (ViSession instrumentHandle,
                                        ViInt32 sequence,
                                        ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    CHECKRETMSG(RSCMU_setEnum(instrumentHandle,
                              "CONFigure:DLBer:BER:NBSignal:DTCH",
                              sequence,
                              RSCMUNB_DLBER_SEQ_PR,
                              RSCMU_ERROR_PARAMETER2
                             ));
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/*===========================================================================*/
/* UL and DL Signal Configuration                                            */
/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
/* Function: Limits                                                          */
/* Purpose:  This function defines upper limits for the BER, the BLER and    */
/*           the DBLER.                                                      */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_ConfDlbCurrLimAsymUppVal (ViSession instrumentHandle,
                                                        ViReal64 BER,
                                                        ViReal64 BLER,
                                                        ViReal64 DBLER,
                                                        ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    ViChar    cmd[WR_BUFFER_SIZE]     = "";

    if (RSCMU_invalidViReal64Range (BER, 0.0, 100.0))
       RETMSG(RSCMU_ERROR_PARAMETER2);
    if (RSCMU_invalidViReal64Range (BLER, 0.0, 100.0))
       RETMSG(RSCMU_ERROR_PARAMETER3);
    if (RSCMU_invalidViReal64Range (DBLER, 0.0, 100.0))
       RETMSG(RSCMU_ERROR_PARAMETER4);

    sprintf(cmd, "CONFigure:DLBer:BER:CURRent:LIMit:SCALar:ASYMmetric:UPPer:VALue %.12f,%.12f,%.12f", BER, BLER, DBLER);

    CHECKRETMSG(RSCMU_sendCmd(instrumentHandle, cmd));

    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Default Limit Line                                              */
/* Purpose:  This function sets all parameters of the subsystem to default   */
/*           values (the setting OFF results in an error message).           */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Sig_DefDlbBerLim (ViSession instrumentHandle,
                                   ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_sendCmd(instrumentHandle,
                                "DEFault:DLBer:BER:LIMit ON"));
    RETMSG(RSCMU_Status);
}




/*===========================================================================*/
/* Function: Change Destination                                              */
/* Purpose:  This function changes the default file name and path for logging*/
/*           files in the current storage device. The command is             */
/*           CMU-specific.                                                   */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_ChangeDestination (ViSession instrumentHandle,
                                             ViChar fileName[],
                                             ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    GET_WRT_BUF (instrumentHandle);

    if (!fileName)
        RETMSG(RSCMU_ERROR_PARAMETER2);
        
    sprintf (wrtBuf, "MMEMory:L3MSg:CDEStination \"%s\"", fileName);

    CHECKRETMSG(RSCMU_sendCmd(instrumentHandle, wrtBuf));
    
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Change Destination                                              */
/* Purpose:  This function saves the current ring buffer content to the      */
/*           default logging file INTERNAL\LOG\WCDMA_L3_?.LOG (if no         */
/*           parameter is specified, see command MMEMory:L3MSg:CDEStination) */
/*           or to the specified file and storage device. In the default file*/
/*           name "WCDMA_L3_?.LOG" the "?" is replaced by current numbers    */
/*           that are automatically incremented, starting with               */
/*           zero. The auto-increment function overwrites an existing file   */
/*           with a "9" in its file name. The command is CMUspecific.        */                                      
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_SaveToFile (ViSession instrumentHandle,
                                      ViChar fileName[],
                                      ViInt32 storageDevice,
                                      ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    GET_WRT_BUF (instrumentHandle);

    if (!fileName)
        RETMSG(RSCMU_ERROR_PARAMETER2);
        
    sprintf (wrtBuf, "MMEMory:L3MSg:SAVE \"%s\", %s", fileName, RSCMUNB_STOR_DEV[storageDevice]);

    CHECKRETMSG(RSCMU_sendCmd(instrumentHandle, wrtBuf));
    
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Buffer Writing                                                  */
/* Purpose:  This function controls data recording into the ring buffer. The */
/*           command is CMU-specific.                                        */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_BufferWriting (ViSession instrumentHandle,
                                         ViInt32 bufferWriting,
                                         ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setEnum(instrumentHandle,
                              "MMEMory:L3MSg:BWRiting",
                              bufferWriting,
                              RSCMUNB_mode_On_Off,
                              RSCMU_ERROR_PARAMETER2));
    RETMSG(RSCMU_Status);      
}

/*===========================================================================*/
/* Function: Buffer Empty                                                    */
/* Purpose:  This function returns whether or not the buffer is empty. The   */
/*           command is CMU-specific.                                        */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_BufferEmpty (ViSession instrumentHandle,
                                       ViInt32 *bufferWriting,
                                       ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_getEnum(instrumentHandle,
                              "MMEMory:L3MSg:BEMPty?",
                              bufferWriting,
                              RSCMUNB_bufferWriting,
                              RSCMU_ERROR_INSTRUMENT_ERROR
                             ));
    RETMSG(RSCMU_Status);    
}

/*===========================================================================*/
/* Function: Logging CDP                                                     */
/* Purpose:  This function enables or disables recording of the Code Domain  */
/*           Power information and selects a destination for the recorded    */
/*           information.                                                    */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_LoggingCdp (ViSession instrumentHandle,
                                           ViInt32 recordingDestination,
                                           ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setEnum(instrumentHandle,
                              "CONFigure:LOGGing:CDPower",
                              recordingDestination,
                              RSCMUNB_recDestination,
                              RSCMU_ERROR_PARAMETER2));
    RETMSG(RSCMU_Status);      
}


/*===========================================================================*/
/* Function: Logging BCH Monitor                                             */
/* Purpose:  This function enables or disables recording of the BCH Monitor  */
/*           information and selects a destination for the recorded          */
/*           information.                                                    */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_LoggingBchMonitor (ViSession instrumentHandle,
                                                  ViInt32 recordingDestination,
                                                  ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setEnum(instrumentHandle,
                              "CONFigure:LOGGing:BCHMonitor",
                              recordingDestination,
                              RSCMUNB_recDestination,
                              RSCMU_ERROR_PARAMETER2));
    RETMSG(RSCMU_Status);      
}


/*===========================================================================*/
/* Function: Logging Transport Channel                                       */
/* Purpose:  This function enables or disables recording of the Transport    */
/*           Channel information and selects a destination for the recorded  */
/*           information.                                                    */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_LoggingTransportChannel (ViSession instrumentHandle,
                                                        ViInt32 recordingDestination,
                                                        ViChar errorMessage[])

{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_setEnum(instrumentHandle,
                              "CONFigure:LOGGing:TCHannel",
                              recordingDestination,
                              RSCMUNB_recDestination,
                              RSCMU_ERROR_PARAMETER2));
    RETMSG(RSCMU_Status);      
}

/*===========================================================================*/
/* Function: Default Logging                                                 */
/* Purpose:  This function sets all parameters of the subsystem to default   */
/*           values.                                                         */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_Nsig_DefaultLogging (ViSession instrumentHandle,
                                               ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;

    CHECKRETMSG(RSCMU_sendCmd(instrumentHandle, "DEFault:LOGging ON"));
    
    RETMSG(RSCMU_Status);
}


        

/*###########################################################################*/
/*                                                                           */
/*  VXI PLUG & PLAY COMPATIBILITY FUNCTIONS                                  */
/*                                                                           */
/*  These functions provide a VXIplug&play compatibily of this instrument    */
/*  driver with regard to the following documents:                           */
/*                                                                           */
/*  VPP-3.1: Instrument Drivers                                              */
/*      Architecture and Design                                              */
/*      Specification                                                        */
/*      Revision 4.0                                                         */
/*      February 5, 1996                                                     */
/*                                                                           */
/*  VPP-3.2: Instrument Driver                                               */
/*      Functional Body Specification                                        */
/*      Revision 4.0                                                         */
/*      February 2, 1996                                                     */
/*                                                                           */
/* By: Jiri Kominek                                                          */
/*                                                                           */
/*###########################################################################*/

#ifndef _REMVXIPNP

/*===========================================================================*/
/* Function: Initialize                                                      */
/* Purpose:  This function opens the instrument, queries the instrument      */
/*           for its ID, and initializes the instrument to a known state.    */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_init (ViRsrc resourceName, ViBoolean IDQuery,
                        ViBoolean resetDevice, ViSession *instrumentHandle)
{
    ViStatus   RSCMU_status = VI_SUCCESS;
    ViString   p2str;
    ViInt32    board = 0,
            primaryAddr = 0,
            secondaryAddr = 0;
    ViChar     errorMessage[WR_BUFFER_SIZE];
    ViSession  rmHandle;

    p2str = resourceName;

    /* --- Set standard "C" locale --- */
    {
        char *locale;

        locale = setlocale(LC_ALL, "C");
    }

   if (strncmp (p2str, "GPIB", 4) != 0)
      return VI_ERROR_INV_RSRC_NAME;

   p2str += 4;

   if (p2str[0] >= '0' && p2str[0] <= '9')
      board = (p2str++)[0] - '0';

   if (strncmp (p2str, "::", 2) != 0)
      return VI_ERROR_INV_RSRC_NAME;

   if (sscanf (p2str += 2, "%ld", &primaryAddr) != 1)
      return VI_ERROR_INV_RSRC_NAME;

   p2str += (primaryAddr > 9) ? 2 : 1;

   if (strlen (p2str) != 0 && strncmp (p2str, "::", 2) != 0)
      return VI_ERROR_INV_RSRC_NAME;

   if (strlen (p2str) != 0) {
      if (sscanf (p2str += 2, "%ld", &secondaryAddr) != 1)
         return VI_ERROR_INV_RSRC_NAME;

      p2str += (secondaryAddr > 9) ? 2 : 1;

      if (strlen (p2str) != 0 && strcmp (p2str, "::INSTR") != 0)
         return VI_ERROR_INV_RSRC_NAME;
   }

   if (secondaryAddr == 0) {
      if ((RSCMU_status = RSCMU_Init_Base (instrumentHandle, RSCMU_GPIB, board, primaryAddr,
                                  IDQuery, resetDevice, errorMessage)) < 0)
         return RSCMU_status;
   } else {
      if ((RSCMU_status = viOpenDefaultRM (&rmHandle)) != 0)
         return RSCMU_status;

      if ((RSCMU_status = viOpen (rmHandle, resourceName, VI_NULL, VI_NULL, instrumentHandle)) != 0)
         return RSCMU_status;

       if ((RSCMU_status = viGpibControlREN (*instrumentHandle, VI_GPIB_REN_ASSERT)) < 0)
          return RSCMU_status;
    }

    return RSCMU_status;
}

/*===========================================================================*/
/* Function: Get Instrument Handle                                           */
/* Purpose:  This function passes the instrument handle from the first       */
/*           parameter to the second parameter. This function is used in VEE */
/*           where it allows to get the current instrument handle.          */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_GetInstrumentHandle (ViSession instrumentHandle,
                                       ViSession *instrumentHandleOut)
{
   *instrumentHandleOut = instrumentHandle;
    return 0;
}

/*===========================================================================*/
/* Function: Init                                                            */
/* Purpose:  This function initializes the NB module's                       */
/*           global data structures. This function is used in VEE where this */
/*           can not be a part of the VXIpnp init function.                  */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_initNB (ViSession instrumentHandle,
                          ViSession baseInstrumentHandle,
                          ViInt32 nodeBFunctionGroup)
{
  ViStatus   RSCMU_Status      = VI_SUCCESS;
  ViInt32    secondAddr        = 0;
  ViChar     FctGrpBuffer[128] = "";
  ViInt32    comp              = 1;
  ViBoolean  Flag              = RSCMU_OFF;
  ViSession  DefaultRM;
  ViChar     baseRevString[RD_BUFFER_SIZE]     ="";
  ViChar     nbRevString[RD_BUFFER_SIZE]       ="";
  ViChar     rdBuf[MID_BUFFER_SIZE]            ="";
  ViChar     search[RD_BUFFER_SIZE] = "";
  ViChar      errorMessage[WR_BUFFER_SIZE];
  ViUInt32    fgroup;

  struct RSCMU_Base_Globals   *p_globalvarBase;
  struct RSCMUNB_NSig_Globals  *p_globalvarNB;

    /* --- Set standard "C" locale --- */
    {
        char *locale;

        locale = setlocale(LC_ALL, "C");
    }

  // Get the data used privately by this driver for this instrument session:
  if ((RSCMU_Status = viGetAttribute (baseInstrumentHandle, VI_ATTR_USER_DATA, (ViAddr)&p_globalvarBase)) < 0)
    return RSCMUNB_errorMessage (baseInstrumentHandle, RSCMU_Status, errorMessage);

  DefaultRM = p_globalvarBase->RSCMU_tGlobal.RSCMU_RmSessionDefault;

  // Verify that the VISA Resource Manager has been opened.
  if (DefaultRM != 0)
  {
    // Record "Function Group Name" for the actual Session.
    fgroup = (nodeBFunctionGroup == RSCMU_WCDMA_NB_SIG) ? RSCMUNB_WCDMA1900FDD_SIG : RSCMUNB_WCDMA1900FDD_NSIG;

//    strcpy (FctGrpBuffer, [fgroup]"\"WCDMA1900NBFDD_NSig\"");
    strcpy (FctGrpBuffer, RSCMUNB_function_group_name[fgroup]);

    // Init the Error Message
    strcpy (errorMessage, "No Error");

    // Find the Secondary address
    while ((comp != 0) && (secondAddr < 30))
    {
      comp = strncmp (p_globalvarBase->RSCMU_tSecAddr[secondAddr].FctGrp,
                      FctGrpBuffer,
                      strlen (FctGrpBuffer));
      secondAddr++;
      if (comp == 0)
      {
        Flag = RSCMU_ON;
      }
    }

    if (Flag == RSCMU_OFF)
    {
      return RSCMUNB_errorMessage (instrumentHandle, RSCMU_ERROR_INVALID_FCT_GRP, errorMessage);
    }

    // Allocate space for "struct RSCMU_SESSION_Globals"
    if ((p_globalvarNB = (struct RSCMUNB_NSig_Globals *) malloc(sizeof(struct RSCMUNB_NSig_Globals))) == 0)
    {
      return RSCMU_initCleanUp (DefaultRM, instrumentHandle, RSCMU_Status);
    }

    if ((RSCMU_Status = viSetAttribute (instrumentHandle, VI_ATTR_USER_DATA, (ViAttrState)p_globalvarNB)) < 0)
    {
      return RSCMU_initCleanUp (DefaultRM, instrumentHandle, RSCMU_Status);
    }

    // Set initial values of variables for function Group 'BT_SIG' .
    strcpy (p_globalvarNB->RSCMU_tGlobal.FGrp_Name, FctGrpBuffer);

    p_globalvarNB->RSCMU_tGlobal.RSCMU_primary_address = p_globalvarBase->RSCMU_tGlobal.RSCMU_primary_address;
    p_globalvarNB->RSCMU_tGlobal.RSCMU_secondary_address = secondAddr - 1;
    p_globalvarNB->RSCMU_tGlobal.RSCMU_TRACE = p_globalvarBase->RSCMU_tGlobal.RSCMU_TRACE;

    // Record the execution of this Init Function.
    p_globalvarBase->RSCMU_tSecAddr[secondAddr - 1].SessionNb = instrumentHandle;

    // Base CMU FW available
    strcpy (p_globalvarNB->RSCMU_tGlobal.CmuVersion, p_globalvarBase->RSCMU_tGlobal.CmuVersion);
    p_globalvarNB->RSCMU_tGlobal.RSCMU_FW_Version = p_globalvarBase->RSCMU_tGlobal.RSCMU_FW_Version;

    CHECKRET(RSCMUNB_Get_NB_FW_Version(instrumentHandle, &RSCMUNB_nbFwVersion, &RSCMUNB_baseFwVersion));

    CHECKRET(RSCMU_DrvVersionStringToIntegers(RSCMUNB_VERSION,&RSCMUNB_nbDrvVersion,&RSCMUNB_baseDrvVersion));
//  RSCMUBT_Get_BT_DRV_Version( &RSCMUBT_baseDrvVersion, &RSCMUBT_btDrvVersion);
    strcpy(nbRevString, RSCMUNB_REVISION);
    strcpy(baseRevString, RSCMU_BASE_REVISION);

    // check, if we use a correct firmware version
    if (RSCMUNB_nbFwVersion < 3201399)  // Version V3.20M
    {
      RSCMU_Status = RSCMUNB_ERROR_FW_VERSION;
        RSCMU_Base_errorMessage (instrumentHandle, RSCMU_Status, errorMessage);
        RSCMU_initCleanUp (DefaultRM, instrumentHandle, RSCMU_Status);
        return RSCMU_Status;
    }

    // check, if we use a base driver version >= 3.10.00
    if (RSCMUNB_nbFwVersion < 3100000)  // Version 3.10.00
    {
        RSCMU_Status = RSCMUNB_ERROR_BASEDRIVER_VERSION;
        RSCMU_Base_errorMessage (instrumentHandle, RSCMU_Status, errorMessage);
        RSCMU_initCleanUp (DefaultRM, instrumentHandle, RSCMU_Status);
        return RSCMU_Status;
    }

    CHECKRETMSG(RSCMU_getViString(baseInstrumentHandle,"*OPT?",rdBuf,MID_BUFFER_SIZE,0));
    strcpy(search, "K75");
    if(strstr(rdBuf,search));
        K75Present = VI_TRUE;
    strcpy(search, "K76");
    if( NULL == strstr(rdBuf,search));
        K76Present = VI_TRUE;

    /* K78 option is same as K76 */
    strcpy(search, "K78");
    if( NULL == strstr(rdBuf,search));
        K76Present = VI_TRUE;

    // Timeout changed to 15s
    if ((RSCMU_Status = viSetAttribute (instrumentHandle, VI_ATTR_TMO_VALUE, 15000)) < 0)
    {
      return (RSCMU_Status);
    }
  }

  return (RSCMU_Status);
}

/*===========================================================================*/
/* Function: Reset                                                           */
/* Purpose:  This function resets the instrument.  If the reset function     */
/*           is not supported by the instrument, this function returns       */
/*           the warning VI_WARN_NSUP_RESET.                                 */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_reset (ViSession instrumentHandle)
{
   return VI_WARN_NSUP_RESET;
}

/*===========================================================================*/
/* Function: Self-Test                                                       */
/* Purpose:  This function executes the instrument self-test and returns     */
/*           the result. If the self test function is not supported by the   */
/*           instrument, this function returns the warning                   */
/*           VI_WARN_NSUP_SELF_TEST.                                         */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_self_test (ViSession instrumentHandle, ViInt16 *selfTestResult,
                             ViChar selfTestMessage[])
{
    return VI_WARN_NSUP_SELF_TEST;
}

/*===========================================================================*/
/* Function: Error Query                                                     */
/* Purpose:  This function queries the instrument error queue, and returns   */
/*           the result.                                        */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_error_query (ViSession instrumentHandle, ViInt32 *errorCode,
                               ViChar errorMessage[])
{
   return VI_WARN_NSUP_ERROR_QUERY;
}

/*===========================================================================*/
/* Function: Error Message                                                   */
/* Purpose:  This function translates the error return value from the        */
/*           instrument driver into a user-readable string.                  */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_error_message (ViSession instrumentHandle, ViStatus statusCode,
                                 ViChar message[])
{
   return RSCMUNB_errorMessage (instrumentHandle, statusCode, message);
}

/*===========================================================================*/
/* Function: Revision Query                                                  */
/* Purpose:  This function returns the driver and instrument revisions.      */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_revision_query (ViSession instrumentHandle,
                                  ViChar instrumentDriverRevision[],
                                  ViChar firmwareRevision[])
{
   ViChar   message[1024];
   return RSCMUNB_RevisionQuery (instrumentHandle, instrumentDriverRevision,
                                 firmwareRevision, message);
}

/*===========================================================================*/
/* Function: Close                                                           */
/* Purpose:  This function closes the instrument.                            */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_close (ViSession instrumentHandle)
{
    ViStatus RSCMU_Status = VI_SUCCESS;
    ViChar  errorMessage[WR_BUFFER_SIZE];
    ViUInt16 secAddr;

   if ((RSCMU_Status = viGetAttribute (instrumentHandle, VI_ATTR_GPIB_SECONDARY_ADDR, &secAddr)) < 0)
      return RSCMU_Status;

    if (secAddr == 0) {
      if ((RSCMU_Status = RSCMU_Close_Base (instrumentHandle, errorMessage)) < 0)
         return RSCMU_Status;
    } else {
      struct RSCMU_RF_NSig_Globals *p_globalvar;

      if ((RSCMU_Status = viGetAttribute (instrumentHandle, VI_ATTR_USER_DATA, (ViAddr) &p_globalvar)) < 0)
         return RSCMU_Status;

      // Deallocate all system resources
      if ((RSCMU_Status = viClose (instrumentHandle)) < 0)
         return RSCMU_Status;

      // Deallocate the space pointed by p_globalvar
      if (p_globalvar)
         free (p_globalvar);
   }
    return RSCMU_Status;
}

/*===========================================================================*/
/*===========================================================================*/
/* Utility                                                                   */
/*===========================================================================*/
/*===========================================================================*/
/*===========================================================================*/
/* Function: Save/Recall File                                                */
/* Purpose:  This function saves/recalls the configuration of the current    */
/*           function group and test mode to a configuration file. A "?" in  */
/*           the specified file name will be replaced by current numbers     */
/*           that are automatically incremented, starting with zero.         */
/*           The auto-increment function overwrites an existing file with    */
/*           a "9" in its file name. For instrument settings that may be     */
/*           different in manual and remote control (e.g. the repetition     */
/*           mode for many measurements) the manual setting is saved.        */
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_SaveRecallCurrent (ViSession instrumentHandle,
                                    ViInt32 saveRecall,
                                    ViChar fileName[],
                                    ViInt32 storageDevice,
                                    ViChar errorMessage[])
{
    ViStatus    RSCMU_Status = VI_SUCCESS;
    ViChar      cmd[WR_BUFFER_SIZE];

    if (RSCMU_invalidViInt32Range (saveRecall, 0, 1))
        RETMSG(RSCMU_ERROR_PARAMETER2);
    if (!fileName)
        RETMSG(RSCMU_ERROR_PARAMETER3);
    if (RSCMU_invalidViInt32Range (storageDevice, 0, 1))
        RETMSG(RSCMU_ERROR_PARAMETER4);

    sprintf (cmd, "MMEMory:%s:CURRent \"%s\",%s" , RSCMUNB_STORAGE[saveRecall], fileName, RSCMUNB_STOR_DEV[storageDevice]);

    CHECKRETMSG(RSCMU_sendCmd(instrumentHandle, cmd));
    RETMSG(RSCMU_Status);
}

#endif

/*******************************************************************************
 * *****************************************************************************
 *    Driver Local Functions
 * *****************************************************************************
 *******************************************************************************/

ViStatus dummy (ViSession instrumentHandle,
                                           ViUInt32 channelDataSourceDTCH,
                                           ViChar errorMessage[])
{
    ViStatus  RSCMU_Status = VI_SUCCESS;
    RETMSG(RSCMU_Status);
}

/*===========================================================================*/
/* Function: Error Message
/* Purpose : This function translates the error return value from the
/*           instrument driver into a user-readable string.
/*===========================================================================*/
ViStatus _VI_FUNC RSCMUNB_errorMessage (ViSession  instrumentHandle,
                                        ViStatus   statusCode,
                                        ViChar _VI_FAR  message[])
{
  ViStatus RSCMU_Status = VI_SUCCESS;
  ViInt16  i;

  /*=======================================================================*/
  /* RSCMU_stringValPair is used in the RSCMU_errorMessage function
  /*=======================================================================*/
  typedef struct  RSCMU_stringValPair
  {
    ViStatus stringVal;
    ViString stringName;
  }  RSCMU_tStringValPair;

  #define  NB_EXPAND(id, value, text) {id, text},
           RSCMU_tStringValPair statusDescArray[] = {
                                  RSCMU_LIST_OF_SPECIFIC_ERROR_MESSAGES_NB
                                };
  #define  NUM_NB_ERRORS (sizeof(statusDescArray) / sizeof(RSCMU_tStringValPair))
  #undef   NB_EXPAND

  #define  EXPAND(id, value, text) {id, text},
           RSCMU_tStringValPair baseDescArray[] = {
                                                  RSCMU_LIST_OF_SPECIFIC_ERROR_MESSAGES
                                                  };
  #define NUM_BASE_ERRORS (sizeof(baseDescArray) / sizeof(RSCMU_tStringValPair))
  #undef  EXPAND

  strcpy (message, "No Error");

  RSCMU_Status = viStatusDesc(instrumentHandle, statusCode, message);
  if (RSCMU_Status == VI_WARN_UNKNOWN_STATUS)
  {
    // Search RSCMU base errors.
    for (i=0;(i < NUM_BASE_ERRORS) && baseDescArray[i].stringName;i++)
    {
      if (baseDescArray[i].stringVal == statusCode)
      {
        strcpy (message, baseDescArray[i].stringName);
        return (statusCode);
      }
    }

    // Search NB specific errors.
    for (i=0;(i < NUM_NB_ERRORS) && statusDescArray[i].stringName;i++)
    {
      if (statusDescArray[i].stringVal == statusCode)
      {
        strcpy (message, statusDescArray[i].stringName);
        return (statusCode);
      }
    }
    sprintf (message, "Unknown Error 0x%08lX", statusCode);
    return (VI_WARN_UNKNOWN_STATUS);
  }
  else if (RSCMU_Status == VI_ERROR_USER_BUF)
  {
    sprintf (message, "The user buffer is not valid for the required size.");
    return (VI_ERROR_USER_BUF);
  }

  // Here the call was successful:
  return (statusCode);
}

/* - - - */

ViStatus _VI_FUNC RSCMUNB_Get_NB_FW_Version(ViSession instrumentHandle, ViInt32* fgFwVersion, ViInt32* baseFwVersion)
{
    ViStatus   RSCMU_Status = VI_SUCCESS;
    ViChar     readBuffer[RD_BUFFER_SIZE] = "";

    CHECKRET(RSCMU_Status = RSCMU_getViString(instrumentHandle,
                                     "SYSTem:VERSion:SW:MMI?",
                                     readBuffer,
                                     RD_BUFFER_SIZE,
                                     RSCMU_ERROR_PARAMETER2));

// For Test only  strcpy(readBuffer,"\"3.10ab\",\"3.10:SP00\"");
    CHECKRET(RSCMU_FwVersionStringToIntegers(readBuffer,fgFwVersion,baseFwVersion));

    return RSCMU_Status;
}
